@@ -157,22 +157,14 @@ analyse_SAR.TL <- function(
   ## Integrity checks -------------------------------------------------------
 
   .validate_class(object, "RLum.Analysis")
-  if (missing("signal.integral.min")) {
-    .throw_error("No value set for 'signal.integral.min'")
-  }
-  if (missing("signal.integral.max")) {
-    .throw_error("No value set for 'signal.integral.max'")
-  }
-
+  .validate_class(signal.integral.min, c("numeric", "integer"))
+  .validate_class(signal.integral.max, c("numeric", "integer"))
   integral_input <- .validate_args(integral_input, c("channel", "temperature"))
 
   # Protocol Integrity Checks --------------------------------------------------
 
-  ##set allowed curve types
-  type.curves <- c("TL")
-
   ##Remove non TL-curves from object by selecting TL curves
-  object@records <- get_RLum(object, recordType = type.curves,
+  object@records <- get_RLum(object, recordType = "TL",
                              recursive = FALSE)
 
   ##ANALYSE SEQUENCE OBJECT STRUCTURE
@@ -199,6 +191,11 @@ analyse_SAR.TL <- function(
                  temp.sequence.structure)
   }
 
+  ## we must have a signal to analyse
+  if (!"SIGNAL" %in% sequence.structure) {
+    .throw_error("'sequence.structure' contains no 'SIGNAL' entry")
+  }
+
   ##check if the wanted curves are a multiple of the structure
   if(length(temp.sequence.structure[,"id"])%%length(sequence.structure)!=0)
     .throw_error("Input TL curves are not a multiple of the sequence structure")
@@ -360,7 +357,7 @@ analyse_SAR.TL <- function(
   col <- get("col", pos = .LuminescenceEnv)
 
   ##set layout matrix
-  layout(matrix(c(
+  graphics::layout(matrix(c(
     1, 1, 2, 2,
     1, 1, 2, 2,
     3, 3, 4, 4,
@@ -621,7 +618,7 @@ analyse_SAR.TL <- function(
   temp.GC <- try(fit_DoseResponseCurve(
     object = temp.sample,
     ...
-  ))
+  ), outFile = stdout()) # redirect error messages so they can be silenced
 
   ## fit_DoseResponseCurve() can fail in two ways:
   ## 1. either with a hard error, in which case there's nothing much we
@@ -643,7 +640,6 @@ analyse_SAR.TL <- function(
 
   }else{
     temp.GC <- data.frame(temp.GC, RC.Status="OK")
-
   }
 
   # Return Values -----------------------------------------------------------
@@ -658,5 +654,4 @@ analyse_SAR.TL <- function(
   )
 
   return(newRLumResults.analyse_SAR.TL)
-
 }
