@@ -1,16 +1,23 @@
 #' @title Analyse portable CW-OSL measurements
 #'
-#' @description The function analyses CW-OSL curve data produced by a SUERC portable OSL reader and
+#' @description
+#' The function analyses CW-OSL curve data produced by a SUERC portable OSL reader and
 #' produces a combined plot of OSL/IRSL signal intensities, OSL/IRSL depletion ratios
 #' and the IRSL/OSL ratio.
 #'
-#' @details This function only works with [RLum.Analysis-class] objects produced by [read_PSL2R].
+#' @details
+#' This function only works with [RLum.Analysis-class] objects produced by [read_PSL2R].
 #' It further assumes (or rather requires) an equal amount of OSL and IRSL curves that
-#' are pairwise combined for calculating the IRSL/OSL ratio. For calculating the depletion ratios
-#' the cumulative signal of the last n channels (same number of channels as specified
-#' by `signal.integral`) is divided by cumulative signal of the first n channels (`signal.integral`).
+#' are pairwise combined for calculating the IRSL/OSL ratio.
+#' For calculating the depletion ratios, the cumulative signal of the last *n*
+#' channels (same number of channels as specified by `signal.integral`) is
+#' divided by cumulative signal of the first *n* channels (`signal.integral`).
 #'
-#' **Note:  The function assumes the following sequence pattern: `DARK COUNT`, `IRSL`, `DARK COUNT`, `BSL`, `DARK COUNT`. If you have written a different sequence, the analysis function will (likely) not work!**.
+#' **Note:  The function assumes the following sequence pattern:
+#' `DARK COUNT`, `IRSL`, `DARK COUNT`, `BSL`, `DARK COUNT`.** Therefore, the
+#' total number of curves in the input object must be a multiple of 5, and
+#' there must be 3 `DARK_COUNT` records for each IRSL/BSL pair. If you have used
+#' a different sequence, the function will produce an error.
 #'
 #' **Signal processing**
 #' The function processes the signals as follows: `BSL` and `IRSL` signals are extracted using the
@@ -25,44 +32,55 @@
 #' strict and has to follow the scheme `_x:<number>|y:<number>`. Example:
 #' `sample_x:0.2|y:0.4`.
 #'
-#' * (2) Alternatively, you can provide a [list] or [matrix] with the sample coordinates.
+#' * (2) Alternatively, you can provide a [list] or [matrix] with the *(x, y)*
+#' coordinates of each sample in meters (m) using the `coord` argument:
 #' Example: `coord = list(c(0.2, 1), c(0.3,1.2))`
 #'
-#' Please note that the unit is meter (m) and the function expects always xy-coordinates.
-#' The latter one is useful for surface interpolations. If you have measured a profile where
-#' the x-coordinates to not measure, x-coordinates should be 0.
+#' If in your profile the x-coordinates were not measured, *x* should be set
+#' to 0. Note that, in such case, a surface plot cannot be produced.
 #'
-#' @param object [RLum.Analysis-class] (**required**): [RLum.Analysis-class] object produced by [read_PSL2R].
-#' The input can be a [list] of such objects, in such case each input is treated as a separate sample
-#' and the results are merged.
+#' @param object [RLum.Analysis-class] (**required**):
+#' object produced by [read_PSL2R]. The input can be a [list] of such objects,
+#' in which case each input is treated as a separate sample and the results
+#' are merged.
 #'
-#' @param signal.integral [numeric] (**required**): A vector of two values specifying the lower and upper channel used to calculate the OSL/IRSL signal. Can be provided in form of `c(1, 5)` or `1:5`.
+#' @param signal.integral [numeric] (**required**):
+#' A vector specifying the range of channels used to calculate the OSL/IRSL
+#' signal. It can be provided as a vector of length 2 such as `c(1, 5)`, or
+#' as a sequence such as `1:5`, in which case the lowest and highest values
+#' define the range.
 #'
 #' @param invert [logical] (*with default*): `TRUE` flip the plot the data in reverse order.
 #'
-#' @param normalise [logical] (*with default*): `TRUE` to normalise the OSL/IRSL signals
-#' to the *mean* of all corresponding data curves.
+#' @param normalise [logical] (*with default*):
+#' whether the OSL/IRSL signals should be normalised to the *mean* of all
+#' corresponding data curves.
 #'
-#' @param mode [character] (*with default*): defines the analysis mode, allowed
-#' are `"profile"` (the default) and `"surface"` for surface interpolation. If you select
-#' something else, nothing will be plotted (similar to `plot = FALSE`).
+#' @param mode [character] (*with default*):
+#' analysis mode, one of `"profile"` (the default) or `"surface"` for surface
+#' interpolation.
 #'
-#' @param coord [list] [matrix] (*optional*): a list or matrix of the same length as
-#' number of samples measured with coordinates for the sampling positions. Coordinates
-#' are expected to be provided in meter (unit: m).
-#' Expected are x and y coordinates, e.g.,
-#' `coord = list(samp1 = c(0.1, 0.2)`. If you have not measured x coordinates, please x should be 0.
+#' @param coord [list] [matrix] (*optional*): a list or a 2-column matrix
+#' with the *x* and *y* coordinates for the sampling positions in meters (m),
+#' of the same length as the number of samples measured. For example, the
+#' coordinates for one sample could be `coord = list(samp1 = c(0.1, 0.2)`.
+#' If the *x* coordinates were not measured, *x* should be set to 0.
+#' Note that, in such case, a surface plot cannot be produced.
 #'
-#' @param plot [logical] (*with default*): enable/disable plot output
+#' @param plot [logical] (*with default*): enable/disable the plot output.
 #'
 #' @param ... other parameters to be passed to modify the plot output.
-#' Supported are `run` to provide the run name ,
-#' if the input is a `list`, this is set automatically. Further plot parameters are
-#' `surface_values` ([character] with value to plot), `legend` (`TRUE`/`FALSE`), `col_ramp` (for
-#' surface mode), `contour` (contour lines `TRUE`/`FALSE` in surface mode), `grid` (`TRUE`/`FALSE`), `col`, `pch` (for profile mode), `xlim` (a name [list] for profile mode), `ylim`,
-#' `zlim` (surface mode only), `ylab`, `xlab`, `zlab` (here x-axis labelling), `main`, `bg_img` (for
-#' profile mode background image, usually a profile photo; should be a raster object),
-#' `bg_img_positions` (a vector with the four corner positions, cf. [graphics::rasterImage])
+#' Supported are `run` to provide the run name (if the input is a `list`, this
+#' is set automatically). Further plot parameters accepted are `main`,
+#' `col`, `xlim` (a named [list] for profile mode), `ylim`, `ylab`, `xlab`.
+#' Additional parameters for `mode = "profile"` are  `type`, `pch`, `grid`
+#' (`TRUE`/`FALSE`), `bg_img` (a raster object for the background image,
+#' usually a profile photo), `bg_img_positions` (a vector with the four corner
+#' positions, see [graphics::rasterImage]), `zlab` (here x-axis labelling).
+#' Additional parameters for `mode = "surface"` are `surface_value`
+#' ([character] with names of the surfaces to plot), `col_ramp`, `legend`
+#' (`TRUE`/`FALSE`), `contour` (`TRUE`/`FALSE`),` `contour_nlevels`,
+#' `contour_col`, ' zlim`.
 #'
 #' @return
 #' Returns an S4 [RLum.Results-class] object with the following elements:
@@ -74,10 +92,11 @@
 #'
 #' @seealso [RLum.Analysis-class], [RLum.Data.Curve-class], [read_PSL2R]
 #'
-#' @author Christoph Burow, University of Cologne (Germany), Sebastian Kreutzer,
-#' Institute of Geography, Ruprecht-Karl University of Heidelberg, Germany
+#' @author Christoph Burow, University of Cologne (Germany) \cr
+#' Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany) \cr
+#' Marco Colombo, Institute of Geography, Heidelberg University (Germany)
 #'
-#' @section Function version: 0.1.1
+#' @section Function version: 0.1.3
 #'
 #' @keywords datagen plot
 #'
@@ -119,31 +138,27 @@ analyse_portableOSL <- function(
   .set_function_name("analyse_portableOSL")
   on.exit(.unset_function_name(), add = TRUE)
 
-  ## TODO
-  ## - add tests for background image option
-  ## - clear docu
-
-# Self-call ---------------------------------------------------------------
+  ## Self-call --------------------------------------------------------------
   if (inherits(object, "list")) {
-      temp <- .warningCatcher(lapply(1:length(object), function(x) {
+      temp <- .warningCatcher(lapply(seq_along(object), function(x) {
         analyse_portableOSL(
           object = object[[x]],
           signal.integral = signal.integral,
           invert = invert,
           normalise = normalise,
+          mode = mode,
           plot = plot,
           run = paste0("RUN #", x))
       }))
 
       return(merge_RLum(temp))
-
   }
 
-# Start function ----------------------------------------------------------
-  ## INPUT VERIFICATION ----
+  ## Start function ---------------------------------------------------------
+
   ## only RLum.Analysis objects
-  if (!inherits(object, "RLum.Analysis"))
-    .throw_error("Only objects of class 'RLum.Analysis' are allowed")
+  .validate_class(object, "RLum.Analysis")
+  .validate_not_empty(object)
 
   ## only curve objects
   if (!all(sapply(object, class) == "RLum.Data.Curve"))
@@ -154,16 +169,36 @@ analyse_portableOSL <- function(
   if (!all(sapply(object, function(x) x@originator) == "read_PSL2R"))
     .throw_error("Only objects originating from 'read_PSL2R()' are allowed")
 
-  ## check sequence pattern
-  if(!all(names(object)[1:5] == c("USER", "IRSL", "USER", "OSL", "USER")))
-    .throw_error("Sequence pattern not supported, please read manual for details")
+  ## check length and start of the sequence pattern, we check it further below
+  if (length(object) %% 5 != 0 ||
+      !all(names(object)[1:5] == c("USER", "IRSL", "USER", "OSL", "USER")))
+    .throw_error("Sequence pattern not supported: see the manual for details")
 
   if (is.null(signal.integral)) {
     signal.integral <- c(1, 1)
     .throw_warning("No value for 'signal.integral' provided. Only the ",
                    "first data point of each curve was used")
+  } else {
+    signal.integral <- range(signal.integral)
   }
 
+  ## set the maximum signal_integral allowed: as this must be valid across all
+  ## records, we cap it to the minimum number of points
+  num.points <- min(sapply(get_RLum(object, recordType = c("OSL", "IRSL")),
+                             length))
+  if (max(signal.integral) > num.points || min(signal.integral) < 1) {
+    orig.signal.int <- signal.integral
+    signal.integral <- pmin(pmax(signal.integral, 1), num.points)
+    .throw_warning("'signal.integral' (",
+                   .collapse(orig.signal.int, quote = FALSE), ") ",
+                   "exceeds the number of data points, reset to (",
+                   .collapse(signal.integral, quote = FALSE), ")")
+  }
+
+  .validate_args(mode, c("profile", "surface"))
+  .validate_logical_scalar(invert)
+  .validate_logical_scalar(normalise)
+  .validate_logical_scalar(plot)
 
   ## set SAMPLE --------
   if("run" %in% names(list(...)))
@@ -189,11 +224,23 @@ analyse_portableOSL <- function(
     .posl_get_signal(x, signal.integral)
   }))
 
+  if (nrow(OSL) != nrow(IRSL)) {
+    .throw_error("Sequence pattern not supported: the number of OSL records ",
+                 "does not match the number of IRSL records")
+  }
+
   ### get DARK counts ----------
   ### we assume that USER contains the dark count measurements
   DARK_COUNT <- .unlist_RLum(list(get_RLum(object, recordType = "USER")))
-  DARK_COUNT <- lapply(seq(1,length(DARK_COUNT),3), function(x) DARK_COUNT[x:(x+2)])
 
+  ## we expect a sequence pattern with 3 DARK_COUNT records for each OSL/IRSL pair
+  num.dark.count <- length(DARK_COUNT)
+  if (num.dark.count %% 3 != 0) {
+    .throw_error("Sequence pattern not supported: expected ", nrow(OSL) * 3,
+                 " DARK_COUNT records, but found ", num.dark.count)
+  }
+
+  DARK_COUNT <- lapply(seq(1, num.dark.count, 3), function(x) DARK_COUNT[x:(x+2)])
   DARK_COUNT <- do.call(rbind, lapply(DARK_COUNT, function(x) {
     .posl_get_dark_count(x)
   }))
@@ -209,18 +256,24 @@ analyse_portableOSL <- function(
 
   ### extract  coordinates -------
   if(is.null(coord)) {
-    coord <- .extract_PSL_coord(object)
+    settings_sample <- unique(
+        vapply(object, function(x) x@info$settings$Sample, character(1)))
+    num.names <- length(settings_sample)
+    if (num.names != length(RATIO)) {
+      .throw_error("'object' references ", num.names, " sample names, but ",
+                   "only ", length(RATIO), " IRSL/OSL pairs found")
+    }
+    coord <- .extract_PSL_coord(settings_sample)
 
   } else {
-    if(!inherits(coord, "matrix") && !inherits(coord, "list"))
-      .throw_error("'coord' must be a matrix or a list")
-
+    .validate_class(coord, c("matrix", "list"))
     if(inherits(coord, "list"))
       coord <- do.call(rbind, coord)
 
     ## check length
     if(nrow(coord) != length(OSL$sum_signal))
-      .throw_error("Number of coordinates differ from the number of samples")
+      .throw_error("The number of coordinates in 'coord' should match the ",
+                   "number of samples (", length(OSL$sum_signal), ")")
   }
 
   ### GENERATE SUMMARY data.frame -----
@@ -241,39 +294,43 @@ analyse_portableOSL <- function(
   )
 
     ## if coordinates exist, sort by depth
-    if(!any(is.na(coord[,2])))
+    if (!anyNA(coord[, 2]))
       summary <- summary[order(coord[,2]),]
 
    ### INVERT ----------
    if(invert)
      summary <- summary[nrow(summary):1,]
 
+  if (mode == "surface" && plot && all(range(summary$COORD_X) == c(0, 0))) {
+    plot <- FALSE
+    message("[analyse_portableOSL()] Surface plot is not available when ",
+            "all x-coordinates are 0, plot reset to FALSE")
+  }
+
   # PLOTTING -------------------------------------------------------------------
   ## generate list of plot matrices
   ## this done to have consistent settings for all plot types
-  parm <-  c("BSL", "BSL_error", "IRSL", "IRSL_error",
-             "BSL_depletion", "IRSL_depletion", "IRSL_BSL_RATIO", "DARK", "DARK_error")
+  parm <- grep("^(BSL|IRSL|DARK)", colnames(summary), value = TRUE)
   m_list <- lapply(parm, function(x){
      cbind(x = summary[["COORD_X"]], y = summary[["COORD_Y"]], value = summary[[x]])
-
   })
 
     ## correct names of the list
     names(m_list) <- parm
 
+  if (plot) {
+
     ## add a few attributes to be used later
     attr(m_list, "xlim") <- lapply(m_list, function(x) range(x[,1]))
     attr(m_list, "ylim") <- if(invert) rev(range(m_list[[1]][,2])) else range(m_list[[1]][,2])
     attr(m_list, "zlim") <- lapply(m_list, function(x) range(x[,3]))
 
     ## account for surface case
-    if (!is.null(mode) && mode == "surface") {
+    if (mode == "surface") {
       attr(m_list, "ylim") <- if (invert) rev(range(summary$COORD_Y)) else range(summary$COORD_Y)
       attr(m_list, "xlim") <- range(summary$COORD_X)
     }
 
-  if (!is.null(mode) && plot[1]) {
-   ## account for surface case
    ## preset plot settings
    ## plot settings -------
    plot_settings <- modifyList(
@@ -283,31 +340,37 @@ analyse_portableOSL <- function(
        bg_img_positions = NULL,
        surface_value = c("BSL", "IRSL", "IRSL_BSL_RATIO"),
        legend = TRUE,
+       type = "b",
+       cex = 1,
        col = c("blue", "red", "blue", "red", "black", "grey"),
        pch = rep(16, length(m_list)),
        xlim = attr(m_list, "xlim"),
        ylim = attr(m_list, "ylim"),
        zlim = if(mode == "surface") NA else attr(m_list, "zlim"),
-       ylab = if(!any(is.na(summary$COORD_Y))) "Depth [m]" else "Index",
+       ylab = if (!anyNA(summary$COORD_Y)) "Depth [m]" else "Index",
        xlab = "x [m]",
        grid = TRUE,
        contour = FALSE,
+       contour_nlevels = 10,
+       contour_col = "grey",
        zlab = c("BSL", "IRSL", "BSL depl.", "IRSL depl.", "IRSL/BSL", "mean DARK"),
        main = summary$RUN[1]
      ),
      val = list(...), keep.null = TRUE)
 
-   ## mode == "surface" ---------
-   if(mode[1] == "surface") {
+    par.default <- par(no.readonly = TRUE)
+    on.exit(par(par.default), add = TRUE)
+
+    ## mode == "surface" ---------
+    if (mode == "surface") {
      ### check for validity of surface value -------
      if(!all(plot_settings$surface_value %in% names(m_list)))
-       .throw_error("Unknown value to plot: Valid are: ",
-                    paste(names(m_list), collapse = ", "))
+       .throw_error("Unknown value to plot, valid values are: ",
+                    .collapse(names(m_list)))
 
      ## set par -------
      if(length(plot_settings$surface_value) > 1) {
-       par.default <- par(mfrow = c(2,2))
-       on.exit(par(par.default), add = TRUE)
+       par(mfrow = c(2, 2))
      }
 
      ## loop over surface values -------
@@ -316,12 +379,15 @@ analyse_portableOSL <- function(
        m <- m_list[[i]]
 
        ## respect xlim and ylim range
-       m <- m[m[,2] >= min(plot_settings$ylim) & m[,2] <= max(plot_settings$ylim), ]
-       m <- m[m[,1] >= min(plot_settings$xlim) & m[,1] <= max(plot_settings$xlim), ]
+       m <- m[m[, 2] >= min(plot_settings$ylim) &
+              m[, 2] <= max(plot_settings$ylim), , drop = FALSE]
+       m <- m[m[, 1] >= min(plot_settings$xlim) &
+              m[, 1] <= max(plot_settings$xlim), , drop = FALSE]
 
        ## respect z_values
        if(!all(is.na(plot_settings$zlim)))
-         m <- m[m[,3] >= min(plot_settings$zlim) & m[,3] <= max(plot_settings$zlim), ]
+         m <- m[m[, 3] >= min(plot_settings$zlim) &
+                m[, 3] <= max(plot_settings$zlim), , drop = FALSE]
 
        ## interpolate ------
        s <-
@@ -336,15 +402,12 @@ analyse_portableOSL <- function(
        ## show only warning
        if(inherits(s, "try-error"))
          .throw_warning("Surface interpolation failed: this happens when ",
-                        "all points are arranged in one line. ",
-                        "Nothing plotted!")
+                        "all points are arranged in one line or xlim/ylim/zlim ",
+                        "are too tight. Nothing plotted!")
 
        ## show error
        if(!inherits(s, "try-error")) {
-         par.default <- c(
-           if(exists("par.default")) par.default else NULL,
-           par(mar = c(4.5,4.5,4,2), xpd = FALSE))
-         on.exit(par(par.default), add = TRUE)
+         par(mar = c(4.5, 4.5, 4, 3), xpd = FALSE)
 
          ## open empty plot
          plot(
@@ -354,14 +417,15 @@ analyse_portableOSL <- function(
            xlim = plot_settings$xlim,
            xlab = plot_settings$xlab,
            ylab = plot_settings$ylab,
+           cex.lab = plot_settings$cex,
+           cex.axis = plot_settings$cex,
            main = plot_settings$main)
 
          ## add background image if available -------
          if (!is.null(plot_settings$bg_img)) {
            ## get corner positions
-           if(!is.null(plot_settings$bg_img_positions))
-             positions <- plot_settings$bg_img_positions[1:4]
-           else
+           positions <- plot_settings$bg_img_positions[1:4]
+           if (is.null(positions))
              positions <- par()$usr
 
            graphics::rasterImage(
@@ -382,18 +446,23 @@ analyse_portableOSL <- function(
 
          ## add contour
          if (plot_settings$contour)
-           graphics::contour(m, add = TRUE, col = "grey")
+           graphics::contour(
+             x = s$x,
+             y = s$y,
+             z = s$z,
+             add = TRUE,
+             nlevels = plot_settings$contour_nlevels,
+             col = plot_settings$contour_col)
 
          ## add points
-         points(m[,1:2], pch = 20)
+         points(m[, 1:2], pch = 20, cex = plot_settings$cex)
 
          ## add what is shown in the plot
-         mtext(side = 3, text = i, cex = 0.7)
+         mtext(side = 3, text = i, cex = plot_settings$cex * 0.7)
 
          ## add legend
          if(plot_settings$legend) {
-           par.default <- c(par.default, par(xpd = TRUE))
-           on.exit(par(par.default), add = TRUE)
+           par(xpd = NA)
 
            col_grad <- plot_settings$col_ramp[
              seq(1, length(plot_settings$col_ramp), length.out = 14)]
@@ -402,8 +471,8 @@ analyse_portableOSL <- function(
 
            for(s in 1:(length(slices) - 1)){
              graphics::rect(
-               xleft = par()$usr[2] * 1.01,
-               xright = par()$usr[2] * 1.03,
+               xleft = par()$usr[2] * 1,
+               xright = par()$usr[2] * 1.02,
                ybottom = slices[s],
                ytop =  slices[s + 1],
                col = col_grad[s],
@@ -412,55 +481,53 @@ analyse_portableOSL <- function(
 
            ## add legend text
            text(
-             x = par()$usr[2] * 1.04,
+             x = par()$usr[2] * 1.03,
              y = par()$usr[4],
              labels = if(is.null(plot_settings$zlim_image)) {
                format(max(m[,3]), digits = 1, scientific = TRUE)
              } else {
                format(plot_settings$zlim_image[2], digits = 1, scientific = TRUE)
              },
-             cex = 0.6,
+             cex = plot_settings$cex * 0.6,
              srt = 270,
              pos = 3)
 
            text(
-             x = par()$usr[2] * 1.04,
+             x = par()$usr[2] * 1.03,
              y = par()$usr[3],
              labels = if(is.null(plot_settings$zlim_image)) {
                format(min(m[,3]), digits = 1, scientific = TRUE)
              } else {
                format(plot_settings$zlim_image[1], digits = 1, scientific = TRUE)
              },
-             cex = 0.6,
+             cex = plot_settings$cex * 0.6,
              pos = 3,
              srt = 270)
 
            ## add legend labelling (central)
            text(
-             x = par()$usr[2] * 1.05,
+             x = par()$usr[2] * 1.035,
              y = (par()$usr[4] - par()$usr[3])/2 + par()$usr[3],
              labels = "Intensity [a.u.]",
-             cex = 0.7,
+             cex = plot_settings$cex * 0.7,
              pos = 3,
              srt = 270)
          }
        }
      }# end for loop
-   }
+    }
 
-   ## mode == "profile" ---------
-   if (!is.null(mode[1]) && mode == "profile") {
-    par.old.full <- par(no.readonly = TRUE)
-    on.exit(par(par.old.full), add = TRUE)
+    ## mode == "profile" ---------
+    if (mode == "profile") {
 
     # default: par(mar = c(5, 4, 4, 2) + 0.1) // bottom, left, top, right
     par(mfrow = c(1, 7))
-
     par(mar = c(5, 4, 4, 1) + 0.1)
 
-    frame()
+    graphics::frame()
 
-    mtext(side= 3, plot_settings$main, cex = 0.7, line = 2)
+      mtext(side = 3, plot_settings$main, line = 2,
+            cex = plot_settings$cex * 0.7)
 
     par(mar = c(5, 0, 4, 1) + 0.1)
 
@@ -471,160 +538,69 @@ analyse_portableOSL <- function(
       plot_settings$zlim <- attr(m_list, "zlim")
     }
 
-    #### BSL -------
-    plot(
-      NA,
-      NA,
-      ylim = plot_settings$ylim,
-      xlim = plot_settings$zlim[["BSL"]],
-      xlab = plot_settings$zlab[1],
-      bty = "n",
-      yaxt = "n"
-    )
-      if(plot_settings$grid) grid()
-      lines(
-        x = m_list[["BSL"]][,"value"],
-        y = m_list[["BSL"]][,"y"],
-        type = "b",
-        pch = plot_settings$pch[1],
-        col = plot_settings$col[1]
-      )
+      ## plot the profile for each measure
+      profile.parts <- c("BSL", "IRSL", "BSL_depletion", "IRSL_depletion",
+                         "IRSL_BSL_RATIO", "DARK")
+      for (prof in profile.parts) {
+        idx <- match(prof, profile.parts)
 
-      ## add error bars
-      segments(
-        x0 = m_list[["BSL"]][,"value"] - m_list[["BSL_error"]][,"value"],
-        x1 = m_list[["BSL"]][,"value"] + m_list[["BSL_error"]][,"value"],
-        y0 = m_list[["BSL"]][,"y"],
-        y1 = m_list[["BSL"]][,"y"],
-        col = plot_settings$col[1])
+        xlim <- plot_settings$zlim[[prof]]
+        if (prof == "DARK") {
+          xlim <- range(c(
+              plot_settings$zlim[["DARK"]] - plot_settings$zlim[["DARK_error"]],
+              plot_settings$zlim[["DARK"]] + plot_settings$zlim[["DARK_error"]]))
+        }
+        plot(
+            NA,
+            NA,
+            ylim = plot_settings$ylim,
+            xlim = xlim,
+            xlab = plot_settings$zlab[idx],
+            ylab = "",
+            cex.lab = plot_settings$cex,
+            cex.axis = plot_settings$cex,
+            bty = "n",
+            yaxt = "n"
+        )
+        if (plot_settings$grid)
+          graphics::grid()
 
-        axis(2, line = 3, at = m_list[["BSL"]][,"y"], labels = m_list[["BSL"]][,"y"])
-        axis(3)
+        x.val <- m_list[[prof]][, "value"]
+        y.val <- m_list[[prof]][, "y"]
+        lines(
+            x = x.val,
+            y = y.val,
+            type = plot_settings$type,
+            cex = plot_settings$cex,
+            lty = ifelse(prof %in% c("BSL_depletion", "IRSL_depletion"), 2, 1),
+            pch = plot_settings$pch[idx],
+            col = plot_settings$col[idx]
+        )
 
-    ## add general y-axis label
-    mtext(plot_settings$ylab[1], side = 2, line = 6)
+        ## add error bars
+        if (prof %in% c("BSL", "IRSL", "DARK")) {
+          x.err <- m_list[[paste0(prof, "_error")]][, "value"]
+          segments(
+              x0 = x.val - x.err,
+              x1 = x.val + x.err,
+              y0 = y.val,
+              y1 = y.val,
+              col = plot_settings$col[idx])
+        }
 
-    ### IRSL --------------
-    plot(
-        NA, NA,
-        ylim = plot_settings$ylim,
-        xlim = plot_settings$zlim[["IRSL"]],
-        xlab = plot_settings$zlab[2],
-        bty = "n",
-        yaxt = "n"
-    )
-      if(plot_settings$grid) grid()
+        axis(3, cex.axis = plot_settings$cex)
 
-      lines(
-        x = m_list[["IRSL"]][,"value"],
-        y = m_list[["IRSL"]][,"y"],
-        type = "b",
-        pch = plot_settings$pch[2],
-        col = plot_settings$col[2])
+        ## add general axis labels
+        if (idx == 1) {
+          labs <- pretty(y.val, n = 10)
+          axis(2, line = 3, at = labs, labels = labs,
+               cex.axis = plot_settings$cex)
+          mtext(plot_settings$ylab[1], side = 2, line = 6,
+                cex = plot_settings$cex * 0.8)
+        }
+      }
 
-      ## add error bars
-      segments(
-        x0 = m_list[["IRSL"]][,"value"] - m_list[["IRSL_error"]][,"value"],
-        x1 = m_list[["IRSL"]][,"value"] + m_list[["IRSL_error"]][,"value"],
-        y0 = m_list[["IRSL"]][,"y"],
-        y1 = m_list[["IRSL"]][,"y"],
-        col = plot_settings$col[2])
-
-      axis(3)
-
-    ### OSL DEPLETATION -------
-    plot(
-      NA, NA,
-      ylim = plot_settings$ylim,
-      xlim = plot_settings$zlim[["BSL_depletion"]],
-      xlab = plot_settings$zlab[3],
-      bty = "n",
-      yaxt = "n"
-    )
-
-      if(plot_settings$grid) grid()
-      lines(
-        x = m_list[["BSL_depletion"]][,"value"],
-        y = m_list[["BSL_depletion"]][,"y"],
-        type = "b",
-        lty = 2,
-        pch = plot_settings$pch[3],
-        col = plot_settings$col[3]
-      )
-
-      axis(3)
-
-    ### IRSL DEPLETION ---------------
-    plot(
-      NA, NA,
-      ylim = plot_settings$ylim,
-      xlim = plot_settings$zlim[["IRSL_depletion"]],
-      xlab = plot_settings$zlab[4],
-      bty = "n",
-      yaxt = "n"
-    )
-
-      if(plot_settings$grid) grid()
-
-      lines(
-        x = m_list[["IRSL_depletion"]][,"value"],
-        y = m_list[["IRSL_depletion"]][,"y"],
-        type = "b",
-        lty = 2,
-        pch = plot_settings$pch[4],
-        col = plot_settings$col[4])
-
-      axis(3)
-
-    ### RATIO -----------------------------
-    plot(
-      NA, NA,
-      ylim = plot_settings$ylim,
-      xlim = plot_settings$zlim[["IRSL_BSL_RATIO"]],
-      xlab = plot_settings$zlab[5],
-      ylab = "",
-      bty = "n",
-      yaxt = "n"
-    )
-
-      if(plot_settings$grid) grid()
-
-      lines(
-        x = m_list[["IRSL_BSL_RATIO"]][,"value"],
-        y = m_list[["IRSL_BSL_RATIO"]][,"y"],
-        type = "b",
-        pch = plot_settings$pch[5],
-        col = plot_settings$col[5])
-
-      axis(3)
-
-    ### DARK -----------------------------
-    plot(
-      x = m_list[["DARK"]][,"value"],
-      y = m_list[["DARK_error"]][,"y"],
-      type = "b",
-      pch = plot_settings$pch,
-      col = plot_settings$col[6],
-      ylim = plot_settings$ylim,
-      xlim = range(c(
-        plot_settings$zlim[["DARK"]] - plot_settings$zlim[["DARK_error"]],
-        plot_settings$zlim[["DARK"]] + plot_settings$zlim[["DARK_error"]])),
-      xlab = plot_settings$zlab[6],
-      ylab = "",
-      bty = "n",
-      yaxt = "n"
-    )
-
-      ## add error bars
-      segments(
-        x0 = m_list[["DARK"]][,"value"] - m_list[["DARK_error"]][,"value"],
-        x1 = m_list[["DARK"]][,"value"] + m_list[["DARK_error"]][,"value"],
-        y0 = m_list[["DARK"]][,"y"],
-        y1 = m_list[["DARK"]][,"y"],
-        col = plot_settings$col[6])
-
-        axis(3)
-   } ## end mode == "profile"
+    } ## end mode == "profile"
   }
 
   ## RETURN VALUE ----
@@ -641,23 +617,13 @@ analyse_portableOSL <- function(
     info = list(call = call))
 
   return(newRLumResults)
-
 }
 
 # HELPER FUNCTIONS ----------
 ## This extracts the relevant curve data information of the RLum.Data.Curve
 ## objects
-.posl_get_signal <- function(x, signal.integral) {
+.posl_get_signal <- function(x, sigint) {
     raw_signal <- get_RLum(x)[,2]
-    sigint <- range(signal.integral)
-    if (sigint[2] > length(raw_signal)) {
-      sigint[2] <- length(raw_signal)
-      .throw_warning("'signal.integral' (",
-                     paste(range(signal.integral), collapse = ", "), ") ",
-                     "exceeded the number of available data points (n = ",
-                     length(raw_signal),") and has been automatically ",
-                     "reduced to the maximum number.")
-    }
     sum_signal <- sum(raw_signal[sigint[1]:sigint[2]])
     sum_signal_err <- sqrt(sum(x@info$raw_data$counts_per_cycle_error[sigint[1]:sigint[2]]^2))
     sum_signal_depletion <- sum(raw_signal[(length(raw_signal)-length(sigint[1]:sigint[2])):length(raw_signal)]) / sum_signal
@@ -670,7 +636,6 @@ analyse_portableOSL <- function(
   counts <- unlist(lapply(x, function(x) as.matrix(x)[,2]))
 
   return(data.frame(mean_dark_count = mean(counts), sd_dark_count = sd(counts)))
-
 }
 
 ## This function normalises the data curve by the mean signal
@@ -682,25 +647,21 @@ analyse_portableOSL <- function(
   return(x)
 }
 
-## This function extracts the coordinates from the file name
-##
-.extract_PSL_coord <- function(object){
-  ## get settings
-  settings_sample <- vapply(object, function(x) x@info$settings$Sample, character(1)) |>
-    unique()
-
+## This function extracts the coordinates from the sample names
+.extract_PSL_coord <- function(settings_sample) {
   ## set character vector
   tmp_coord <- character(length(settings_sample))
 
   ## search for pattern match ... why?
   ## because otherwise the dataset becomes inconsistent
+  pattern <- "_x:[0-9].*\\|y:[0-9].*"
   pattern_match <- grepl(
-    pattern = "\\_x\\:[0-9].+\\|y\\:[0-9].+",
+    pattern = pattern,
     x = settings_sample, perl = TRUE)
 
   ## extract coordinates
   tmp_coord[pattern_match] <- regexpr(
-    pattern = "\\_x\\:[0-9].+\\|y\\:[0-9].+",
+    pattern = pattern,
     text = settings_sample[pattern_match ], perl = TRUE) |>
     regmatches(x = settings_sample[pattern_match], m = _)
 
@@ -717,8 +678,8 @@ analyse_portableOSL <- function(
     numeric(2)) |> t()
 
   ## if NA, assign index
-  if(any(is.na(coord[,1]))) coord[,1] <- 0
-  if(any(is.na(coord[,2]))) coord[,2] <- 1:nrow(coord)
+  if (anyNA(coord[, 1])) coord[, 1] <- 0
+  if (anyNA(coord[, 2])) coord[, 2] <- 1:nrow(coord)
 
   return(coord)
 }
