@@ -1,9 +1,10 @@
 #' Apply the central age model (CAM) after Galbraith et al. (1999) to a given
 #' De distribution
 #'
+#' @description
 #' This function calculates the central dose and dispersion of the De
 #' distribution, their standard errors and the profile log likelihood function
-#' for sigma.
+#' for `sigma`.
 #'
 #' This function uses the equations of Galbraith & Roberts (2012). The
 #' parameters `delta` and `sigma` are estimated by numerically solving
@@ -16,23 +17,23 @@
 #' Galbraith & Roberts (2012, 15)
 #'
 #' @param data [RLum.Results-class] or [data.frame] (**required**):
-#' for [data.frame]: two columns with De `(data[,1])` and De error `(data[,2])`
+#' for [data.frame]: two columns with De `(data[,1])` and De error `(data[,2])`.
+#' Records containing missing values will be removed.
 #'
 #' @param sigmab [numeric] (*with default*):
 #' additional spread in De values.
 #' This value represents the expected overdispersion in the data should the sample be
 #' well-bleached (Cunningham & Walling 2012, p. 100).
 #' **NOTE**: For the logged model (`log = TRUE`) this value must be
-#' a fraction, e.g. 0.2 (= 20 \%). If the un-logged model is used (`log = FALSE`),
+#' a fraction, e.g. 0.2 (= 20 %). If the un-logged model is used (`log = FALSE`),
 #' sigmab must be provided in the same absolute units of the De values (seconds or Gray).
 #'
 #' @param log [logical] (*with default*):
-#' fit the (un-)logged central age model to De data
-#'
-#' @param na.rm [logical] (*with default*): strip `NA` values before the computation proceeds
+#' fit the (un-)logged central age model to De data. Log transformation is
+#' allowed only if the De values are positive.
 #'
 #' @param plot [logical] (*with default*):
-#' plot output
+#' enable/disable the plot output.
 #'
 #' @param ... further arguments (`trace`, `verbose`).
 #'
@@ -45,9 +46,9 @@
 #' \item{.$call}{[call] the function call}
 #' \item{.$profile}{[data.frame] the log likelihood profile for sigma}
 #'
-#' The output should be accessed using the function [get_RLum]
+#' The output should be accessed using the function [get_RLum].
 #'
-#' @section Function version: 1.4.1
+#' @section Function version: 1.5
 #'
 #' @author
 #' Christoph Burow, University of Cologne (Germany) \cr
@@ -98,41 +99,55 @@
 #'
 #' @md
 #' @export
-calc_CentralDose <- function(data, sigmab, log = TRUE, na.rm = FALSE, plot = TRUE, ...) {
-  ## ============================================================================##
-  ## CONSISTENCY CHECK OF INPUT DATA
-  ## ============================================================================##
-  if (!missing(data)) {
-    if (!is(data, "data.frame") & !is(data, "RLum.Results")) {
-      stop("[calc_CentralDose()] 'data' has to be of type 'data.frame' or 'RLum.Results'!", call. = FALSE)
-    } else {
-      if (is(data, "RLum.Results")) {
-        data <- get_RLum(data, "data")
-      }
-    }
+calc_CentralDose <- function(
+  data,
+  sigmab = 0,
+  log = TRUE,
+  plot = TRUE,
+  ...
+) {
+  .set_function_name("calc_CentralDose")
+  on.exit(.unset_function_name(), add = TRUE)
+
+  ## Integrity checks -------------------------------------------------------
+
+  .validate_class(data, c("data.frame", "RLum.Results"))
+  if (inherits(data, "RLum.Results")) {
+    data <- get_RLum(data, "data")
   }
 
   ##remove NA values
-  if(na.rm == TRUE && any(is.na(data))){
-    warning("[calc_CentralDose()] ", length(which(is.na(data))), " NA value(s) removed from dataset!", call. = FALSE)
+  if (anyNA(data)) {
+    message("[calc_CentralDose()] ", length(which(is.na(data))),
+            " NA values removed from dataset")
     data <- na.exclude(data)
   }
 
-  ##make sure we consider onlyt take the first two columns
+  ## check that we have enough data available
   if(ncol(data) < 2 || nrow(data) < 2)
-    stop("[calc_CentralDose()] 'data' should have at least two columns and two rows!", call. = FALSE)
+    .throw_error("'data' should have at least two columns and two rows")
 
   ##extract only the first two columns and set column names
   data <- data[,1:2]
   colnames(data) <- c("ED", "ED_Error")
 
-  if (!missing(sigmab)) {
-    if (sigmab < 0 | sigmab > 1 & log)
-      stop("[calc_CentralDose()] sigmab needs to be given as a fraction between 0 and 1 (e.g., 0.2)!", call. = FALSE)
-
+  ## don't allow log transformation if there are non-positive values
+  if (any(data[, 1] <= 0) && log == TRUE) {
+    log <- FALSE
+    .throw_warning("'data' contains non-positive De values, 'log' set to FALSE")
   }
 
+  ## don't allow negative errors, silently make them positive
+  if (any(data[, 2] < 0)) {
+    data[, 2] <- abs(data[, 2])
+  }
 
+  if (sigmab < 0) {
+    .throw_error("'sigmab' should be a non-negative value")
+  }
+  if (log && sigmab > 1) {
+    .throw_error("'sigmab' should be a value between 0 and 1 if log = TRUE")
+  }
 
 
   ## ============================================================================##
@@ -144,15 +159,17 @@ calc_CentralDose <- function(data, sigmab, log = TRUE, na.rm = FALSE, plot = TRU
 
   options <- modifyList(options, list(...))
 
+  ## deprecated argument
+  if ("na.rm" %in% names(list(...))) {
+    .throw_warning("'na.rm' is deprecated, missing values are always removed ",
+                   "by default")
+  }
+
 
   ## ============================================================================##
   ## CALCULATIONS
   ## ============================================================================##
 
-  # set default value of sigmab
-  if (missing(sigmab))
-    sigmab <- 0
-
   # calculate yu = log(ED) and su = se(logED)
   if (log) {
     yu <- log(data$ED)
@@ -162,14 +179,13 @@ calc_CentralDose <- function(data, sigmab, log = TRUE, na.rm = FALSE, plot = TRU
     su <- sqrt((data$ED_Error)^2 + sigmab^2)
   }
 
-
   # What does the code do?
   # >> email conversation with Rex Galbraith 2019-06-29
   # >> "fixed point iteration" method to estimate sigma
   # >> starting with a fixed value
   # >> once sqrt(sum((wu^2) * (yu - delta)^2 / sum(wu))) gets equal to 1
   # >> the iteration is complete
-  # >> if everything has converged agains those fixed values
+  # >> if everything has converged against those fixed values
   # >> this is the maximum likelihood estimate for
   # >> sigma and delta
 
@@ -188,39 +204,41 @@ calc_CentralDose <- function(data, sigmab, log = TRUE, na.rm = FALSE, plot = TRU
     # print iterations
     if (options$trace)
       print(round(c(delta, sigma), 4))
-  }
 
-  # save parameters for terminal output
-  out.delta <- ifelse(log, exp(delta), delta)
-  out.sigma <- ifelse(log, sigma * 100, sigma / out.delta * 100)
+    ## don't let sigma become zero
+    if (sigma < 1e-16)
+      break()
+  }
 
   # log likelihood
   llik <- 0.5 * sum(log(wu)) - 0.5 * sum(wu * (yu - delta)^2)
 
-  # save parameter for terminal output
-  out.llik <- round(llik, 4)
-  Lmax <- llik
-
   # standard errors
   sedelta <- 1 / sqrt(sum(wu))
   sesigma <- 1 / sqrt(2 * sigma^2 * sum(wu^2))
 
   # save parameters for terminal output
   if (log) {
+    out.delta <- exp(delta)
+    out.sigma <- sigma * 100
     out.sedelta <- sedelta * 100
     out.sesigma <- sesigma
   } else {
+    out.delta <- delta
+    out.sigma <- sigma / out.delta * 100
     out.sedelta <- sedelta / out.delta * 100
     out.sesigma <- sqrt((sedelta / delta)^2 +
                           (sesigma / out.delta * 100 / out.sigma)^2) * out.sigma / 100
-
   }
+  if (is.infinite(out.sesigma))
+    out.sesigma <- NA
+  out.llik <- round(llik, 4)
 
   # profile log likelihood
-  sigmax <- sigma
+  Lmax <- llik
   llik <- 0
-  sig0 <- max(0, sigmax - 8 * sesigma)
-  sig1 <- sigmax + 9.5 * sesigma
+  sig0 <- max(0, sigma - 8 * sesigma)
+  sig1 <- sigma + 9.5 * sesigma
   sig <- try(seq(sig0, sig1, sig1 / 1000), silent = TRUE)
 
   if (!inherits(sig, "try-error")) {
@@ -241,35 +259,32 @@ calc_CentralDose <- function(data, sigmab, log = TRUE, na.rm = FALSE, plot = TRU
 
   if (options$verbose) {
     cat("\n [calc_CentralDose]")
-    cat(paste("\n\n----------- meta data ----------------"))
-    cat(paste("\n n:                      ", n))
-    cat(paste("\n log:                    ", log))
-    cat(paste("\n----------- dose estimate ------------"))
-    cat(paste("\n abs. central dose:      ", format(out.delta, digits = 2, nsmall = 2)))
-    cat(paste("\n abs. SE:                ", format(out.delta * out.sedelta/100,
-                                                   digits = 2, nsmall = 2)))
-    cat(paste("\n rel. SE [%]:            ", format(out.sedelta, digits = 2, nsmall = 2)))
-    cat(paste("\n----------- overdispersion -----------"))
-    cat(paste("\n abs. OD:                ", format(ifelse(log, sigma * out.delta, sigma), digits = 2, nsmall = 2)))
-    cat(paste("\n abs. SE:                ", format(ifelse(log, sesigma * out.delta, sesigma), digits = 2, nsmall = 2)))
-    cat(paste("\n OD [%]:                 ", format(out.sigma, digits = 2, nsmall = 2)))
-    cat(paste("\n SE [%]:                 ", if (!inherits(sig, "try-error")) {
+    cat("\n\n----------- meta data ----------------")
+    cat("\n n:                      ", n)
+    cat("\n log:                    ", log)
+    cat("\n----------- dose estimate ------------")
+    cat("\n abs. central dose:      ", format(out.delta, digits = 2, nsmall = 2))
+    cat("\n abs. SE:                ", format(out.delta * out.sedelta / 100,
+                                              digits = 2, nsmall = 2))
+    cat("\n rel. SE [%]:            ", format(out.sedelta, digits = 2, nsmall = 2))
+    cat("\n----------- overdispersion -----------")
+    cat("\n abs. OD:                ", format(ifelse(log, sigma * out.delta, sigma),
+                                              digits = 2, nsmall = 2))
+    cat("\n abs. SE:                ", format(ifelse(log, sesigma * out.delta, sesigma),
+                                              digits = 2, nsmall = 2))
+    cat("\n OD [%]:                 ", format(out.sigma, digits = 2, nsmall = 2))
+    cat("\n SE [%]:                 ", if (!is.na(out.sesigma)) {
       format(out.sesigma * 100, digits = 2, nsmall = 2)
     } else {
       "-"
-    }))
-    cat(paste("\n-------------------------------------\n\n"))
+    })
+    cat("\n-------------------------------------\n\n")
   }
 
   ## ============================================================================##
   ## RETURN VALUES
   ## ============================================================================##
 
-  if (inherits(sig, "try-error")) {
-    out.sigma <- 0
-    out.sesigma <- NA
-  }
-
   if(!log)  sig <- sig / delta
 
   summary <- data.frame(
