@@ -15,41 +15,43 @@
 #'**The input table layout**
 #'
 #'\tabular{lll}{
-#'COLUMN \tab DATATYPE \tab DESCRIPTION\cr
-#'Distance \tab `numeric` \tab distance from the surface of the cobble to the top of each rock slice in mm. The distance for each slice will be listed in this column\cr
-#'DistanceError \tab `numeric` \tab Error on the distance in mm\cr
-#'Thickness \tab `numeric` \tab Thickness of each slice in mm\cr
-#'TicknessError \tab `numeric` \tab uncertainty of the thickness in mm.\cr
-#'Mineral \tab `character` \tab `'FS'` for feldspar, `'Q'` for quartz, depending which mineral in the cobble is used for dating\cr
-#'Cobble_K \tab `numeric` \tab K nuclide content in % of the bulk cobble\cr
-#'Cobble_K_SE \tab `numeric` \tab error on K nuclide content in % of the bulk cobble\cr
-#'Cobble_Th \tab `numeric` \tab Th nuclide content in ppm of the bulk cobble\cr
-#'Cobble_Th_SE \tab `numeric` \tab error on Th nuclide content in ppm of the bulk cobble\cr
-#'Cobble_U \tab `numeric` \tab U nuclide content in ppm of the bulk cobble\cr
-#'CobbleU_SE \tab `numeric` \tab error on U nuclide content in ppm of the bulk cobble\cr
-#'GrainSize \tab `numeric` \tab average grain size in µm of the grains used for dating\cr
-#'Density \tab `numeric` \tab Density of the cobble. Default is 2.7 g cm^-3\cr
-#'CobbleDiameter \tab `numeric` \tab Diameter of the cobble in cm.\cr
-#'Sed_K \tab `numeric` \tab K nuclide content in % of the sediment matrix\cr
-#'Sed_K_SE \tab `numeric` \tab error on K nuclide content in % of the sediment matrix\cr
-#'Sed_Th \tab `numeric` \tab Th nuclide content in ppm of the sediment matrix\cr
-#'Sed_Th_SE \tab `numeric` \tab error on Th nuclide content in ppm of the sediment matrix\cr
-#'Sed_U \tab `numeric` \tab U nuclide content in ppm of the sediment matrix\cr
-#'Sed_U_SE \tab `numeric` \tab error on U nuclide content in ppm of the sediment matrix\cr
-#'GrainSize \tab `numeric` \tab average grain size of the sediment matrix\cr
-#'WaterContent \tab `numeric` \tab mean water content of the sediment matrix in %\cr
-#'WaterContent_SE \tab `numeric` \tab relative error on water content
+#'COLUMN \tab DATA TYPE \tab DESCRIPTION\cr
+#'`Distance` \tab `numeric` \tab distance from the surface of the cobble to the top of each rock slice in mm. The distance for each slice will be listed in this column\cr
+#'`DistanceError` \tab `numeric` \tab Error on the distance in mm\cr
+#'`Thickness` \tab `numeric` \tab Thickness of each slice in mm\cr
+#'`ThicknessError` \tab `numeric` \tab uncertainty of the thickness in mm.\cr
+#'`Mineral` \tab `character` \tab `'FS'` for feldspar, `'Q'` for quartz, depending which mineral in the cobble is used for dating\cr
+#'`Cobble_K` \tab `numeric` \tab K nuclide content in % of the bulk cobble\cr
+#'`Cobble_K_SE` \tab `numeric` \tab error on K nuclide content in % of the bulk cobble\cr
+#'`Cobble_Th` \tab `numeric` \tab Th nuclide content in ppm of the bulk cobble\cr
+#'`Cobble_Th_SE` \tab `numeric` \tab error on Th nuclide content in ppm of the bulk cobble\cr
+#'`Cobble_U` \tab `numeric` \tab U nuclide content in ppm of the bulk cobble\cr
+#'`Cobble_U_SE` \tab `numeric` \tab error on U nuclide content in ppm of the bulk cobble\cr
+#'`GrainSize` \tab `numeric` \tab average grain size in µm of the grains used for dating\cr
+#'`Density` \tab `numeric` \tab Density of the cobble. Default is 2.7 g cm^-3\cr
+#'`CobbleDiameter` \tab `numeric` \tab Diameter of the cobble in cm.\cr
+#'`Sed_K` \tab `numeric` \tab K nuclide content in % of the sediment matrix\cr
+#'`Sed_K_SE` \tab `numeric` \tab error on K nuclide content in % of the sediment matrix\cr
+#'`Sed_Th` \tab `numeric` \tab Th nuclide content in ppm of the sediment matrix\cr
+#'`Sed_Th_SE` \tab `numeric` \tab error on Th nuclide content in ppm of the sediment matrix\cr
+#'`Sed_U` \tab `numeric` \tab U nuclide content in ppm of the sediment matrix\cr
+#'`Sed_U_SE` \tab `numeric` \tab error on U nuclide content in ppm of the sediment matrix\cr
+#'`GrainSize_Sed` \tab `numeric` \tab average grain size of the sediment matrix\cr
+#'`Density_Sed` \tab `numeric` \tab average density of the sediment matrix\cr
+#'`WaterContent` \tab `numeric` \tab mean water content of the sediment matrix in %\cr
+#'`WaterContent_SE` \tab `numeric` \tab relative error on water content
 #'}
 #'
 #'**Water content**
 #'The water content provided by the user should be calculated according to:
 #'
-#'\deqn{(Wet_weight - Dry_weight) / Dry_weight * 100}
+#'\deqn{(Wet\_weight - Dry\_weight) / Dry\_weight * 100}
 #'
 #'@param input [data.frame] (**required**): A table containing all relevant information
 #'for each individual layer. For the table layout see details.
 #'
-#'@param conversion Which dose rate conversion factors to use. For accepted values see [BaseDataSet.ConversionFactors]
+#' @param conversion [character] (*with default*): dose rate conversion factors
+#' to use, see [BaseDataSet.ConversionFactors] for the accepted values.
 #'
 #'@references
 #'Riedesel, S., Autzen, M., 2020. Beta and gamma dose rate attenuation in rocks and sediment.
@@ -81,13 +83,39 @@
 #'@md
 #'@export
 calc_CobbleDoseRate <- function(input,conversion = "Guerinetal2011"){
+  .set_function_name("calc_CobbleDoseRate")
+  on.exit(.unset_function_name(), add = TRUE)
+
+  ## Integrity checks -------------------------------------------------------
+  .validate_class(input, "data.frame")
+  .validate_not_empty(input)
+
+  ## this lists only the columns that are referenced by name in the script
+  exp.cols <- c("Distance", "DistanceError", "Thickness", "ThicknessError",
+                "Density", "CobbleDiameter")
+  mis.cols <- setdiff(exp.cols, colnames(input))
+  if (length(mis.cols) > 0)
+    .throw_error("'input' doesn't contain the following columns: ",
+                 .collapse(mis.cols))
 
-  # Integrity tests ---------------------------------------------------------
   if ((max(input[,1])>input$CobbleDiameter[1]*10) ||
       ((max(input[,1]) + input[length(input[,1]),3]) > input$CobbleDiameter[1]*10))
-    stop("[calc_CobblDoseRate()] Slices outside of cobble. Please check your distances and make sure they are in mm and diameter is in cm!", call. = FALSE)
+    .throw_error("Slices outside of cobble: please ensure your distances ",
+                 "are in mm and diameter is in cm")
 
 
+  ## conversion factors: we do not use BaseDataSet.ConversionFactors directly
+  ## as it is in alphabetical level, but we want to have 'Guerinetal2011'
+  ## in first position, as that is our default value
+  BaseDataSet.ConversionFactors <- NULL
+  load(system.file("data", "BaseDataSet.ConversionFactors.rda",
+                   package = "Luminescence"))
+  valid_conversion_factors <- c("Guerinetal2011", "Cresswelletal2018",
+                                "AdamiecAitken1998", "Liritzisetal2013")
+  stopifnot(all(names(BaseDataSet.ConversionFactors) %in%
+                valid_conversion_factors))
+  conversion <- .validate_args(conversion, valid_conversion_factors)
+
   # Calculate Dose Rate -----------------------------------------------------
   SedDoseData <- matrix(data = NA, nrow = 1, ncol = 10)
   CobbleDoseData <- matrix(data = 0, nrow = 1, ncol = 10)
