@@ -159,7 +159,7 @@
 #' function at the time of writing.
 #'
 #'
-#' @section Function version: 0.5.2
+#' @section Function version: 0.5.3
 #'
 #' @author
 #' Christoph Burow, University of Cologne (Germany)
@@ -255,10 +255,15 @@ calc_CosmicDoseRate<- function(
   ## Integrity checks -------------------------------------------------------
 
   .validate_class(depth, "numeric")
+  .validate_not_empty(depth)
   .validate_class(density, "numeric")
+  .validate_not_empty(density)
   .validate_class(latitude, "numeric")
+  .validate_not_empty(latitude)
   .validate_class(longitude, "numeric")
+  .validate_not_empty(longitude)
   .validate_class(altitude, "numeric")
+  .validate_not_empty(altitude)
 
   if(any(depth < 0) || any(density < 0)) {
     .throw_error("No negative values allowed for 'depth' and 'density'")
@@ -276,9 +281,10 @@ calc_CosmicDoseRate<- function(
     }
   }
 
-  if(length(density) > length(depth)) {
-    .throw_error("If you provide more than one value for density, please ",
-                 "provide an equal number of values for depth")
+  if (length(depth) < length(density) ||
+      (length(depth) > length(density) && length(density) > 1)) {
+    .throw_error("The number of values for 'density' should either be 1 ",
+                 "or correspond to the number of values for 'depth'")
   }
 
   settings <- list(verbose = TRUE)
@@ -303,19 +309,10 @@ calc_CosmicDoseRate<- function(
 
   profile.mode<- FALSE
 
-  #calculate absorber (hgcm) of one depth and one absorber [single sample]
-  if(length(depth)==1) {
-    hgcm<- depth*density
-    if(half.depth == TRUE) {
-      hgcm<- hgcm/2
-    }
-  }
-
-  #calculate total absorber of n depths and n densities [single sample]
+  ## calculate total absorber of n depths and n densities [single sample]
+  ## the calculation is still valid if there is only one depth and one density
   if(length(depth)==length(density)){
-
     hgcm<- 0
-
     for(i in 1:length(depth)) {
       hgcm<- hgcm + depth[i]*density[i]
     }
@@ -480,24 +477,23 @@ calc_CosmicDoseRate<- function(
     ##============================================================================##
     if (settings$verbose) {
       cat("\n\n [calc_CosmicDoseRate]")
-      cat(paste("\n\n ---------------------------------------------------------"))
-      cat(paste("\n depth (m)              :", depth))
-      cat(paste("\n density (g cm^-3)      :", density))
-      cat(paste("\n latitude (N deg.)      :", latitude))
-      cat(paste("\n longitude (E deg.)     :", longitude))
-      cat(paste("\n altitude (m)           :", altitude))
-      cat(paste("\n ---------------------------------------------------------"))
-      cat(paste("\n total absorber (g cm^-2)       :", round(hgcm[i]*100,3)))
-      cat(paste("\n"))
-      cat(paste("\n cosmic dose rate (Gy ka^-1)    :", round(d0,4)))
-      cat(paste("\n  [@sea-level & 55 deg. N G.lat]"))
-      cat(paste("\n"))
-      cat(paste("\n geomagnetic latitude (deg.)    :", round(true.gml,1)))
-      cat(paste("\n"))
-      cat(paste("\n cosmic dose rate (Gy ka^-1)    :", round(dc,4),"+-",
-                round(dc.err,4)))
-      cat(paste("\n  [corrected]                 "))
-      cat(paste("\n ---------------------------------------------------------\n\n"))
+      cat("\n\n ---------------------------------------------------------")
+      cat("\n depth (m)              :", depth)
+      cat("\n density (g cm^-3)      :", density)
+      cat("\n latitude (N deg.)      :", latitude)
+      cat("\n longitude (E deg.)     :", longitude)
+      cat("\n altitude (m)           :", altitude)
+      cat("\n ---------------------------------------------------------")
+      cat("\n total absorber (g cm^-2)       :", round(hgcm[i] * 100, 3))
+      cat("\n")
+      cat("\n cosmic dose rate (Gy ka^-1)    :", round(d0, 4))
+      cat("\n  [@sea-level & 55 deg. N G.lat]")
+      cat("\n")
+      cat("\n geomagnetic latitude (deg.)    :", round(true.gml, 1))
+      cat("\n")
+      cat("\n cosmic dose rate (Gy ka^-1)    :", round(dc, 4), "+-", round(dc.err, 4))
+      cat("\n  [corrected]                 ")
+      cat("\n ---------------------------------------------------------\n\n")
     }
     ##============================================================================##
     ##RETURN VALUES
@@ -539,8 +535,7 @@ calc_CosmicDoseRate<- function(
     #terminal output
     if (settings$verbose) {
       cat("\n\n [calc_CosmicDoseRate]")
-      cat(paste("\n\n Calculating cosmic dose rate for",length(depth),
-                "samples. \n\n"))
+      cat("\n\n Calculating cosmic dose rate for", length(depth), "samples.\n\n")
       print(profile.results)
     }
 
