@@ -91,22 +91,26 @@
 #' whether another fading correction model might be more appropriate.
 #'
 #' @param age.faded [numeric] [vector] (**required**):
-#' uncorrected age with error in ka (see example)
+#' vector of length 2 containing the uncorrected age and the error in ka
+#' (see example).
 #'
 #' @param g_value [vector] or [RLum.Results-class] (**required**):
-#' g-value and error obtained from separate fading measurements (see example).
-#' It can be an [RLum.Results-class] object produced by
-#' [analyse_FadingMeasurement], in which case `tc` is set automatically
+#' either a vector of length 2 containing the g-value and error obtained from
+#' separate fading measurements (see example), or an [RLum.Results-class]
+#' object produced by [analyse_FadingMeasurement]. If the latter, the `tc`
+#' argument is set automatically.
 #'
 #' @param tc [numeric] (**required**):
 #' time in seconds between irradiation and the prompt measurement (cf. Huntley & Lamothe 2001).
-#' The argument is ignored when `g_value` is an [RLum.Results-class] object
+#' The argument is ignored when `g_value` is an [RLum.Results-class] object.
 #'
 #' @param tc.g_value [numeric] (*with default*):
-#' the time in seconds between irradiation and the prompt measurement used for estimating the g-value.
-#' If the g-value was normalised to, e.g., 2 days, this time in seconds (i.e., 172800) should be given here.
-#' If nothing is provided the time is set to tc, which is usual case for g-values obtained using the
-#' SAR method and \eqn{g}-values that had been not normalised to 2 days.
+#' time in seconds between irradiation and the prompt measurement used in the
+#' estimation of the g-value. If the g-value was normalised, the normalisation
+#' time (in seconds) should be given, e.g., for a g-value normalised to 2 days,
+#' the value 172800 should be used. If nothing is provided the time is set to
+#' `tc`, which is usual case for g-values obtained using the
+#' SAR method and \eqn{g}-values that have been not normalised to 2 days.
 #'
 #' @param n.MC [integer] (*with default*):
 #' number of Monte Carlo simulation runs for error estimation.
@@ -147,7 +151,7 @@
 #' @note Special thanks to Sébastien Huot for his support and clarification via e-mail.
 #'
 #'
-#' @section Function version: 0.4.3
+#' @section Function version: 0.4.4
 #'
 #' @author Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
 #'
@@ -209,20 +213,25 @@ calc_FadingCorr <- function(
 
   ## Integrity checks -------------------------------------------------------
 
-  .validate_class(age.faded, "numeric")
-  .validate_class(g_value, c("numeric", "RLum.Results"))
+  .validate_class(age.faded, c("numeric", "integer"))
+  .validate_length(age.faded, 2)
+  .validate_class(g_value, c("numeric", "integer", "RLum.Results"))
   if(inherits(g_value, "RLum.Results")){
     if(g_value@originator == "analyse_FadingMeasurement"){
       tc <- get_RLum(g_value)[["TC"]]
       g_value <- as.numeric(get_RLum(g_value)[,c("FIT", "SD")])
     }else{
-      .throw_message("Unknown originator for the provided 'g_value' object")
+      .throw_message("Unknown originator for the 'g_value' object provided")
       return(NULL)
     }
+  } else {
+    .validate_length(g_value, 2)
   }
 
   ## tc is validated only now, as it may be set in the previous block
-  .validate_class(tc, "numeric")
+  .validate_class(tc, c("numeric", "integer"))
+  .validate_positive_scalar(tc)
+  .validate_positive_scalar(tc.g_value)
   .validate_class(interval, "numeric")
   .validate_length(interval, 2)
   .validate_logical_scalar(txtProgressBar)
@@ -251,10 +260,9 @@ calc_FadingCorr <- function(
   ##calculate kappa (equation [5] in Huntley and Lamothe, 2001)
   kappa <- g_value / log(10) / 100
 
-  ##transform tc in ka years
-  ##duration of the year over a long term taken from http://wikipedia.org
-  tc <- tc[1] / 60 / 60 / 24 / 365.2425  / 1000
-  tc.g_value <- tc.g_value[1] / 60 / 60 / 24 / 365.2425  / 1000
+  ## transform tc from s to ka years
+  tc <- tc / (1000 * .const$year_s)
+  tc.g_value <- tc.g_value / (1000 * .const$year_s)
 
   ##calculate mean value
   temp <-
@@ -300,12 +308,12 @@ calc_FadingCorr <- function(
   # Start loop  ---------------------------------------------------------------------------------
 
   ##set object and preallocate memory
-  tempMC <- vector("numeric", length = 1e+07)
+  tempMC <- vector("numeric", length = if (n.MC == "auto") 1e+07 else n.MC)
   tempMC[] <- NA
   i <- 1
   j <- n.MC.i
 
-  while(length(unique(tempMC.sd.count))>1 | j > 1e+07){
+  while (length(unique(tempMC.sd.count)) > 1 || j > length(tempMC)) {
 
     ##set previous
     if(!is.na(tempMC.sd.recent)){
@@ -337,7 +345,7 @@ calc_FadingCorr <- function(
       ##otherwise the automatic error value finding
       ##will never work
       res <- NA
-      if(!is(temp,"try-error") && temp$root<1e8) {
+      if (!inherits(temp, "try-error") && temp$root < 1e8) {
         res <- temp$root
       }
       return(res)
@@ -380,6 +388,9 @@ calc_FadingCorr <- function(
   ##remove all NA values from tempMC
   tempMC <- tempMC[!is.na(tempMC)]
 
+  ## discard wild outliers, as they will bias the error if present
+  tempMC <- tempMC[tempMC < 100 * stats::IQR(tempMC)]
+
   ##obtain corrected age
   age.corr <- data.frame(
     AGE = round(temp$root, digits = 4),
