@@ -1,7 +1,7 @@
-#' Apply a simple homogeneity test after Galbraith (2003)
-#'
-#' A simple homogeneity test for De estimates
+#' @title Apply a simple homogeneity test after Galbraith (2003)
 #'
+#' @description
+#' A simple homogeneity test for De estimates.
 #' For details see Galbraith (2003).
 #'
 #' @param data [RLum.Results-class] or [data.frame] (**required**):
@@ -10,7 +10,7 @@
 #' @param log [logical] (*with default*):
 #' perform the homogeneity test with (un-)logged data
 #'
-#' @param ... further arguments (for internal compatibility only).
+#' @param ... further arguments (`verbose`).
 #'
 #' @return
 #' Returns a terminal output. In addition an
@@ -22,20 +22,19 @@
 #' \item{args}{[list] used arguments}
 #' \item{call}{[call] the function call}
 #'
-#' The output should be accessed using the function [get_RLum]
+#' The output should be accessed using the function [get_RLum].
 #'
 #' @section Function version: 0.3.0
 #'
 #' @author Christoph Burow, University of Cologne (Germany), Sebastian Kreutzer,
 #' IRAMAT-CRP2A, Université Bordeaux Montaigne (France)
 #'
-#' @seealso [pchisq]
+#' @seealso [stats::pchisq]
 #'
 #' @references
 #' Galbraith, R.F., 2003. A simple homogeneity test for estimates
 #' of dose obtained using OSL. Ancient TL 21, 75-77.
 #'
-#'
 #' @examples
 #'
 #' ## load example data
@@ -59,25 +58,18 @@ calc_HomogeneityTest <- function(
   data,
   log = TRUE,
   ...
-){
+) {
+  .set_function_name("calc_HomogeneityTest")
+  on.exit(.unset_function_name(), add = TRUE)
 
-  ##============================================================================##
-  ## CONSISTENCY CHECK OF INPUT DATA
-  ##============================================================================##
+  ## Integrity checks -------------------------------------------------------
 
-  if(missing(data)==FALSE){
-    if(!is(data, "data.frame") & !is(data, "RLum.Results")){
-      stop(
-        "[calc_HomogeneityTest()] 'data' object has to be of type 'data.frame' or 'RLum.Results'!",
-        call. = FALSE
-      )
-    } else {
-      if(is(data, "RLum.Results")){
-        data <- get_RLum(data, "data")
-
-      }
-    }
+  .validate_class(data, c("data.frame", "RLum.Results"))
+  .validate_not_empty(data)
+  if (inherits(data, "RLum.Results")) {
+    data <- get_RLum(data, "data")
   }
+  .validate_logical_scalar(log)
 
   ##==========================================================================##
   ## ... ARGUMENTS
@@ -100,7 +92,6 @@ calc_HomogeneityTest <- function(
 
   } else {
     dat <- data
-
   }
 
   wi <- 1 / dat[[2]] ^ 2
@@ -112,7 +103,7 @@ calc_HomogeneityTest <- function(
   df <- length(wi) - 1
   n <- length(wi)
 
-  P <- pchisq(G, df, lower.tail = FALSE)
+  P <- stats::pchisq(G, df, lower.tail = FALSE)
 
 
   ##============================================================================##
@@ -121,14 +112,14 @@ calc_HomogeneityTest <- function(
 
   if(verbose) {
     cat("\n [calc_HomogeneityTest()]")
-    cat(paste("\n\n ---------------------------------"))
-    cat(paste("\n n:                 ", n))
-    cat(paste("\n ---------------------------------"))
-    cat(paste("\n mu:                ", round(mu,4)))
-    cat(paste("\n G-value:           ", round(G,4)))
-    cat(paste("\n Degrees of freedom:", df))
-    cat(paste("\n P-value:           ", round(P,4)))
-    cat(paste("\n ---------------------------------\n\n"))
+    cat("\n\n ---------------------------------")
+    cat("\n n:                 ", n)
+    cat("\n ---------------------------------")
+    cat("\n mu:                ", round(mu,4))
+    cat("\n G-value:           ", round(G,4))
+    cat("\n Degrees of freedom:", df)
+    cat("\n P-value:           ", round(P,4))
+    cat("\n ---------------------------------\n\n")
   }
 
   ##============================================================================##
@@ -152,5 +143,4 @@ calc_HomogeneityTest <- function(
     ),
     info = list(call = sys.call())
   ))
-
 }
