@@ -1,37 +1,40 @@
-#' Calculate De value based on the gSGC by Li et al., 2015
+#' @title Calculate De value based on the gSGC by Li et al., 2015
 #'
-#' Function returns De value and De value error using the global standardised growth
-#' curve (gSGC) assumption proposed by Li et al., 2015 for OSL dating of sedimentary quartz
+#' @description The function computes De value and De value error using the
+#' global standardised growth curve (gSGC) assumption proposed by Li et al.,
+#' 2015 for OSL dating of sedimentary quartz.
 #'
+#' @details
 #' The error of the De value is determined using a Monte Carlo simulation approach.
 #' Solving of the equation is realised using [uniroot].
 #' Large values for `n.MC` will significantly increase the computation time.
 #'
 #'
 #' @param data [data.frame] (**required**):
-#' input data of providing the following columns: `LnTn`, `LnTn.error`, `Lr1Tr1`, `Lr1Tr1.error`, `Dr1`
-#' **Note:** column names are not required. The function expects the input data in the given order
+#' input data  the following columns five columns in the given order:
+#' `LnTn`, `LnTn.error`, `Lr1Tr1`, `Lr1Tr1.error`, `Dr1`. Column names are
+#' not required.
 #'
 #' @param gSGC.type [character] (*with default*):
-#' define the function parameters that
-#' should be used for the iteration procedure: Li et al., 2015 (Table 2)
-#' presented function parameters for two dose ranges: `"0-450"` and `"0-250"`
+#' function parameters to use for the iteration procedure, either `"0-450"` or
+#' `"0-250"`, as presented in Li et al., 2015 (Table 2). This is ignored if
+#' `gSGC.parameters` is set.
 #'
 #' @param gSGC.parameters [list] (*optional*):
 #' option to provide own function parameters used for fitting as named list.
-#' Nomenclature follows Li et al., 2015, i.e. `list(A,A.error,D0,D0.error,c,c.error,Y0,Y0.error,range)`,
-#' range requires a vector for the range the function is considered as valid, e.g. `range = c(0,250)`\cr
-#' Using this option overwrites the default parameter list of the gSGC, meaning the argument
-#' `gSGC.type` will be without effect
+#' Nomenclature follows Li et al., 2015, i.e. `list(A, A.error, D0, D0.error,
+#' c, c.error, Y0, Y0.error, range)`, where `range` is defines the interval
+#' where the function is considered as valid, e.g. `range = c(0,250)`.\cr
+#' If set, option `gSGC.type` will be ignored.
 #'
 #' @param n.MC [integer] (*with default*):
 #' number of Monte Carlo simulation runs for error estimation, see details.
 #'
-#' @param verbose [logical]:
-#' enable or disable terminal output
+#' @param verbose [logical] (*with default*):
+#' enable/disable output to the terminal.
 #'
-#' @param plot [logical]:
-#' enable or disable graphical feedback as plot
+#' @param plot [logical] (*with default*):
+#' enable/disable the plot output.
 #'
 #' @param ... parameters will be passed to the plot output
 #'
@@ -48,8 +51,7 @@
 #' **`@info`**\cr
 #' `$ call`` ([call]) the original function call
 #'
-#'
-#' @section Function version: 0.1.1
+#' @section Function version: 0.1.2
 #'
 #' @author
 #' Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
@@ -86,20 +88,20 @@ calc_gSGC<- function(
   .set_function_name("calc_gSGC")
   on.exit(.unset_function_name(), add = TRUE)
 
-##============================================================================##
-##CHECK INPUT DATA
-##============================================================================##
+  ## Integrity checks -------------------------------------------------------
 
-  if (!is.data.frame(data))
-    .throw_error("'data' must be a data.frame")
+  .validate_class(data, "data.frame")
   if (ncol(data) != 5)
     .throw_error("'data' is expected to have 5 columns")
-  if (!is.character(gSGC.type))
-    .throw_error("'gSGC.type' must be of type 'character'")
+  gSGC.type <- .validate_args(gSGC.type, c("0-250", "0-450"))
 
   ##rename columns for consistency reasons
   colnames(data) <- c('LnTn', 'LnTn.error', 'Lr1Tr1', 'Lr1Tr1.error', 'Dr1')
 
+  ## ensure errors are not negative
+  data$LnTn.error <- abs(data$LnTn.error)
+  data$Lr1Tr1.error <- abs(data$Lr1Tr1.error)
+
 
 ##============================================================================##
 ##DEFINE FUNCTION
@@ -147,9 +149,6 @@ calc_gSGC<- function(
         Y0.error <- 0.00490
 
         range <- c(0.1,250)
-
-      }else{
-        .throw_error("Unknown 'gSGC.type'")
       }
     }
 
@@ -180,7 +179,7 @@ for(i in 1:nrow(data)){
     LnTn.error <- data[i,"LnTn.error"]
 
   ##calculate mean value
-    temp <- try(uniroot(
+  temp <- try(uniroot(
       f,
       interval = c(0.1,450),
       tol = 0.001,
@@ -196,6 +195,17 @@ for(i in 1:nrow(data)){
       maxiter = 1000
     ), silent = TRUE)
 
+  ## allocate matrix
+  temp.MC.matrix <- matrix(nrow = n.MC, ncol = 8)
+  colnames(temp.MC.matrix) <- c("LnTn", "Lr1Tr1", "A", "D0", "c", "Y0", "De", "Eta")
+
+  ## fill the first 6 columns of the matrix
+  temp.MC.matrix[, 1:6] <- matrix(rnorm(
+      n.MC * 6,
+      mean = c(LnTn, Lr1Tr1, A, D0, c, Y0),
+      sd = c(LnTn.error, Lr1Tr1.error, A.error, D0.error, c.error, Y0.error)
+  ), ncol = 6, byrow = TRUE)
+
   if(!inherits(temp, "try-error")){
 
     ##get De
@@ -207,17 +217,6 @@ for(i in 1:nrow(data)){
     ##--------------------------------------------------------------------------##
     ##Monte Carlo simulation for error estimation
 
-    ##set matrix
-    temp.MC.matrix <- matrix(nrow = n.MC, ncol = 8)
-
-    ##fill matrix
-    temp.MC.matrix[,1:6] <- matrix(rnorm(
-      n.MC * 6,
-      mean = c(LnTn, Lr1Tr1, A, D0, c, Y0),
-      sd = c(LnTn.error, Lr1Tr1.error, A.error, D0.error, c.error, Y0.error)
-    ), ncol = 6, byrow = TRUE)
-
-
       ##run uniroot to get the De
       temp.MC.matrix[,7] <- vapply(X = 1:n.MC, FUN = function(x){
 
@@ -236,16 +235,12 @@ for(i in 1:nrow(data)){
                 maxiter = 1000
                 )$root
 
-      }, FUN.VALUE = vector(mode = "numeric", length = 1))
+      }, FUN.VALUE = numeric(1))
 
       ##calculate also the normalisation factor
       temp.MC.matrix[,8] <- (temp.MC.matrix[,3] * (1 - exp( - Dr1 / temp.MC.matrix[,4])) +
         temp.MC.matrix[,5] * Dr1 + temp.MC.matrix[,6])/temp.MC.matrix[,2]
 
-
-      ##re-name matrix
-      colnames(temp.MC.matrix) <- c("LnTn","Lr1Tr1","A","D0","c","Y0","De","Eta")
-
       ##get De error as SD
       De.error <- sd(temp.MC.matrix[,7])
 
@@ -254,16 +249,6 @@ for(i in 1:nrow(data)){
     De <- NA
     Eta <- NA
     De.error <- NA
-
-    ##set matrix
-    temp.MC.matrix <- matrix(nrow = n.MC, ncol = 8)
-
-    ##fill matrix
-    temp.MC.matrix[,1:6] <- matrix(rnorm(
-      n.MC * 6,
-      mean = c(LnTn, Lr1Tr1, A, D0, c, Y0),
-      sd = c(LnTn.error, Lr1Tr1.error, A.error, D0.error, c.error, Y0.error)
-    ), ncol = 6, byrow = TRUE)
   }
 
 
@@ -326,25 +311,20 @@ for(i in 1:nrow(data)){
       }else{
 
         if(temp$root < 450){
-          shape::Arrows(
-            x0 = 450,
+          x0 <- 450
+          x1 <- 500
+        } else {
+          x0 <- 50
+          x1 <- 0
+        }
+        shape::Arrows(
+            x0 = x0,
             y0 = par()$usr[4] - 0.2,
-            x1 = 500,
+            x1 = x1,
             y1 = par()$usr[4] - 0.2,
             arr.type = "triangle",
             col = "red"
-          )
-        }else{
-
-            shape::Arrows(
-              x0 = 50,
-              y0 = par()$usr[4] - 0.2,
-              x1 = 0,
-              y1 = par()$usr[4] - 0.2,
-              arr.type = "triangle",
-              col = "red"
-            )
-        }
+        )
 
         mtext(side = 1, text = "Out of bounds!", col = "red")
       }
@@ -375,17 +355,10 @@ for(i in 1:nrow(data)){
 ##CREATE OUTPUT OBJECTS
 ##============================================================================##
 
-    ##needed for data.table
-    temp.De <- De
-    temp.De.error <- De.error
-    temp.Eta <- Eta
-
     ##replace values in the data.table with values
-    output.data[i, `:=` (DE = temp.De,
-                         DE.ERROR = temp.De.error,
-                         ETA = temp.Eta)]
-
-    rm(list = c('temp.De', 'temp.De.error', 'temp.Eta'))
+    output.data[i, `:=` (DE = De,
+                         DE.ERROR = De.error,
+                         ETA = Eta)]
 
     ##matrix - to prevent memory overload limit output
     if(n.MC * nrow(data) > 1e6){
