@@ -20,7 +20,7 @@
 #'@param n.MC [numeric] (*with default*): number of Monte-Carlo runs for the
 #'error calculation
 #'
-#'@param plot [logical] (*with default*): enables/disables the control plot output
+#'@param plot [logical] (*with default*): enable/disable the plot output.
 #'
 #'@return Returns an S4 object of type [RLum.Results-class].
 #'
@@ -73,17 +73,14 @@ calc_gSGC_feldspar <- function (
   gSGC.parameters,
   n.MC = 100,
   plot = FALSE
-){
+) {
+  .set_function_name("calc_gSGC_feldspar")
+  on.exit(.unset_function_name(), add = TRUE)
 
-# Integrity checks --------------------------------------------------------
-  if (!is(data, "data.frame")) {
-    stop("[calc_gSGC_feldspar()] 'data' needs to be of type data.frame.", call. = FALSE)
-  }
-  if (!is(gSGC.type[1], "character")) {
-    stop("[calc_gSGC_feldspar()] 'gSGC.type' needs to be of type character.", call. = FALSE)
-  }
+  ## Integrity checks -------------------------------------------------------
+  .validate_class(data, "data.frame")
   if (ncol(data) != 5) {
-    stop("[calc_gSGC_feldspar()] Structure of 'data' does not fit the expectations.", call. = FALSE)
+    .throw_error("'data' should have 5 columns")
   }
   colnames(data) <- c("LnTn", "LnTn.error", "Lr1Tr1", "Lr1Tr1.error",
                       "Dr1")
@@ -104,6 +101,7 @@ calc_gSGC_feldspar <- function (
     D0_2.3 = c( 2000, 2450, 1420, 1420, 2300, 2900, 1500, 2340, 2880, 1320, 2080, 2980, 1000, 1780, 2500),
     D0_3 = c( 2780, 3280, 2520, 1950, 3100, 4960, 2060, 3130, 4760, 1780, 2800, 5120, 1380, 2360, 4060)
   )
+  .validate_args(gSGC.type, params$Type)
 
   # these are user specified parameters if they so desire
   if (!missing(gSGC.parameters)){
@@ -117,7 +115,6 @@ calc_gSGC_feldspar <- function (
     y0_err <- gSGC.parameters$y0_err
 
   } else {
-   if (gSGC.type[1] %in% params$Type){
    # take the user input pIRSL temperature and assign the correct parameters
    index <- match(gSGC.type,params$Type)
 
@@ -135,14 +132,6 @@ calc_gSGC_feldspar <- function (
 
       y0 <- params$y0[index]
       y0_err <- params$y0_err[index]
-
-    } else {
-      # give error if input is wrong
-      stop(
-        paste0("[calc_gSGC_feldspar()] 'gSGC.type' needs to be one of the accepted values, such as: ",
-          paste(params$Type, collapse = ", ")),
-        call. = FALSE)
-    }
   }
 
   ##set function for uniroot
@@ -154,7 +143,6 @@ calc_gSGC_feldspar <- function (
     f_Dr <-  y1 * (1 - exp(-Dr1 / D1)) + y2 * (1 - exp(-Dr1 / D2)) + y0
     ##return(f_D/Lr1Tr1 - f_Dr/LnTn) ##TODO double check seems to be wrong
     return(f_Dr/Lr1Tr1 - f_D/LnTn)
-
   }
 
 # Run calculation ---------------------------------------------------------
@@ -186,8 +174,7 @@ calc_gSGC_feldspar <- function (
 
       ## in case the initial uniroot solve does not work
       if(inherits(temp, "try-error")) {
-        message("[calc_gSGC_feldspar()] Error: No solution found for ",
-                "dataset #", i, ", NA returned")
+        .throw_message("No solution found for dataset #", i, ", NA returned")
         return(NA)
       }
 
@@ -234,7 +221,6 @@ calc_gSGC_feldspar <- function (
             # give an NA if uniroot cannot find a root (usually due to bad random values)
             temp.MC.matrix[j,8] <- NA
           }
-
         }
 
         # set the De uncertainty as the standard deviations of the randomly generated des
@@ -290,9 +276,7 @@ calc_gSGC_feldspar <- function (
          ylim = c(0, y_max),
          main = ""
        )
-
     }
-
   }
 
 
@@ -311,7 +295,6 @@ calc_gSGC_feldspar <- function (
     HPD <- .calc_HPDI(na.exclude(l[[i]]$m.MC[,8]))
     m[i,3] <- HPD[1,1]
     m[i,4] <- HPD[1,2]
-
   }
 
   df <- data.frame(
@@ -331,5 +314,4 @@ calc_gSGC_feldspar <- function (
         call = sys.call()
       )
     ))
-
 }
