@@ -1,7 +1,8 @@
-#' Create LaTex tables from data.frames and RLum objects
+#' @title Create LaTeX tables from data.frames and RLum objects
 #'
-#' This function takes a data.frame and returns a table in LaTex code that
-#' can be copied into any tex document.
+#' @description
+#' This function takes a [data.frame] and returns a table in LaTeX code that
+#' can be copied into any TeX document.
 #'
 #' @param x [data.frame] or `RLum` object (**required**)
 #'
@@ -10,68 +11,74 @@
 #' @param col.names currently unused
 #'
 #' @param comments [logical] (*with default*):
-#' insert LaTex comments
+#' insert LaTeX comments
 #'
-#' @param pos [character] (*with default*):
-#' `character` of length one specifying the alignment of each column, e.g.,
-#' pos'clr' for a three column data frame and center, left
-#'  and right alignment
+#' @param pos [character] (*with default*): `character` of length one specifying the
+#' alignment of each column, e.g., `pos = 'clr'` for a three column data frame and
+#' center, left and right alignment
 #'
-#' @param digits [numeric] (*with default*):
-#' number of digits (numeric fields)
+#' @param digits [numeric] (*with default*): number of digits to be displayed (numeric fields)
 #'
 #' @param rm.zero [logical] (*with default*): remove columns containing
-#' only zeros, however this might not be wanted in all cases
+#' only zeros, however, this might not be wanted in all cases
 #'
-#' @param select [character] (*optional*):
-#' a [character] vector passed to [subset]
+#' @param select [character] (*optional*): a [character] vector passed to [subset]
 #'
-#' @param split [integer] (*optional*):
-#' an [integer] specifying the number of individual tables
-#' the data frame is split into. Useful for wide tables. Currently unused.
+#' @param split [integer] (*optional*): an [integer] specifying the number of individual tables
+#' the [data.frame] is split into. Useful for wide tables. Currently unused.
 #'
 #' @param tabular_only [logical] (*with default*): if `TRUE` only the tabular but not the
 #' table environment is returned. This gives a lot of additional flexibility at hand
 #'
 #' @param ... options: `verbose`
 #'
-#' @section TODO:
-#' - Improve by using RegEx to dynamically find error fields, eg. ( "([ ]err)|(^err)" )
-#' -
+#' @section Function version: 0.2.0
+#'
+#' @author
+#' Christoph Burow, University of Cologne (Germany), Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
+#'
+#' @keywords internal
 #'
 #' @return
-#' Returns LaTex code
+#' Returns LaTeX code
 #'
 #' @examples
-#' df <- data.frame(x = 1:10, y = letters[1:10])
+#'  df <- data.frame(x = 1:10, y = letters[1:10])
 #' .as.latex.table(df)
 #' .as.latex.table(df, pos = "lr")
 #' .as.latex.table(df, select = "y", pos = "r")
 #'
 #' @md
-#' @noRd
-.as.latex.table <- function(x,
-                            row.names = NULL,
-                            col.names = NULL,
-                            comments = TRUE,
-                            pos = "c",
-                            digits = 3,
-                            rm.zero = TRUE,
-                            select,
-                            split = NULL,
-                            tabular_only = FALSE,
-                            ...) {
+#' @export
+.as.latex.table <- function(
+    x,
+    row.names = NULL,
+    col.names = NULL,
+    comments = TRUE,
+    pos = "c",
+    digits = 3,
+    rm.zero = TRUE,
+    select,
+    split = NULL,
+    tabular_only = FALSE,
+...) {
 
-  args <- list(x = x,
-               row.names = row.names,
-               col.names = col.names,
-               comments = comments,
-               pos = pos,
-               digits = digits,
-               rm.zero = rm.zero,
-               split = split,
-               tabular_only = tabular_only,
-               ... = ...)
+
+  ## TODO:
+  # - Improve by using RegEx to dynamically find error fields, e.g. ( "([ ]err)|(^err)" )
+  # -
+
+  args <- list(
+    x = x,
+    row.names = row.names,
+    col.names = col.names,
+    comments = comments,
+    pos = pos,
+    digits = digits,
+    rm.zero = rm.zero,
+    split = split,
+    tabular_only = tabular_only,
+    ... = ...)
   if (!missing(select))
     args$select <- select
 
@@ -84,16 +91,17 @@
 ################################################################################
 ## "Method"                  RLum.Results                                     ##
 ##----------------------------------------------------------------------------##
-.as.latex.table.RLum.Results <- function(x,
-                                         row.names = NULL,
-                                         col.names = NULL,
-                                         comments = TRUE,
-                                         pos = "c",
-                                         digits = 3,
-                                         rm.zero = TRUE,
-                                         select,
-                                         split = NULL,
-                                         ...) {
+.as.latex.table.RLum.Results <- function(
+    x,
+    row.names = NULL,
+    col.names = NULL,
+    comments = TRUE,
+    pos = "c",
+    digits = 3,
+    rm.zero = TRUE,
+    select,
+    split = NULL,
+    ...) {
 
   ## Object: DRAC.highlights
   if (x@originator == "use_DRAC") {
@@ -104,13 +112,8 @@
     if(rm.zero){
       x <- x[sapply(x, function(y){
         y <- suppressWarnings(as.numeric(y))
-        if(anyNA(y) || sum(y, na.rm = TRUE) != 0){
-          TRUE
+        if(anyNA(y) || sum(y, na.rm = TRUE) != 0) TRUE else FALSE
 
-        }else{
-          FALSE
-
-        }
       })]
     }
 
@@ -128,13 +131,10 @@
     text <- strsplit(text[[1]], split = "\n", fixed = TRUE)
 
     ##exchange columns ... or delete them at all (2nd step)
-
       ##Mineral ID
       for(i in 1:length(text)){
         text[[i]][grepl(pattern = "Mineral", x = text[[i]], fixed = TRUE)] <-
           "\t\\multicolumn{1}{p{0.5cm}}{\\centering \\textbf{M.}} & "
-
-
       }
 
     ##put things again together (single character)
@@ -156,28 +156,27 @@
     return(text)
 
   }# EndOf::use_DRAC
-
 }
 
 ################################################################################
 ## "Method"                     data.frame                                    ##
 ##----------------------------------------------------------------------------##
-.as.latex.table.data.frame <- function(x,
-                                       row.names = NULL,
-                                       col.names = NULL,
-                                       comments = TRUE,
-                                       pos = "c",
-                                       digits = 3,
-                                       select,
-                                       split = NULL,
-                                       tabular_only = FALSE,
-                                       ...) {
+.as.latex.table.data.frame <- function(
+    x,
+    row.names = NULL,
+    col.names = NULL,
+    comments = TRUE,
+    pos = "c",
+    digits = 3,
+    select,
+    split = NULL,
+    tabular_only = FALSE,
+    ...) {
   .set_function_name("as.latex.table.data.frame")
   on.exit(.unset_function_name(), add = TRUE)
 
-  ## Integrity checks ----
-  if (!is.data.frame(x))
-    .throw_error("'x' must be a data frame")
+  ## Integrity tests --------------------------------------------------------
+  .validate_class(x, "data.frame")
   if (!is.null(col.names) && length(col.names) != ncol(x))
     .throw_error("Length of 'col.names' does not match the number of columns")
   if (!is.null(row.names) && length(row.names) != nrow(x))
@@ -214,7 +213,6 @@
   tex.table.list <- vector("list", split)
 
   for (i in 1:length(tex.table.list)) {
-
     x.chunk <- x[ ,chunks.start[i]:chunks.end[i]]
 
     if (ncol(x) == 1) {
@@ -222,7 +220,6 @@
       colnames(x.chunk) <- names(x[i])
     }
 
-
     ## Comments ----
     tex.comment.usePackage <- ifelse(comments,
                                      "% add usepackage{adjustbox} to latex preamble \n",
@@ -252,39 +249,45 @@
     tex.table.rows <- gsub("_", "\\_", tex.table.rows, fixed = TRUE)
 
     ## Tex table ----
-    if (nchar(pos) != 1 && nchar(pos) != ncol(x))
-      pos <- "c"
-    if (!any(strsplit(pos, split = "")[[1]] %in% c("l", "c", "r")))
+    pos.chars <- unlist(strsplit(pos, split = ""))
+    if (!all(pos.chars %in% c("l", "c", "r")) ||
+        (length(pos.chars) != 1 && length(pos.chars) != ncol(x))) {
       pos <- "c"
+    }
     if (nchar(pos) == 1)
       pos <- paste0(rep(pos, ncol(x)), collapse = "")
 
     if(tabular_only){
-      tex.table.begin <- paste0(paste("  \\begin{tabular}{", pos, "}\n"),
-                                "     \\hline \n")
+      tex.table.begin <- paste0(
+        paste("  \\begin{tabular}{", pos, "}\n"),
+              "     \\hline \n")
 
-      tex.table.end <-  paste0("     \\hline \n",
-                               "   \\end{tabular}")
+      tex.table.end <-  paste0(
+        "     \\hline \n",
+        "   \\end{tabular}")
 
     }else{
-      tex.table.begin <- paste0("\\begin{table}[ht] \n",
-                                "  \\centering \n",
-                                "  \\begin{adjustbox}{max width=\\textwidth} \n",
-                                paste("  \\begin{tabular}{", pos, "}\n"),
-                                "     \\hline \n")
+      tex.table.begin <- paste0(
+        "\\begin{table}[ht] \n",
+        "  \\centering \n",
+        "  \\begin{adjustbox}{max width=\\textwidth} \n",
+        paste("  \\begin{tabular}{", pos, "}\n"),
+        "     \\hline \n")
 
-      tex.table.end <-  paste0("     \\hline \n",
-                               "   \\end{tabular} \n",
-                               "   \\end{adjustbox} \n",
-                               "\\end{table}")
+      tex.table.end <-  paste0(
+        "     \\hline \n",
+        "   \\end{tabular} \n",
+        "   \\end{adjustbox} \n",
+        "\\end{table}")
     }
 
-    tex.table <- paste0(tex.comment.usePackage,
-                        tex.table.begin,
-                        tex.table.header,
-                        "\\hline \n",
-                        tex.table.rows,
-                        tex.table.end)
+    tex.table <- paste0(
+      tex.comment.usePackage,
+      tex.table.begin,
+      tex.table.header,
+      "\\hline \n",
+      tex.table.rows,
+      tex.table.end)
 
     if (options$verbose)
       cat(tex.table)
@@ -296,7 +299,7 @@
 }
 
 # This function takes a data.frame, checks each column and tries to
-# force the specified amount of digits if numeric or coerceable to numeric
+# force the specified amount of digits if numeric or coercible to numeric
 .digits <- function(x, digits) {
   for (i in 1:ncol(x)) {
     if (is.factor(x[ ,i]))
