@@ -84,8 +84,8 @@
       assign(x = "warning_collector",
              value = temp,
              envir = env)
-      ##TODO should be replaced tryInvokeRestart once R 4.1 was released
-      invokeRestart("muffleWarning")
+
+      tryInvokeRestart ("muffleWarning")
     }
   )
 
@@ -114,33 +114,33 @@
 #+ .smoothing()      +
 #+++++++++++++++++++++
 
-#' Allows smoothing of data based on the function zoo::rollmean
+#' Allows smoothing of data based on rolling means or medians
 #'
-#' The function just allows a direct and meaningful access to the functionality of the zoo::rollmean()
-#' function. Arguments of the function are only partly valid.
+#' The function just allows a direct and meaningful access to the
+#' functionality of `data.table::frollmean()` and `data.table::frollmedian()`.
+#' Arguments of the function are only partly valid.
 #'
 #' @param x [numeric] (**required**):
 #' the object for which the smoothing should be applied.
 #'
 #' @param k [integer] (*with default*):
-#' window for the rolling mean; must be odd for rollmedian.
-#' If nothing is set k is set automatically
+#' window for the rolling mean. If not set, `k` is set automatically.
 #'
 #' @param fill [numeric] (*with default*):
-#' a vector defining the left and the right hand data
+#' value used to pad the result so to have the same length as the input
 #'
 #' @param align [character] (*with default*):
-#' specifying whether the index of the result should be
-#' left- or right-aligned or centered (default) compared to the rolling window of observations,
-#' allowed `"right"`, `"center"` and `left`
+#' one of `"right"`, `"center"` or `"left"`, specifying whether the index
+#' of the result should be right-aligned (default), centered, or lef-aligned
+#' compared to the rolling window of observations
 #'
 #' @param method [method] (*with default*):
 #' defines which method should be applied for the smoothing: `"mean"` or `"median"`
 #'
 #' @return
-#' Returns the same object as the input and a warning table
+#' Returns the same object as the input
 #'
-#' @section Function version: 0.1.1
+#' @section Function version: 0.2
 #'
 #' @author Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
 #'
@@ -157,28 +157,116 @@
   fill = NA,
   align = "right",
   method = "mean") {
+  .set_function_name(".smoothing")
+  on.exit(.unset_function_name(), add = TRUE)
+
+  .validate_args(align, c("right", "center", "left"))
+  .validate_args(method, c("mean", "median"))
 
   ##set k
-  if (is.null(k)){
+  if (is.null(k))
    k <- ceiling(length(x) / 100)
-   if(method == "median" && k %%2 ==0)
-     k <- k + 1
-  }
 
   ##smooth data
   if(method == "mean"){
-    zoo::rollmean(x, k = k, fill = fill, align = align)
+    data.table::frollmean(x, n = k, fill = fill, align = align)
 
   }else if(method == "median"){
-    zoo::rollmedian(x, k = k, fill = fill, align = align)
-
-  }else{
-    stop("[Luminescence:::.smoothing()] Unvalid input for 'method'!")
-
+    data.table::frollapply(x, n = k, FUN = "median",
+                           fill = fill, align = align)
   }
-
 }
 
+#++++++++++++++++++++++++++++++
+#+ Curve normalisation        +
+#++++++++++++++++++++++++++++++
+
+#' Curve normalisation
+#'
+#' Details on the normalisation methods are specified in [plot_RLum.Analysis]
+#' and [plot_RLum.Data.Curve].
+#'
+#' The function assumes that `NA` or other invalid values have already been
+#' removed by the caller function, and that the `norm` option has already
+#' been validated.
+#'
+#' @param data [numeric] (**required**):
+#' the curve data to be normalised
+#'
+#' @param norm [logical] [character] (**required**):
+#' if logical, whether curve normalisation should occur; alternatively, one
+#' of `"max"` (used with `TRUE`), `"last"` and `"huot"`.
+#'
+#' @md
+#' @noRd
+.normalise_curve <- function(data, norm) {
+
+  if (norm == "max" || norm == TRUE) {
+    data <- data / max(data)
+  }
+  else if (norm == "last") {
+    data <- data / data[length(data)]
+  }
+  else if (norm == "huot") {
+    bg <- median(data[floor(length(data) * 0.8):length(data)])
+    data <- (data - bg) / max(data - bg)
+  }
+
+  ## check for Inf and NA
+  if (any(is.infinite(data)) || anyNA(data)) {
+    data[is.infinite(data) | is.na(data)] <- 0
+    .throw_warning("Curve normalisation produced Inf/NaN values, ",
+                   "values replaced by 0")
+  }
+
+  return(data)
+}
+
+#++++++++++++++++++++++++++++++
+#+ LxTx error calculation     +
+#++++++++++++++++++++++++++++++
+
+#' Calculation of the `Lx/Tx` error
+#'
+#' Calculates the `Lx/Tx` error according Galbraith (2014).
+#'
+#' @param LnLxTnTx [data.frame] (**required**): containing columns
+#'        `Net_LnLx`, `Net_TnTx`, `Net_LnLx.Error`, `Net_TnTx.Error`.
+#' @param sig0 [numeric] (**required**): extra error component to be added
+#'        to the final `Lx/Tx` error value (e.g., instrumental error).
+#' @param digits [integer] (**required**): round numbers to the specified
+#'        digits. If set to `NULL`, no rounding occurs.
+#'
+#' @md
+#' @noRd
+.calculate_LxTx_error <- function(LnLxTnTx, sig0, digits) {
+
+  ## extract fields from the data.frame
+  LnLx <- LnLxTnTx$Net_LnLx
+  TnTx <- LnLxTnTx$Net_TnTx
+  LnLx.Error <- LnLxTnTx$Net_LnLx.Error
+  TnTx.Error <- LnLxTnTx$Net_TnTx.Error
+
+  ## calculate Ln/Tx
+  LxTx <- LnLx / TnTx
+  if (is.nan(LxTx)) LxTx <- 0
+
+  ## calculate Ln/Tx error
+  LxTx.relError <- sqrt((LnLx.Error / LnLx)^2 + (TnTx.Error / TnTx)^2)
+  LxTx.Error <- abs(LxTx * LxTx.relError)
+  if (is.nan(LxTx.Error)) LxTx.Error <- 0
+
+  ## add an extra component of error
+  LxTx.Error <- sqrt(LxTx.Error^2 + (sig0 * LxTx)^2)
+
+  ## combined values
+  res <- cbind(LnLxTnTx, LxTx, LxTx.Error)
+  if (!is.null(digits)) {
+    res[1, ] <- round(res[1, ], digits = digits)
+  }
+
+  return(res)
+}
 
 #++++++++++++++++++++++++++++++
 #+ Scientific axis annotation +
@@ -279,6 +367,59 @@ fancy_scientific <- function(l) {
     args))
 }
 
+#' Convert position keywords into summary placement coordinates
+#'
+#' @param pos [numeric] or [character] (**required**) Position coordinates
+#'        or keyword (one of "topleft", "top", "topright", "left", "center",
+#'        "right", "bottomleft", "bottom", "bottomright").
+#' @param xlim [numeric] (**required**) The x-axis limits.
+#' @param ylim [numeric] (**required**) The y-axis limits.
+#'
+#' @return
+#' A list of two elements: `pos` and `adj`.
+#'
+#' @md
+#' @noRd
+.get_keyword_coordinates <- function(pos, xlim, ylim) {
+  adj <- NA
+  if (missing(pos)) {
+    pos <- c(xlim[1], ylim[2])
+    adj <- c(0, 1)
+  } else if (length(pos) == 2) {
+    pos <- pos
+    adj <- c(0, 1)
+  } else if (pos[1] == "topleft") {
+    pos <- c(xlim[1], ylim[2])
+    adj <- c(0, 1)
+  } else if (pos[1] == "top") {
+    pos <- c(mean(xlim), ylim[2])
+    adj <- c(0.5, 1)
+  } else if (pos[1] == "topright") {
+    pos <- c(xlim[2], ylim[2])
+    adj <- c(1, 1)
+  } else if (pos[1] == "left") {
+    pos <- c(xlim[1], mean(ylim))
+    adj <- c(0, 0.5)
+  } else if (pos[1] == "center") {
+    pos <- c(mean(xlim), mean(ylim))
+    adj <- c(0.5, 0.5)
+  } else if (pos[1] == "right") {
+    pos <- c(xlim[2], mean(ylim))
+    adj <- c(1, 0.5)
+  }else if (pos[1] == "bottomleft") {
+    pos <- c(xlim[1], ylim[1])
+    adj <- c(0, 0)
+  } else if (pos[1] == "bottom") {
+    pos <- c(mean(xlim), ylim[1])
+    adj <- c(0.5, 0)
+  } else if (pos[1] == "bottomright") {
+    pos <- c(xlim[2], ylim[1])
+    adj <- c(1, 0)
+  }
+
+  return(list(pos = pos, adj = adj))
+}
+
 
 #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 #+ Statistical Summary for Plot functions
@@ -289,18 +430,22 @@ fancy_scientific <- function(l) {
 #' the package. This should unify the approach how such things are created and support, theoretically
 #' all keywords for all plot functions in a similar way.
 #'
-#'@param x [data.frame] (optional): output from the function `calc_Statistics()`. If nothing is
-#'provided a list of prefix keyword combinations supported by the function `calc_Statistics()` is returned.
+#' @param x [data.frame] (*optional*): output from function `calc_Statistics()`.
+#' If nothing is provided, a list of prefix keyword combinations supported by
+#' `calc_Statistics()` is returned.
 #'
-#'@param keywords[character] (with default): keywords supported by the function `calc_Statistics()`
+#' @param keywords[character] (*with default*): keywords supported by function
+#' `calc_Statistics()`.
 #'
-#'@param digits [numeric] (with default): modifiy the digits independently for the plot output
+#' @param digits [numeric] (*with default*): modifiy the digits independently
+#' for the plot output.
 #'
-#'@param sep [character] (with default): a separator used for the creation of the output of the plot
+#' @param sep [character] (*with default*): separator used for the creation of
+#' the output of the plot.
 #'
-#'@param prefix [character] (with default): allows to add a leading prefix to the string
+#'@param prefix [character] (*with default*): prefix to add to the string
 #'
-#'@param suffix [character] (with default): allows to add a suffix to the entire string
+#'@param suffix [character] (*with default*): suffix to add to the string
 #'
 #'@author Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
 #'
@@ -371,7 +516,7 @@ fancy_scientific <- function(l) {
   })
 
   ##remove NULL entries
-  l <- l[!sapply(l, is.null)]
+  l <- .rm_NULL_elements(l)
 
   ##construct final call
   return(paste0(prefix, paste(unlist(l), collapse = sep), suffix))
@@ -443,6 +588,28 @@ fancy_scientific <- function(l) {
   x[vapply(x, "class", character(1)) == class[1]]
 }
 
+#++++++++++++++++++++++++++++++
+#+ .rm_NULL                   +
+#++++++++++++++++++++++++++++++
+#' @title Removes all NULL elements from list
+#'
+#' @param x [list] (**required**): list
+#'
+#' @author Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
+#'
+#' @examples
+#' l <- list("a", NULL)
+#' .rm_NULL_elements(l)
+#'
+#' @return [list] without `NULL` elements, can be empty
+#'
+#' @md
+#' @noRd
+.rm_NULL_elements <- function(x){
+  x[vapply(x, is.null, logical(1))] <- NULL
+  x
+}
+
 #++++++++++++++++++++++++++++++
 #+ .matrix_binning            +
 #++++++++++++++++++++++++++++++
@@ -489,10 +656,11 @@ fancy_scientific <- function(l) {
   bin_size = 1,
   bin_col = FALSE,
   names = NULL) {
+  .set_function_name(".matrix_binning")
+  on.exit(.unset_function_name(), add = TRUE)
 
   #@ The only check
-  if(!inherits(m, "matrix"))
-    stop("[.matrix_binning()] Input is not of class 'matrix'!", call. = FALSE)
+  .validate_class(m, "matrix")
 
   ## transpose in column mode
   if(bin_col) m <- t(m)
@@ -602,8 +770,7 @@ fancy_scientific <- function(l) {
   for(i in 1:length(args)){
     if(is.na(names(args[i])) || names(args[i]) == "...") next
     if(class(args[[i]])[1] == "name" & names(args[i]) != "...") {
-      stop(paste0("[",f_call[[1]],"()]: Argument <",
-                  names(args[i]), "> missing; with no default!"), call. = FALSE)
+      .throw_error("Argument '", names(args[i]), "' missing, with no default")
     }
 
     ##evaluate and cover special cases
@@ -647,8 +814,7 @@ fancy_scientific <- function(l) {
 #'
 #' @param prob [numeric] (*with default*): sets aimed probability interval
 #'
-#' @param plot [logical] (*with default*): enables/disables additional control
-#' plot
+#' @param plot [logical] (*with default*): enable/disable the plot output.
 #'
 #' @param ... further arguments passed to [stats::density]
 #'
@@ -717,6 +883,9 @@ fancy_scientific <- function(l) {
 #'
 #'@param dest [character] (*with default*)
 #'
+#' @param verbose [logical] (*with default*)
+#' enable/disable output to the terminal.
+#'
 #'@returns Returns either nothing (no URL) or the file path of the downloaded file
 #'
 #'@author Sebastian Kreutzer, Insitut of Geography, Heidelberg University, Germany
@@ -737,9 +906,9 @@ fancy_scientific <- function(l) {
 #'@noRd
 .download_file <- function(
     url,
-    destfile = tempfile()
+    destfile = tempfile(),
+    verbose = TRUE
 ) {
-
   ## get name of calling function
   caller <- paste0("[", as.character(sys.call(which = -1)[[1]]), "()]")
   out_file_path <- NULL
@@ -747,12 +916,19 @@ fancy_scientific <- function(l) {
   ## detect and extract URL
   if(grepl(pattern = "https?\\:\\/\\/", x = url, perl = TRUE)) {
     ## status reports
-    message(paste0(caller, " URL detected: ", url), appendLF = TRUE)
-    message(paste0(caller, " Attempting download ... "), appendLF = FALSE)
+    if (verbose) {
+      message(caller, " URL detected: ", url)
+      message(caller, " Attempting download ... ", appendLF = FALSE)
+    }
 
     ## extract URL from string only
     url <- regmatches(x = url, m = regexec(pattern = "https?\\:\\/\\/.+", text = url, perl = TRUE))[[1]]
 
+    fail.msg <- function(w) {
+      if (verbose)
+        message("FAILED")
+      NULL
+    }
     ## use internal download
     t <- tryCatch(
       expr = download.file(
@@ -762,27 +938,19 @@ fancy_scientific <- function(l) {
         mode = "wb", ## this is needed for Windows otherwise the download does not work
         cacheOK = FALSE,
         method = "auto"),
-      warning = function(w) {
-        message("FAILED ", appendLF = TRUE)
-        return(NULL)
-      },
-      error = function(e) {
-        message("FAILED ", appendLF = TRUE)
-        return(NULL)
-      })
+      warning = fail.msg,
+      error = fail.msg)
 
     if(!is.null(t) && t == 0) {
-      message("OK ", appendLF = TRUE)
+      if (verbose)
+        message("OK ", appendLF = TRUE)
       out_file_path <- destfile
       unlink(url)
-
     }
-
   }
 
   ## return file path
   return(out_file_path)
-
 }
 
 #'@title Extract named element from nested list
@@ -834,10 +1002,9 @@ fancy_scientific <- function(l) {
 #' @title Set/unset the function name for error/warning reporting
 #'
 #' @description
-#' These utilities allow for more precise error reporting from `.throw_error()`
-#' and `.throw_warning()`. They must be called just once per function
-#' (`.set_function_name() at the start and `.unset_function_name() at the
-#' end) if the function calls either `.throw_error()` or `.throw_warning()`.
+#' These utilities allow more precise error reporting from `.throw_error()`,
+#' `.throw_warning()` and `.throw_message()`. They must be called exactly once
+#' if the function calls one of the `.throw_*()` or `.validate_*()` functions.
 #'
 #' @param name [character] (**required**): the name of the function
 #'
@@ -856,7 +1023,7 @@ fancy_scientific <- function(l) {
 #' `.unset_function_name()` must be delegated to `on.exit(..., add = TRUE)`.
 #'
 #' Therefore, it is suggested to put these two lines at the very beginning
-#' of each function (if one of the throwing functions is used by it):
+#' of each function (if any of the throwing/validating functions is used):
 #'
 #'   .set_function_name("name_of_the_function")
 #'   on.exit(.unset_function_name(), add = TRUE)
@@ -896,6 +1063,17 @@ fancy_scientific <- function(l) {
   warning("[", .LuminescenceEnv$fn_stack[[top.idx]], "()] ", ..., call. = FALSE)
 }
 
+#'@title Throws a Custom Tailored Message
+#'
+#'@param ... the message to throw, preceded by "Error:"
+#'
+#'@md
+#'@noRd
+.throw_message <- function(...) {
+  top.idx <- length(.LuminescenceEnv$fn_stack)
+  message("[", .LuminescenceEnv$fn_stack[[top.idx]], "()] Error: ", ...)
+}
+
 #' @title Silence Output and Warnings during Tests
 #'
 #' @description
@@ -922,6 +1100,212 @@ SW <- function(expr) {
   capture.output(suppressMessages(suppressWarnings(expr)))
 }
 
+#' @title Validate a character argument from a list of choices
+#'
+#' @description
+#' This is inspired by [base::match.arg], but is has a more user-friendly
+#' error message as it reports the exact name of the argument that is being
+#' validated. This function always requires the choices to be specified: this
+#' better fits with the current state of the Luminescence package, which only
+#' rarely lists all choices in the formal function arguments, so
+#' [base::match.arg] would have very limited use.
+#'
+#' @param arg [character] (**required**): variable to validate.
+#' @param choices [vector] [character] (**required**): a vector of candidate
+#'        values.
+#' @param null.ok [logical] (*with default*): whether a `NULL` value should be
+#'        considered valid (`FALSE` by default).
+#' @param name [character] (*with default*): variable name to report in case
+#'        of error; if not specified it's inferred from the name of the
+#'        variable tested.
+#' @param extra [character] (*with default*): additional choice to be reported
+#'        after the valid choices and preceded by "or". If `null.ok = TRUE`,
+#'        the text reported is automatically set to `"or NULL"`, otherwise it
+#'        will take the form `"or {extra}"`.
+#'
+#' @return
+#' The validated choice (may be `NULL` if `arg = NULL` and `null.ok = TRUE`).
+#' If `arg` contains multiple elements, only the first matching one will be
+#' returned.
+#'
+#' @md
+#' @noRd
+.validate_args <- function(arg, choices, null.ok = FALSE,
+                           name = NULL, extra = NULL) {
+
+  if (is.null(arg) && null.ok)
+    return(NULL)
+
+  ## name of the argument to report if not specified
+  if (is.null(name))
+    name <- sprintf("'%s'", all.vars(match.call())[1])
+
+  ## `arg` will have multiple values when the available choices are listed
+  ## in the function's formal arguments: in that case all elements in `arg`
+  ## are also in `choices` and we return the first one
+  if (length(arg) > 1L) {
+    if (all(arg %in% choices))
+      return(arg[1L])
+
+    ## we throw an error to catch cases when the formal arguments are
+    ## changed but `choices` has not been updated
+    .throw_error(name, " contains multiple values but not all of them ",
+                 "match 'choices'")
+  }
+
+  ## additional text to append after the valid choices to account for
+  ## extra options that cannot be validated or for NULL
+  choices.extra <- c(sQuote(choices, q = FALSE), extra)
+  if (null.ok)
+    choices.extra <- c(choices.extra, "NULL")
+
+  ## use an 'or' instead of a comma before the last choice
+  if (length(choices.extra) > 1) {
+    msg.head <- head(choices.extra, -1)
+    msg.tail <- paste(" or", tail(choices.extra, 1))
+  } else {
+    msg.head <- choices.extra
+    msg.tail <- NULL
+  }
+
+  idx.match <- pmatch(arg, choices, nomatch = 0L, duplicates.ok = TRUE)
+  if (all(idx.match == 0L))
+    .throw_error(name, " should be one of ",
+                 .collapse(msg.head, quote = FALSE), msg.tail)
+  idx <- idx.match[idx.match > 0L]
+  choices[idx]
+}
+
+#' @title Validate an argument from a list of classes
+#'
+#' @param arg [character] (**required**): variable to validate.
+#' @param classes [vector] [character] (**required**): a vector of candidate
+#'        classes or types.
+#' @param throw.error [logical] (*with default*): whether an error should be
+#'        thrown in case of failed validation (`TRUE` by default). If `FALSE`,
+#'        the function raises a warning and proceeds.
+#' @param extra [character] (*with default*): additional choice to be reported
+#'        after the valid choices and preceded by "or".
+#' @param name [character] (*with default*): variable name to report in case
+#'        of error: if specified, it's reported as is; if not specified it's
+#'        inferred from the name of the variable tested and reported with
+#'        quotes.
+#'
+#' @return
+#' If `throw.error = TRUE`, the function throws an error and doesn't return
+#' anything. Otherwise, it will return a boolean to indicate whether validation
+#' was successful or not.
+#'
+#' @md
+#' @noRd
+.validate_class <- function(arg, classes, throw.error = TRUE,
+                            name = NULL, extra = NULL) {
+
+  ## name of the argument to report if not specified
+  if (is.null(name))
+    name <- sprintf("'%s'", all.vars(match.call())[1])
+
+  if (missing(arg) || sum(inherits(arg, classes)) == 0L) {
+    ## additional text to append after the valid classes to account for
+    ## extra options that cannot be validated but we want to report
+    classes.extra <- c(sQuote(classes, q = FALSE), extra)
+
+    ## use an 'or' instead of a comma before the last choice
+    if (length(classes.extra) > 1) {
+      msg.head <- head(classes.extra, -1)
+      msg.tail <- paste(" or", tail(classes.extra, 1))
+    } else {
+      msg.head <- classes.extra
+      msg.tail <- NULL
+    }
+    msg <- paste0(name, " should be of class ",
+                  .collapse(msg.head, quote = FALSE), msg.tail)
+    if (throw.error)
+      .throw_error(msg)
+    .throw_warning(msg)
+    return(FALSE)
+  }
+  return(TRUE)
+}
+
+#' @title Validate that a variable is not empty
+#'
+#' @param arg [character] (**required**): variable to validate.
+#' @param what [character] (**required**): the type of the variable, used
+#'        only in the message reported; if not specified it's inferred from
+#'        they type of the variable tested.
+#' @param throw.error [logical] (*with default*): whether an error should be
+#'        thrown in case of failed validation (`TRUE` by default). If `FALSE`,
+#'        the function raises a warning and proceeds.
+#' @param name [character] (*with default*): variable name to report in case
+#'        of error: if specified, it's reported as is; if not specified it's
+#'        inferred from the name of the variable tested and reported with
+#'        quotes.
+#'
+#' @return
+#' If `throw.error = TRUE`, the function throws an error and doesn't return
+#' anything. Otherwise, it will return a boolean to indicate whether validation
+#' was successful or not.
+#'
+#' @md
+#' @noRd
+.validate_not_empty <- function(arg, what = NULL, throw.error = TRUE,
+                                name = NULL) {
+
+  ## type of the argument to report if not specified
+  if (is.null(what))
+    what <- class(arg)[1]
+
+  ## name of the argument to report if not specified
+  if (is.null(name))
+    name <- sprintf("'%s'", all.vars(match.call())[1])
+
+  if (NROW(arg) == 0 || NCOL(arg) == 0) {
+    msg <- paste0(name, " cannot be an empty ", what)
+    if (throw.error)
+      .throw_error(msg)
+    .throw_warning(msg)
+    return(FALSE)
+  }
+  return(TRUE)
+}
+
+#' @title Validate the length of a variable
+#'
+#' @param arg [character] (**required**): variable to validate.
+#' @param exp.length [integer] (**required**): the expected length.
+#' @param throw.error [logical] (*with default*): whether an error should be
+#'        thrown in case of failed validation (`TRUE` by default). If `FALSE`,
+#'        the function raises a warning and proceeds.
+#' @param name [character] (*with default*): variable name to report in case
+#'        of error: if specified, it's reported as is; if not specified it's
+#'        inferred from the name of the variable tested and reported with
+#'        quotes.
+#'
+#' @return
+#' If `throw.error = TRUE`, the function throws an error and doesn't return
+#' anything. Otherwise, it will return a boolean to indicate whether validation
+#' was successful or not.
+#'
+#' @md
+#' @noRd
+.validate_length <- function(arg, exp.length, throw.error = TRUE,
+                            name = NULL) {
+
+  ## name of the argument to report if not specified
+  if (is.null(name))
+    name <- sprintf("'%s'", all.vars(match.call())[1])
+
+  if (length(arg) != exp.length) {
+    msg <- paste0(name, " should have length ", exp.length)
+    if (throw.error)
+      .throw_error(msg)
+    .throw_warning(msg)
+    return(FALSE)
+  }
+  return(TRUE)
+}
+
 #' @title Validate Scalar Variables Expected to be Positive
 #'
 #' @param val [numeric] (**required**): value to validate
@@ -929,20 +1313,186 @@ SW <- function(expr) {
 #'        integer (`FALSE` by default)
 #' @param null.ok [logical] (*with default*): whether a `NULL` value should be
 #'        considered valid (`FALSE` by default)
-#' @param name [character] (*with default*): Variable name to report in case of error; if not specified
-#'        it's inferred from the name of the name of the variable tested
+#' @param name [character] (*with default*): variable name to report in case
+#'        of error: if specified, it's reported as is; if not specified it's
+#'        inferred from the name of the variable tested and reported with
+#'        quotes.
 #'
 #' @md
 #' @noRd
 .validate_positive_scalar <- function(val, int = FALSE, null.ok = FALSE,
                                       name = NULL) {
-  if (is.null(val) && null.ok)
+  if (missing(val) || is.null(val) && null.ok)
     return()
   if (!is.numeric(val) || length(val) != 1 || is.na(val) || val <= 0 ||
       (int && val != as.integer(val))) {
     if (is.null(name))
-      name <- all.vars(match.call())[1]
-    .throw_error("'", name, "' must be a positive ", if (int) "integer ",
-                 "scalar")
+      name <- sprintf("'%s'", all.vars(match.call())[1])
+    .throw_error(name, " should be a positive ", if (int) "integer ",
+                 "scalar", if (null.ok) " or NULL")
   }
 }
+
+#' @title Validate logical scalar variables
+#'
+#' @param val [numeric] (**required**): value to validate
+#' @param null.ok [logical] (*with default*): whether a `NULL` value should be
+#'        considered valid (`FALSE` by default)
+#' @param name [character] (*with default*): variable name to report in case
+#'        of error: if specified, it's reported as is; if not specified it's
+#'        inferred from the name of the variable tested and reported with
+#'        quotes.
+#'
+#' @md
+#' @noRd
+.validate_logical_scalar <- function(val, null.ok = FALSE, name = NULL) {
+  if (missing(val) || is.null(val) && null.ok)
+    return()
+  if (!is.logical(val) || length(val) != 1 || is.na(val)) {
+    if (is.null(name))
+      name <- sprintf("'%s'", all.vars(match.call())[1])
+    .throw_error(name, " should be a single logical value",
+                 if (null.ok) " or NULL")
+  }
+}
+
+#' Check that a suggested package is installed
+#'
+#' Report a message with installation instructions if a suggested package
+#' is not available.
+#'
+#' @param pkg [character] (**required**): name of the package to check.
+#' @param reason [character] (*with default*): subject of the sentence,
+#'        helpful to clarify why the package is being required.
+#' @param throw.error [logical] (*with default*): whether an error should be
+#'        thrown in case of failed validation (`TRUE` by default). If `FALSE`,
+#'        the function raises a warning and proceeds.
+#'
+#' @return
+#' If `throw.error = TRUE`, the function throws an error and doesn't return
+#' anything. Otherwise, it will return a boolean to indicate whether validation
+#' was successful or not.
+#'
+#' @md
+#' @noRd
+.require_suggested_package <- function(pkg, reason = "This function",
+                                       throw.error = TRUE) {
+  if (!requireNamespace(pkg, quietly = TRUE)) {
+    msg <- paste("%s requires the '%s' package: to install it, run",
+                 "`install.packages('%s')` in your R console")
+    msg <- sprintf(msg, reason, pkg, pkg)
+    if (throw.error)
+      .throw_error(msg)
+    .throw_warning(msg)
+    return(FALSE)
+  }
+  return(TRUE)
+}
+
+#' Create a list of objects repeated a given number of times
+#'
+#' @param x [vector] (**required**) An object to be repeated into a list.
+#' @param length [integer] (**required**) Length of the list to be produced.
+#'
+#' @return
+#' A list with the object repeated the specified number of times.
+#'
+#' @md
+#' @noRd
+.listify <- function(x, length) {
+  if (!inherits(x, "list"))
+    x <- list(x)
+  rep(x, length = length)
+}
+
+#' Comma-separated string concatenation
+#'
+#' Collapse the elements of a vector into a comma-separated string, with
+#' the option of quoting each element.
+#'
+#' @param x [vector] A vector of elements to be collapsed into a comma-separated
+#'        string.
+#' @param quote [logical] (*with default*) Whether each element should be
+#'        surrounded by single quotes (`TRUE` by default).
+#'
+#' @return
+#' A comma-separated string where each element of the original vector is
+#' optionally surrounded by single quotes.
+#'
+#' @md
+#' @noRd
+.collapse <- function(x, quote = TRUE) {
+  paste0(if (quote) sQuote(x, FALSE) else x, collapse=", ")
+}
+
+#' Shorten a filename
+#'
+#' Shorten a filename to the given width by cutting out characters from the
+#' middle, leaving the most significant parts.
+#'
+#' @param filename [character] (**required**) A file name; can be a vector
+#' @param max.width [integer] (*with default*) The maximum width available
+#'
+#' @return
+#' A filename not longer than the maximum available width.
+#'
+#' @md
+#' @noRd
+.shorten_filename <- function(filename, max.width = 70) {
+  ## optimised for speed with ChatGPT, 2025
+  ## get length
+  name.len <- nchar(filename, keepNA = FALSE)
+
+  ## logical index of names to shorten
+  to_shorten <- name.len > max.width
+
+  ## get out if nothing needs shortening
+  if (!any(to_shorten))
+    return(filename)
+
+  # calculate parts
+  dots <- "..."  # \u2026
+  max.width <- max.width - nchar(dots)
+  part1_last <- floor(max.width / 2)
+  part2_first <- floor(name.len[to_shorten] - max.width / 2) + 1
+
+  # shorten what needs to be shortened
+  filename[to_shorten] <- paste0(
+    substring(filename[to_shorten], 1, part1_last),
+    dots,
+    substring(filename[to_shorten], part2_first)
+  )
+
+  ## return
+  filename
+}
+#'@title Affine Transformation of Values
+#'
+#'@description Affine (linear) transformation of values; which we use
+#'a couple of times within functions
+#'
+#'@param x [numeric] (*required*): values to be transformed
+#'
+#'@param range_old [numeric] (*required*): original scale limits
+#'
+#'@param range_new [numeric] (*required*): new scale limits
+#'
+#'@returns rescaled values values
+#'
+#'@examples
+#'
+#' x <- stats::rnorm(100)
+#' y_dens <- stats::density(x)
+#' .rescale(
+#'  x = y_dens$y,
+#'  range_old = c(min(y_dens$y), max(y_dens$y)),
+#'  range_new = c(0,20))
+#'
+#'@md
+#'@noRd
+.rescale <- function(x, range_old, range_new) {
+  range_new[1] +
+    (x - range_old[1]) * (range_new[2] - range_new[1]) /
+    (range_old[2] - range_old[1])
+
+}
