@@ -1,11 +1,13 @@
 #' @title Visualise dose recovery test results
 #'
-#' @description The function provides a standardised plot output for dose recovery test
+#' @description
+#' The function provides a standardised plot output for dose recovery test
 #' measurements.
 #'
-#' @details Procedure to test the accuracy of a measurement protocol to reliably
+#' @details
+#' The procedure tests the accuracy of a measurement protocol to reliably
 #' determine the dose of a specific sample. Here, the natural signal is erased
-#' and a known laboratory dose administered which is treated as unknown. Then
+#' and a known laboratory dose administered, which is treated as unknown. Then
 #' the De measurement is carried out and the degree of congruence between
 #' administered and recovered dose is a measure of the protocol's accuracy for
 #' this sample.\cr
@@ -18,27 +20,27 @@
 #'
 #' @param given.dose [numeric] (*optional*):
 #' given dose used for the dose recovery test to normalise data.
-#' If only one given dose is provided this given dose is valid for all input
-#' data sets (i.e., `values` is a list).  Otherwise a given dose for each input
+#' If only one given dose is provided, this given dose is valid for all input
+#' data sets (i.e., `values` is a list). Otherwise, a given dose for each input
 #' data set has to be provided (e.g., `given.dose = c(100,200)`).
-#' If `given.dose` in `NULL` the values are plotted without normalisation
+#' If `given.dose` is `NULL` or 0, the values are plotted without normalisation
 #' (might be useful for preheat plateau tests).
 #' **Note:** Unit has to be the same as from the input values (e.g., Seconds or
 #' Gray).
 #'
-#' @param error.range [numeric]:
+#' @param error.range [numeric] (*with default*):
 #' symmetric error range in percent will be shown as dashed lines in the plot.
-#' Set `error.range` to 0 to void plotting of error ranges.
+#' It can be set to 0 to remove the error ranges.
 #'
-#' @param preheat [numeric]:
+#' @param preheat [numeric] (*optional*):
 #' optional vector of preheat temperatures to be used for grouping the De values.
 #' If specified, the temperatures are assigned to the x-axis.
 #'
-#' @param boxplot [logical]:
-#' optionally plot values, that are grouped by preheat temperature as boxplots.
-#'  Only possible when `preheat` vector is specified.
+#' @param boxplot [logical] (*with default*):
+#' plot values that are grouped by preheat temperature as boxplots.
+#' Only possible when `preheat` vector is specified.
 #'
-#' @param mtext [character]:
+#' @param mtext [character] (*with default*):
 #' additional text below the plot title.
 #'
 #' @param summary [character] (*optional*):
@@ -72,11 +74,12 @@
 #' column and one row. If `par.local = FALSE`, global parameters are inherited,
 #' i.e. parameters provided via `par()` work
 #'
-#' @param na.rm [logical]: indicating whether `NA` values are
+#' @param na.rm [logical] (*with default*): indicating whether `NA` values are
 #' removed before plotting from the input data set
 #'
-#' @param ... further arguments and graphical parameters passed to [plot], supported are:
-#' `xlab`, `ylab`, `xlim`, `ylim`, `main`, `cex`, `las` and `pch``
+#' @param ... further arguments and graphical parameters passed to [plot],
+#' supported are:
+#' `xlab`, `ylab`, `xlim`, `ylim`, `main`, `cex`, `las` and `pch`.
 #'
 #' @return A plot is returned.
 #'
@@ -184,17 +187,21 @@ plot_DRTResults <- function(
   error.range = 10,
   preheat,
   boxplot = FALSE,
-  mtext,
-  summary,
-  summary.pos,
+  mtext = "",
+  summary = "",
+  summary.pos = "topleft",
   legend,
-  legend.pos,
+  legend.pos = "topright",
   par.local = TRUE,
   na.rm  = FALSE,
   ...
-){
+) {
+  .set_function_name("plot_DRTResults")
+  on.exit(.unset_function_name(), add = TRUE)
 
-  ## Validity checks ----------------------------------------------------------
+  ## Integrity checks -------------------------------------------------------
+
+  .validate_not_empty(values)
 
   ##avoid crash for wrongly set boxlot argument
   if(missing(preheat) & boxplot == TRUE){
@@ -203,32 +210,46 @@ plot_DRTResults <- function(
                    "reset to FALSE")
   }
 
-  if(missing(summary) == TRUE) {summary <- NULL}
-  if(missing(summary.pos) == TRUE) {summary.pos <- "topleft"}
-  if(missing(legend.pos) == TRUE) {legend.pos <- "topright"}
-  if(missing(mtext) == TRUE) {mtext <- ""}
+  .validate_class(summary, "character")
+  .validate_class(summary.pos, c("numeric", "character"))
+  if (is.numeric(summary.pos)) {
+    .validate_length(summary.pos, 2)
+  }
+  else {
+    .validate_args(summary.pos, c("sub", "left", "center", "right",
+                                  "topleft", "top", "topright",
+                                  "bottomleft", "bottom", "bottomright"))
+  }
 
   ## Homogenise and check input data
-  if(is(values, "list") == FALSE) {values <- list(values)}
+  if (!inherits(values, "list"))
+      values <- list(values)
 
-  for(i in 1:length(values)) {
-    if(is(values[[i]], "RLum.Results")==FALSE &
-         is(values[[i]], "data.frame")==FALSE){
-      .throw_error("Input data must be one of 'data.frame' or 'RLum.Results'")
-    } else {
-      if(is(values[[i]], "RLum.Results")==TRUE){
-        values[[i]] <- get_RLum(values[[i]])[,1:2]
-      }
+  for (i in seq_along(values)) {
+    .validate_class(values[[i]], c("data.frame", "RLum.Results"),
+                    name = "'values'")
+    if (inherits(values[[i]], "RLum.Results")) {
+      val <- get_RLum(values[[i]])[, 1:2]
+      if (is.null(val))
+        val <- NA
+      values[[i]] <- val
     }
   }
 
+  ## remove invalid records
+  values[is.na(values)] <- NULL
+  if (length(values) == 0) {
+    .throw_error("No valid records in 'values'")
+  }
+
   ## Check input arguments ----------------------------------------------------
-  for(i in 1:length(values)) {
+  for (i in seq_along(values)) {
 
     ##check for preheat temperature values
     if(missing(preheat) == FALSE) {
-      if(length(preheat) != nrow(values[[i]])){
-        .throw_error("Number of preheat temperatures != De values")
+      if (length(preheat) < nrow(values[[i]])) {
+        .throw_error("'preheat' should have length equal to the number ",
+                     "of De values")
       }
     }
 
@@ -237,13 +258,9 @@ plot_DRTResults <- function(
 
       ##currently we assume that all input data sets comprise a similar of data
       if(!missing(preheat) & i == length(values)){
-
-        ## find and mark NA value indices
-        temp.NA.values <- unique(c(which(is.na(values[[i]][,1])), which(is.na(values[[i]][,2]))))
-
-        ##remove preheat entries
-        preheat <- preheat[-temp.NA.values]
-
+        ## remove preheat entries corresponding to NA values
+        preheat <- preheat[!is.na(values[[i]][, 1]) &
+                           !is.na(values[[i]][, 2])]
       }
 
       values[[i]] <- na.exclude(values[[i]])
@@ -253,7 +270,7 @@ plot_DRTResults <- function(
   ## create global data set
   values.global <- NULL
   n.values <- NULL
-  for(i in 1:length(values)) {
+  for (i in seq_along(values)) {
     values.global <- rbind(values.global, values[[i]])
     n.values <- c(n.values, nrow(values[[i]]))
   }
@@ -269,12 +286,11 @@ plot_DRTResults <- function(
   }
 
   ylab <- if("ylab" %in% names(extraArgs)) {extraArgs$ylab} else
-  {if(!is.null(given.dose)){
-    expression(paste("Normalised ", D[e], sep=""))
-  }else{expression(paste(D[e], " [s]"), sep = "")}}
+  { if (!is.null(given.dose) && length(given.dose) > 0 && given.dose[1] > 0) expression(paste("Normalised ", D[e], sep = ""))
+    else expression(paste(D[e], " [s]", sep = "")) }
 
   xlim <- if("xlim" %in% names(extraArgs)) {extraArgs$xlim} else
-  {c(1, max(n.values) + 1)}
+  { c(0, max(n.values)) + 0.5 }
 
   ylim <- if("ylim" %in% names(extraArgs)) {extraArgs$ylim} else
   {c(0.75, 1.25)} #check below for further corrections if boundaries exceed set range
@@ -293,25 +309,23 @@ plot_DRTResults <- function(
   ## calculations and settings-------------------------------------------------
 
   ## normalise data if given.dose is given
-  if(!is.null(given.dose)){
-
-    if(length(given.dose) > 1){
-
-      if(length(values) < length(given.dose)){
-        .throw_error("'given.dose' > number of input data sets")
+  if (!is.null(given.dose)) {
+    .validate_class(given.dose, "numeric")
+    .validate_not_empty(given.dose)
+    if (length(given.dose) == 1) {
+      given.dose <- rep(given.dose, length(values))
+    }
+    else if (length(given.dose) != length(values)) {
+      .throw_error("'given.dose' should have length equal to the number ",
+                   "of input data sets")
+    }
 
+    if (all(given.dose > 0)) {
+      for (i in 1:length(values)) {
+        values[[i]] <- values[[i]] / given.dose[i]
       }
-
-      for(i in 1:length(values)) {
-        values[[i]] <- values[[i]]/given.dose[i]
-      }
-
-    }else{
-
-      for(i in 1:length(values)) {
-        values[[i]] <- values[[i]]/given.dose
-      }
-
+    } else {
+      given.dose <- NULL
     }
   }
 
@@ -325,15 +339,12 @@ plot_DRTResults <- function(
         min(values[[x]][,1], na.rm = TRUE) - max(values[[x]][,2], na.rm = TRUE)})),
       max(sapply(1:length(values), function(x){
         max(values[[x]][,1], na.rm = TRUE) + max(values[[x]][,2], na.rm = TRUE)})))
-
   }
 
-
   ## optionally group data by preheat temperature
-  if(missing(preheat) == FALSE) {
-    modes <- as.numeric(rownames(as.matrix(table(preheat))))
-    values.preheat <- list(NA)
-    values.boxplot <- list(NA)
+  if (!missing(preheat)) {
+    modes <- as.numeric(names(table(preheat)))
+    values.preheat <- values.boxplot <- list()
     for(i in 1:length(modes)) {
       for(j in 1:length(values)) {
         values.preheat[[length(values.preheat) + 1]] <-
@@ -344,15 +355,15 @@ plot_DRTResults <- function(
       }
       j <- 1
     }
-    values.preheat[[1]] <- NULL
-    values.boxplot[[1]] <- NULL
     modes.plot <- rep(modes, each = length(values))
-
+    xlim <- c(min(modes.plot) * 0.9, max(modes.plot) * 1.1)
   } else {
     modes <- 1
-
   }
 
+  if (boxplot)
+    xlim <- c(0.5, length(unique(preheat)) + 0.5)
+
   ## assign colour indices
   col <- if("col" %in% names(extraArgs)) {extraArgs$col} else {
     if(missing(preheat) == TRUE) {
@@ -373,7 +384,6 @@ plot_DRTResults <- function(
         prefix = paste(rep("\n", (i - 1) * length(summary)), collapse = "")
       )
     })
-
   }else{
     label.text <- lapply(1:length(values), function(i) {
       .create_StatisticalSummaryText(
@@ -383,94 +393,32 @@ plot_DRTResults <- function(
         sep = " | "
       )
     })
-
   }
 
-  ## convert keywords into summary placement coordinates
-  if(missing(summary.pos) == TRUE) {
-    summary.pos <- c(xlim[1], ylim[2])
-    summary.adj <- c(0, 1)
-  } else if(length(summary.pos) == 2) {
-    summary.pos <- summary.pos
-    summary.adj <- c(0, 1)
-  } else if(summary.pos[1] == "topleft") {
-    summary.pos <- c(xlim[1], ylim[2])
-    summary.adj <- c(0, 1)
-  } else if(summary.pos[1] == "top") {
-    summary.pos <- c(mean(xlim), ylim[2])
-    summary.adj <- c(0.5, 1)
-  } else if(summary.pos[1] == "topright") {
-    summary.pos <- c(xlim[2], ylim[2])
-    summary.adj <- c(1, 1)
-  }  else if(summary.pos[1] == "left") {
-    summary.pos <- c(xlim[1], mean(ylim))
-    summary.adj <- c(0, 0.5)
-  } else if(summary.pos[1] == "center") {
-    summary.pos <- c(mean(xlim), mean(ylim))
-    summary.adj <- c(0.5, 0.5)
-  } else if(summary.pos[1] == "right") {
-    summary.pos <- c(xlim[2], mean(ylim))
-    summary.adj <- c(1, 0.5)
-  }else if(summary.pos[1] == "bottomleft") {
-    summary.pos <- c(xlim[1], ylim[1])
-    summary.adj <- c(0, 0)
-  } else if(summary.pos[1] == "bottom") {
-    summary.pos <- c(mean(xlim), ylim[1])
-    summary.adj <- c(0.5, 0)
-  } else if(summary.pos[1] == "bottomright") {
-    summary.pos <- c(xlim[2], ylim[1])
-    summary.adj <- c(1, 0)
-  }
+  ## keep track if the summary is in the bottom row as we may need to compute
+  ## an adjustment further down, after the plot device has been opened
+  summary.pos_is_bottom <- grepl("bottom", summary.pos[1])
 
-  ## convert keywords into legend placement coordinates
-  if(missing(legend.pos) == TRUE) {
-    legend.pos <- c(xlim[2], ylim[2])
-    legend.adj <- c(1, 1)
-  } else if(length(legend.pos) == 2) {
-    legend.pos <- legend.pos
-    legend.adj <- c(0, 1)
-  } else if(legend.pos[1] == "topleft") {
-    legend.pos <- c(xlim[1], ylim[2])
-    legend.adj <- c(0, 1)
-  } else if(legend.pos[1] == "top") {
-    legend.pos <- c(mean(xlim), ylim[2])
-    legend.adj <- c(0.5, 1)
-  } else if(legend.pos[1] == "topright") {
-    legend.pos <- c(xlim[2], ylim[2])
-    legend.adj <- c(1, 1)
-  } else if(legend.pos[1] == "left") {
-    legend.pos <- c(xlim[1], mean(ylim))
-    legend.adj <- c(0, 0.5)
-  } else if(legend.pos[1] == "center") {
-    legend.pos <- c(mean(xlim), mean(ylim))
-    legend.adj <- c(0.5, 0.5)
-  } else if(legend.pos[1] == "right") {
-    legend.pos <- c(xlim[2], mean(ylim))
-    legend.adj <- c(1, 0.5)
-  } else if(legend.pos[1] == "bottomleft") {
-    legend.pos <- c(xlim[1], ylim[1])
-    legend.adj <- c(0, 0)
-  } else if(legend.pos[1] == "bottom") {
-    legend.pos <- c(mean(xlim), ylim[1])
-    legend.adj <- c(0.5, 0)
-  } else if(legend.pos[1] == "bottomright") {
-    legend.pos <- c(xlim[2], ylim[1])
-    legend.adj <- c(1, 0)
-  }
+  ## convert keywords into summary and legend placement coordinates
+  coords <- .get_keyword_coordinates(summary.pos, xlim, ylim)
+  summary.pos <- coords$pos
+  summary.adj <- c(coords$adj[1], 1) # always top-aligned
+  coords <- .get_keyword_coordinates(legend.pos, xlim, ylim)
+  legend.pos <- coords$pos
+  legend.adj <- coords$adj
 
-  ## Plot output --------------------------------------------------------------
+  ## Plot output ------------------------------------------------------------
 
-  ## determine number of subheader lines to shif the plot
-  shift.lines <- if(summary.pos[1] == "sub" & mtext == "") {
-    length(label.text) - 1
-  } else {1}
+  ## determine number of subheader lines to shift the plot
+  shift.lines <- if (mtext == "") {
+                   if (summary.pos[1] == "sub") length(label.text) else 0
+                 } else 1
 
   ## setup plot area
   if(par.local){
-    if (shift.lines <= 0)
-    shift.lines <- 1
-    par.default <- par(mfrow = c(1, 1), cex = cex, oma = c(0, 1, shift.lines - 1, 1))
-    on.exit(par(par.default))
+    par.default <- par(mfrow = c(1, 1), cex = cex,
+                       mar = c(2.5, 2.5, shift.lines, 0) + 2.1)
+    on.exit(par(par.default), add = TRUE)
   }
 
   ## optionally plot values and error bars
@@ -492,7 +440,7 @@ plot_DRTResults <- function(
 
       ## add title
       title(main = main,
-            line = shift.lines + 2)
+            line = shift.lines + 0.5)
 
       ## add additional lines
       if (!is.null(given.dose)) {
@@ -519,41 +467,50 @@ plot_DRTResults <- function(
             cex = 0.8
           )
         }
-
       }
 
+      ## allow assigning a separate colour to each point, but only if there
+      ## is one input dataset
+      oneinput <- length(values) == 1
+      multicol <- oneinput && nrow(values[[1]]) == length(col)
+
       ## add data and error bars
       for(i in 1:length(values)) {
 
         points(x = c(1:nrow(values[[i]])),
                y = values[[i]][,1],
-               pch = if(nrow(values[[i]]) == length(pch)){ pch } else { pch[i] },
-               col = if(nrow(values[[i]]) == length(col)){ col } else { col[i] },
+               pch = if (oneinput && nrow(values[[i]]) == length(pch)) pch else pch[i],
+               col = if (multicol) col else col[i],
                cex = 1.2 * cex)
 
-        arrows(c(1:nrow(values[[i]])),
+        graphics::arrows(c(1:nrow(values[[i]])),
                values[[i]][,1] + values[[i]][,2],
                c(1:nrow(values[[i]])),
                values[[i]][,1] - values[[i]][,2],
                angle = 90,
                length = 0.075,
                code = 3,
-               col = if(nrow(values[[i]]) == length(col)){ col } else { col[i] })
+               col = if (multicol) col else col[i])
 
         ## add summary content
         if(summary.pos[1] != "sub") {
+          vadj <- 0
+          if (summary.pos_is_bottom) {
+            ## adjust the vertical coordinate by the height of the longest label
+            vadj <- graphics::strheight(tail(label.text, 1), cex = 0.8)
+          }
           text(x = summary.pos[1],
-               y = summary.pos[2],
+               y = summary.pos[2] + vadj,
                adj = summary.adj,
                labels = label.text[[i]],
-               cex = 0.8 * cex,
-               col = if(nrow(values[[i]]) == length(col)){ "black" } else { col[i] })
+               cex = 0.8,
+               col = if (multicol) "black" else col[i])
         } else {
           if(mtext == "") {
             mtext(side = 3,
-                  line = - i + 2.5,
+                  line = shift.lines - i,
                   text = label.text[[i]],
-                  col = if(nrow(values[[i]]) == length(col)){ "black" } else { col[i] },
+                  col = if (multicol) "black" else col[i],
                   cex = cex * 0.8)
           }
         }
@@ -563,7 +520,7 @@ plot_DRTResults <- function(
       ## option for provided preheat data
       ## create empty plot
       plot(NA,NA,
-           xlim = c(min(modes.plot) * 0.9, max(modes.plot) * 1.1),
+           xlim = xlim,
            ylim = ylim,
            xlab = xlab,
            ylab = ylab,
@@ -580,7 +537,7 @@ plot_DRTResults <- function(
 
       ## add title
       title(main = main,
-            line = shift.lines + 2)
+            line = shift.lines + 0.5)
 
       ## add additional lines
       if (!is.null(given.dose)) {
@@ -617,7 +574,7 @@ plot_DRTResults <- function(
                col = col[i],
                cex = 1.2 * cex)
 
-        arrows(values.preheat[[i]][,3],
+        graphics::arrows(values.preheat[[i]][,3],
                values.preheat[[i]][,1] + values.preheat[[i]][,2],
                values.preheat[[i]][,3],
                values.preheat[[i]][,1] - values.preheat[[i]][,2],
@@ -632,7 +589,7 @@ plot_DRTResults <- function(
   ## optionally, plot boxplot
   if(boxplot) {
     ## create empty plot
-    boxplot(values.boxplot,
+    graphics::boxplot(values.boxplot,
             names = modes.plot,
             ylim = ylim,
             xlab = xlab,
@@ -643,10 +600,10 @@ plot_DRTResults <- function(
             border = col)
 
     ## add axis label, if necessary
-    if (length(modes.plot) == 1) {
-      axis(side = 1, at = 1, labels = modes.plot, las = las)
-
-    } else if (length(modes.plot) > length(unique(modes.plot))){
+    if (length(modes.plot) <= length(unique(modes.plot))) {
+      axis(side = 1, at = 1:length(unique(modes.plot)),
+           labels = unique(modes.plot), las = las)
+    } else {
       ticks <- seq(from = 1 + ((length(values.boxplot)/length(unique(modes.plot)) - 1)/2),
                    to = length(values.boxplot),
                    by = length(values.boxplot)/length(unique(modes.plot)))
@@ -664,6 +621,8 @@ plot_DRTResults <- function(
       )
 
       polygon.step <- unique(diff(polygon.x) - 1)
+      if (length(polygon.step) == 0)
+        polygon.step <- 1
 
       for (x.plyg in polygon.x) {
         polygon(
@@ -676,17 +635,12 @@ plot_DRTResults <- function(
           ),
           col = "grey",
           border = "grey")
-
       }
-
-    }else{
-      axis(side = 1, at = 1:length(unique(modes.plot)), labels = unique(modes.plot), las = las)
-
     }
 
     ## add title
     title(main = main,
-          line = shift.lines + 2)
+          line = shift.lines + 0.5)
 
     ## add additional lines
     if(!is.null(given.dose)){
@@ -709,16 +663,21 @@ plot_DRTResults <- function(
     for(i in 1:length(values)) {
       ## add summary content
       if(summary.pos[1] != "sub") {
+        vadj <- 0
+        if (summary.pos_is_bottom) {
+          ## adjust the vertical coordinate by the height of the longest label
+          vadj <- graphics::strheight(tail(label.text, 1), cex = 0.8)
+        }
         text(x = summary.pos[1],
-             y = summary.pos[2],
+             y = summary.pos[2] + vadj,
              adj = summary.adj,
              labels = label.text[[i]],
-             cex = 0.8 * cex,
+             cex = 0.8,
              col = if(nrow(values[[i]]) == length(col)){ "black" } else { col[i] })
       } else {
         if(mtext == "") {
           mtext(side = 3,
-                line = - i + 2.5,
+                line = shift.lines - i,
                 text = label.text[[i]],
                 col = if(nrow(values[[i]]) == length(col)){ "black" } else { col[i] },
                 cex = cex * 0.8)
@@ -737,12 +696,11 @@ plot_DRTResults <- function(
            col = unique(col),
            pch = unique(pch),
            lty = 1,
-           cex = cex * 0.8)
+           cex = 0.8)
   }
 
   ## optionally add subheader text
   mtext(side = 3,
-        line = shift.lines,
         text = mtext,
         cex = 0.8 * cex)
 
