@@ -19,8 +19,9 @@
 #'
 #' **analyse_function.control**
 #'
-#' The argument `analyse_function.control` currently supports the following arguments
-#' `sequence.structure`, `dose.points`, `mtext.outer`, `fit.method`, `fit.force_through_origin`, `plot`, `plot.single`
+#' The argument `analyse_function.control` currently supports the following arguments:
+#' `sequence.structure`, `dose.points`, `mtext.outer`, `fit.method`,
+#' `fit.force_through_origin`, `plot`, `plot_singlePanels`
 #'
 #' @param object [RLum.Analysis-class] (**required**): input object containing data for analysis
 #' Can be provided as a [list] of such objects.
@@ -62,20 +63,20 @@
 #' integral.
 #'
 #' @param show_ShineDownCurve [logical] (*with default*):
-#' enables or disables shine down curve in the plot output
+#' enable/disable shine down curve in the plot output.
 #'
 #' @param respect_RC.Status [logical] (*with default*):
-#'  remove De-values with 'FAILED' RC.Status from the plot
-#'  (cf. [analyse_SAR.CWOSL] and [analyse_pIRIRSequence])
+#' remove De values with 'FAILED' RC.Status from the plot (cf. [analyse_SAR.CWOSL]
+#' and [analyse_pIRIRSequence]).
 #'
 #' @param verbose [logical] (*with default*):
-#' enables or disables terminal feedback
+#' enable/disable output to the terminal.
 #'
-#' @param multicore [logical] (*with default*) : enables/disables multi core
+#' @param multicore [logical] (*with default*) : enable/disable multi core
 #' calculation if `object` is a [list] of [RLum.Analysis-class] objects. Can be an
 #' [integer] specifying the number of cores
 #'
-#' @param plot [logical] (*with default*): enables/disables plot output
+#' @param plot [logical] (*with default*): enable/disable the plot output.
 #' Disabling the plot is useful in cases where the output need to be processed
 #' differently.
 #'
@@ -110,7 +111,7 @@
 #' It means, that every sequence should be checked carefully before running long
 #' calculations using several hundreds of channels.
 #'
-#' @section Function version: 0.1.7
+#' @section Function version: 0.1.8
 #'
 #' @author Sebastian Kreutzer, Institute of Geography, Ruprecht-Karl University of Heidelberg (Germany)
 #'
@@ -173,38 +174,41 @@ plot_DetPlot <- function(
    f_def <- sys.call(sys.parent(n = -1))
    args_default <- as.list(f_def)[-(1:2)]
 
-   ## detect cores
-   cores <- if(inherits(multicore[1], "logical") && multicore[1]) {
-     parallel::detectCores()
-    } else {
-      max(c(as.numeric(multicore[1])), 1)
+    ## detect cores
+    .validate_class(multicore, c("logical", "numeric", "integer"))
+    .validate_length(multicore, 1)
+    cores <- if (is.logical(multicore) && multicore) {
+               parallel::detectCores() # nocov
+             } else {
+               max(multicore, 1)
+             }
+
+    ## function that calls plot_DetPlot() on each element of the list
+    nested.fun <- function(x) {
+      do.call(plot_DetPlot, c(list(object = x), args_default))
     }
 
-   cl <- parallel::makeCluster(cores)
-   on.exit(parallel::stopCluster(cl), add = TRUE)
+    if (cores > 1) {
+      ## run in parallel
+      cl <- parallel::makeCluster(cores)
+      on.exit(parallel::stopCluster(cl), add = TRUE)
 
-   if(!multicore) {
-     cores <- parallel::detectCores()
-     cl <- parallel::makeCluster(cores)
-   }
-
-   ##terminal return
-   if(verbose) cat("\n[plot_DetPlot()] Running mulitcore session with", cores, "core(s) ...")
-
-   ## run in parallel
-   return_list <- parallel::parLapply(
-     cl = cl,
-     X = object,
-     fun = function(x, arg = args_default) do.call(plot_DetPlot, c(list(object = x), arg)))
-
-   return(merge_RLum(return_list))
+      if (verbose)
+        message("\n[plot_DetPlot()] Running multicore session using ", cores,
+                " cores ...")
+      res_list <- parallel::parLapply(cl = cl, X = object, fun = nested.fun)
+    } else {
+      ## run in serial
+      res_list <- lapply(object, nested.fun)
+    }
 
+    return(merge_RLum(res_list))
   }
 
-# Integrity Tests -----------------------------------------------------------------------------
-  ##check input
-  if(!inherits(object, "RLum.Analysis"))
-    .throw_error("Input must be an 'RLum.Analysis' object")
+  ## Integrity checks -------------------------------------------------------
+
+  .validate_class(object, "RLum.Analysis")
+  .validate_not_empty(object)
 
   ##get structure
   object.structure <- structure_RLum(object)
@@ -220,6 +224,17 @@ plot_DetPlot <- function(
   .validate_positive_scalar(background.integral.min, int = TRUE)
   .validate_positive_scalar(background.integral.min, int = TRUE)
 
+  ## analyse_function
+  analyse_function <- .validate_args(analyse_function,
+                                     c("analyse_SAR.CWOSL", "analyse_pIRIRSequence"))
+
+  ## deprecated argument
+  if ("plot.single" %in% names(list(...))) {
+    plot_singlePanels <- list(...)$plot.single
+    .throw_warning("'plot.single' is deprecated, use 'plot_singlePanels' ",
+                   "instead")
+  }
+
 # Set parameters ------------------------------------------------------------------------------
   ##set n.channels
   if(is.null(n.channels)){
@@ -239,7 +254,7 @@ plot_DetPlot <- function(
      fit.force_through_origin = FALSE,
      trim_channels = FALSE,
      plot = FALSE,
-     plot.single = FALSE
+     plot_singlePanels = FALSE
   )
 
   analyse_function.settings <- modifyList(analyse_function.settings, analyse_function.control)
@@ -266,7 +281,7 @@ plot_DetPlot <- function(
         fit.method = analyse_function.settings$fit.method,
         trim_channels = analyse_function.settings$trim_channels,
         plot = analyse_function.settings$plot,
-        plot.single = analyse_function.settings$plot.single,
+        plot_singlePanels = analyse_function.settings$plot_singlePanels,
         verbose = verbose
       )
     }))
@@ -283,7 +298,7 @@ plot_DetPlot <- function(
         dose.points = analyse_function.settings$dose.points,
         mtext.outer = analyse_function.settings$mtext.outer,
         plot = analyse_function.settings$plot,
-        plot.single = analyse_function.settings$plot.single,
+        plot_singlePanels = analyse_function.settings$plot_singlePanels,
         sequence.structure = analyse_function.settings$sequence.structure,
         verbose = verbose
       )
@@ -302,9 +317,6 @@ plot_DetPlot <- function(
     }
     rm(result.temp.list)
   }
-  else{
-   .throw_error("Unknown 'analyse_function'")
-  }
 
 
 # Plot ----------------------------------------------------------------------------------------
@@ -314,7 +326,6 @@ plot_DetPlot <- function(
 
     }else{
       pIRIR_signals <- NA
-
     }
 
     ##run this in a loop to account for pIRIR data
@@ -424,11 +435,9 @@ plot_DetPlot <- function(
       } ## end plot
       ##set return
       return(df_final)
-
     })
 
 
-
 # Return ------------------------------------------------------------------
   ##merge results
   return(set_RLum(
@@ -440,5 +449,4 @@ plot_DetPlot <- function(
     info = list(
       call = sys.call())
   ))
-
 }
