@@ -1,5 +1,6 @@
-#' Plot kernel density estimate with statistics
+#' @title Plot kernel density estimate with statistics
 #'
+#' @description
 #' Plot a kernel density estimate of measurement values in combination with the
 #' actual values and associated error bars in ascending order. If enabled, the
 #' boxplot will show the usual distribution parameters (median as
@@ -31,17 +32,16 @@
 #' - `"skewness"` (skewness)
 #'
 #'
-#' **Note** that the input data for the statistic summary is sent to the function
-#' `calc_Statistics()` depending on the log-option for the z-scale. If
+#' **Note** that the input data for the statistic summary is sent to function
+#' [calc_Statistics] depending on the log-option for the z-scale. If
 #' `"log.z = TRUE"`, the summary is based on the logarithms of the input
-#' data. If `"log.z = FALSE"` the linearly scaled data is used.
+#' data. If `"log.z = FALSE"` the linearly-scaled data is used.
 #'
 #' **Note** as well, that `"calc_Statistics()"` calculates these statistic
 #' measures in three different ways: `unweighted`, `weighted` and
 #' `MCM-based` (i.e., based on Monte Carlo Methods). By default, the
-#' MCM-based version is used. If you wish to use another method, indicate this
-#' with the appropriate keyword using the argument `summary.method`.
-#'
+#' MCM-based version is used. This can be controlled via the `summary.method`
+#' argument.
 #'
 #' @param data [data.frame], [vector] or [RLum.Results-class] object (**required**):
 #' for `data.frame`: either two columns: De (`values[,1]`) and De error
@@ -57,7 +57,7 @@
 #' @param values.cumulative [logical] (*with default*):
 #' show cumulative individual data.
 #'
-#' @param order [logical]:
+#' @param order [logical] (*with default*):
 #' Order data in ascending order.
 #'
 #' @param boxplot [logical] (*with default*):
@@ -68,7 +68,7 @@
 #' @param rug [logical] (*with default*):
 #' optionally add rug.
 #'
-#' @param summary [character] (*optional*):
+#' @param summary [character] (*with default*):
 #' add statistic measures of centrality and dispersion to the plot. Can be one
 #' or more of several keywords. See details for available keywords.
 #'
@@ -81,16 +81,12 @@
 #'
 #' @param summary.method [character] (*with default*):
 #' keyword indicating the method used to calculate the statistic summary.
-#' One out of `"unweighted"`, `"weighted"` and `"MCM"`.
+#' One out of `"MCM"` (default), `"weighted"` or `"unweighted"`.
 #' See [calc_Statistics] for details.
 #'
 #' @param bw [character] (*with default*):
 #' bin-width, chose a numeric value for manual setting.
 #'
-#' @param output [logical]:
-#' Optional output of numerical plot parameters. These can be useful to
-#' reproduce similar plots. Default is `TRUE`.
-#'
 #' @param ... further arguments and graphical parameters passed to [plot].
 #'
 #' @note
@@ -101,7 +97,7 @@
 #'
 #' @author
 #' Michael Dietze, GFZ Potsdam (Germany)\cr
-#' Geography & Earth Sciences, Aberystwyth University (United Kingdom)
+#' Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
 #'
 #' @seealso [density], [plot]
 #'
@@ -110,7 +106,7 @@
 #' ## read example data set
 #' data(ExampleData.DeValues, envir = environment())
 #' ExampleData.DeValues <-
-#'   Second2Gray(ExampleData.DeValues$BT998, c(0.0438,0.0019))
+#'   convert_Second2Gray(ExampleData.DeValues$BT998, c(0.0438,0.0019))
 #'
 #' ## create plot straightforward
 #' plot_KDE(data = ExampleData.DeValues)
@@ -156,8 +152,7 @@
 #'
 #' ## example of how to use the numerical output of the function
 #' ## return plot output to draw a thicker KDE line
-#' KDE_out <- plot_KDE(data = ExampleData.DeValues,
-#' output = TRUE)
+#' KDE_out <- plot_KDE(data = ExampleData.DeValues)
 #'
 #' @md
 #' @export
@@ -168,41 +163,16 @@ plot_KDE <- function(
   order = TRUE,
   boxplot = TRUE,
   rug = TRUE,
-  summary,
-  summary.pos,
+  summary = "",
+  summary.pos = "sub",
   summary.method = "MCM",
   bw = "nrd0",
-  output = TRUE,
   ...
 ) {
   .set_function_name("plot_KDE")
   on.exit(.unset_function_name(), add = TRUE)
 
-  ## check data and parameter consistency -------------------------------------
-
-  ## account for depreciated arguments
-  if("centrality" %in% names(list(...))) {
-    boxplot <- TRUE
-    .throw_warning("Argument 'centrality' no longer supported. ",
-                   "Replaced by 'boxplot = TRUE'.")
-  }
-
-  if("dispersion" %in% names(list(...))) {
-    boxplot <- TRUE
-    .throw_warning("Argument 'dispersion' no longer supported. ",
-                   "Replaced by 'boxplot = TRUE'.")
-  }
-
-  if("polygon.col" %in% names(list(...))) {
-    boxplot <- TRUE
-    .throw_warning("Argument 'polygon.col' no longer supported. ",
-                   "Replaced by 'boxplot = TRUE'.")
-  }
-
-  if("weights" %in% names(list(...))) {
-    .throw_warning("Argument 'weights' no longer supported. ",
-                   "Weights are omitted.")
-  }
+  ## Integrity checks -------------------------------------------------------
 
   if (is(data, "list") && length(data) == 0) {
     .throw_error("'data' is an empty list")
@@ -215,17 +185,13 @@ plot_KDE <- function(
 
   ## check/adjust input data structure
   for(i in 1:length(data)) {
-    if(is(data[[i]], "RLum.Results") == FALSE &
-         is(data[[i]], "data.frame") == FALSE &
-         is.numeric(data[[i]]) == FALSE) {
-      .throw_error("Input data must be one of 'data.frame', ",
-                   "'RLum.Results' or 'numeric'")
-    } else {
+    .validate_class(data[[i]], c("RLum.Results", "data.frame", "numeric"),
+                    name = "'data'")
 
-      ##extract RLum.Results
-      if(is(data[[i]], "RLum.Results") == TRUE) {
-        data[[i]] <- get_RLum(data[[i]], "data")[,1:2]
-      }
+    ## extract RLum.Results
+    if (inherits(data[[i]], "RLum.Results")) {
+      data[[i]] <- get_RLum(data[[i]], "data")[,1:2]
+    }
 
       ## ensure that the dataset it not degenerate
       if (NROW(data[[i]]) == 0) {
@@ -243,7 +209,6 @@ plot_KDE <- function(
         data[[i]] <- data[[i]][, 1:2]
         attr(data[[i]], "De.errors.available") <- TRUE
       }
-    }
 
     ## find the index Inf values in each of the two columns and remove the
     ## corresponding rows if needed
@@ -251,7 +216,7 @@ plot_KDE <- function(
     if (length(inf.idx) > 0) {
       inf.row <- sort(unique(inf.idx))
       .throw_warning("Inf values removed in rows: ",
-                     paste(inf.row, collapse = ", "), " in data.frame ", i)
+                     .collapse(inf.row, quote = FALSE), " in data.frame ", i)
       data[[i]] <- data[[i]][-inf.row, ]
       rm(inf.idx, inf.row)
 
@@ -266,13 +231,20 @@ plot_KDE <- function(
   if(length(data) == 0)
     .throw_error("Your input is empty due to Inf removal")
 
-  ## check/set function parameters
-  if(missing(summary) == TRUE) {
-    summary <- ""
+  .validate_logical_scalar(values.cumulative)
+  .validate_logical_scalar(order)
+  .validate_logical_scalar(boxplot)
+  .validate_logical_scalar(rug)
+  .validate_args(summary.method, c("MCM", "weighted", "unweighted"))
+  .validate_class(summary, "character")
+  .validate_class(summary.pos, c("numeric", "character"))
+  if (is.numeric(summary.pos)) {
+    .validate_length(summary.pos, 2)
   }
-
-  if(missing(summary.pos) == TRUE) {
-    summary.pos <- "sub"
+  else {
+    .validate_args(summary.pos, c("sub", "left", "center", "right",
+                                  "topleft", "top", "topright",
+                                  "bottomleft", "bottom", "bottomright"))
   }
 
   ## set mtext output
@@ -296,12 +268,9 @@ plot_KDE <- function(
     for(i in 1:length(data)) {
       na.idx <- which(is.na(data[[i]][, 1]))
       n.NA <- length(na.idx)
-      if(n.NA == 1) {
-        message(paste("1 NA value excluded from data set", i, "."))
-      } else if(n.NA > 1) {
-        message(paste(n.NA, "NA values excluded from data set", i, "."))
-      }
       if (n.NA > 0) {
+        message(sprintf("%d NA value%s excluded from data set %d\n",
+                        n.NA, ifelse(n.NA > 1, "s", ""), i))
         data[[i]] <- data[[i]][-na.idx, ]
       }
     }
@@ -319,7 +288,7 @@ plot_KDE <- function(
   colnames(De.stats) <- c("n",
                           "mean",
                           "median",
-                          "kde.max",
+                          "kdemax",
                           "sd.abs",
                           "sd.rel",
                           "se.abs",
@@ -350,7 +319,6 @@ plot_KDE <- function(
       De.density[[length(De.density) + 1]] <- density(data[[i]][,1],
                                                       kernel = "gaussian",
                                                       bw = bw)
-
     }else{
       De.density[[length(De.density) + 1]] <- NA
       .throw_warning("Single data point found, no density calculated")
@@ -393,203 +361,69 @@ plot_KDE <- function(
                         min(De.density.range[,3]),
                         max(De.density.range[,4]))
 
-  label.text = list(NA)
+  ## helper to generate an element of the statistical summary
+  .summary_line <- function(keyword, summary, val, label = keyword,
+                            percent = FALSE, sep = FALSE, digits = 2) {
+    ifelse(keyword %in% summary,
+           paste0(label, " = ", round(val, digits),
+                  if (percent) " %" else NULL, if (sep) " | " else  "\n"),
+           "")
+  }
 
-  if(summary.pos[1] != "sub") {
-    n.rows <- length(summary)
+  ## initialize list with a dummy element, it will be removed afterwards
+  label.text <- list(NA)
 
-    for(i in 1:length(data)) {
-      stops <- paste(rep("\n", (i - 1) * n.rows), collapse = "")
+  is.sub <- summary.pos[1] == "sub"
+  stops <- NULL
+  for (i in 1:length(data)) {
+    if (!is.sub)
+      stops <- paste(rep("\n", (i - 1) * length(summary)), collapse = "")
 
-      summary.text <- character(0)
-
-      for(j in 1:length(summary)) {
-        summary.text <- c(summary.text,
-                          paste(
-                            "",
-                            ifelse("n" %in% summary[j] == TRUE,
-                                   paste("n = ",
-                                         De.stats[i,1],
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("mean" %in% summary[j] == TRUE,
-                                   paste("mean = ",
-                                         round(De.stats[i,2], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("median" %in% summary[j] == TRUE,
-                                   paste("median = ",
-                                         round(De.stats[i,3], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("kde.max" %in% summary[j] == TRUE,
-                                   paste("kdemax = ",
-                                         round(De.stats[i,4], 2),
-                                         " \n ",
-                                         sep = ""),
-                                   ""),
-                            ifelse("sd.abs" %in% summary[j] == TRUE,
-                                   paste("sd = ",
-                                         round(De.stats[i,5], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("sd.rel" %in% summary[j] == TRUE,
-                                   paste("rel. sd = ",
-                                         round(De.stats[i,6], 2), " %",
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("se.abs" %in% summary[j] == TRUE,
-                                   paste("se = ",
-                                         round(De.stats[i,7], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("se.rel" %in% summary[j] == TRUE,
-                                   paste("rel. se = ",
-                                         round(De.stats[i,8], 2), " %",
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("skewness" %in% summary[j] == TRUE,
-                                   paste("skewness = ",
-                                         round(De.stats[i,11], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("kurtosis" %in% summary[j] == TRUE,
-                                   paste("kurtosis = ",
-                                         round(De.stats[i,12], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("in.2s" %in% summary[j] == TRUE,
-                                   paste("in 2 sigma = ",
-                                         round(sum(data[[i]][,1] >
-                                                     (De.stats[i,2] - 2 *
-                                                        De.stats[i,5]) &
-                                                     data[[i]][,1] <
-                                                     (De.stats[i,2] + 2 *
-                                                        De.stats[i,5])) /
-                                                 nrow(data[[i]]) * 100 , 1),
-                                         " %",
-                                         sep = ""),
-                                   ""),
-                            sep = ""))
-      }
-
-      summary.text <- paste(summary.text, collapse = "")
-
-      label.text[[length(label.text) + 1]] <- paste(stops,
-                                                    summary.text,
-                                                    stops,
-                                                    sep = "")
-    }
-  } else {
-    for(i in 1:length(data)) {
-
-      summary.text <- character(0)
-
-      for(j in 1:length(summary)) {
-        summary.text <- c(summary.text,
-                          ifelse("n" %in% summary[j] == TRUE,
-                                 paste("n = ",
-                                       De.stats[i,1],
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("mean" %in% summary[j] == TRUE,
-                                 paste("mean = ",
-                                       round(De.stats[i,2], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("median" %in% summary[j] == TRUE,
-                                 paste("median = ",
-                                       round(De.stats[i,3], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("kde.max" %in% summary[j] == TRUE,
-                                 paste("kdemax = ",
-                                       round(De.stats[i,4], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("sd.rel" %in% summary[j] == TRUE,
-                                 paste("rel. sd = ",
-                                       round(De.stats[i,6], 2), " %",
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("sd.abs" %in% summary[j] == TRUE,
-                                 paste("abs. sd = ",
-                                       round(De.stats[i,5], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("se.rel" %in% summary[j] == TRUE,
-                                 paste("rel. se = ",
-                                       round(De.stats[i,8], 2), " %",
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("se.abs" %in% summary[j] == TRUE,
-                                 paste("abs. se = ",
-                                       round(De.stats[i,7], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("skewness" %in% summary[j] == TRUE,
-                                 paste("skewness = ",
-                                       round(De.stats[i,11], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("kurtosis" %in% summary[j] == TRUE,
-                                 paste("kurtosis = ",
-                                       round(De.stats[i,12], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("in.2s" %in% summary[j] == TRUE,
-                                 paste("in 2 sigma = ",
-                                       round(sum(data[[i]][,1] >
-                                                   (De.stats[i,2] - 2 *
-                                                      De.stats[i,5]) &
-                                                   data[[i]][,1] <
-                                                   (De.stats[i,2] + 2 *
-                                                      De.stats[i,5])) /
-                                               nrow(data[[i]]) * 100 , 1),
-                                       " %   ",
-                                       sep = ""),
-                                 "")
-        )
-      }
-
-      summary.text <- paste(summary.text, collapse = "")
-
-      label.text[[length(label.text) + 1]]  <- paste(
-        "  ",
-        summary.text,
-        sep = "")
+    summary.text <- character(0)
+    for (j in 1:length(summary)) {
+      summary.text <-
+        c(summary.text,
+          .summary_line("n", summary[j], De.stats[i, 1], sep = is.sub),
+          .summary_line("mean", summary[j], De.stats[i, 2], sep = is.sub),
+          .summary_line("median", summary[j], De.stats[i, 3], sep = is.sub),
+          .summary_line("kdemax", summary[j], De.stats[i, 4], sep = is.sub),
+          .summary_line("sd.abs", summary[j], De.stats[i, 5], sep = is.sub,
+                        label = "sd"),
+          .summary_line("sd.rel", summary[j], De.stats[i, 6], sep = is.sub,
+                        label = "rel. sd", percent = TRUE),
+          .summary_line("se.abs", summary[j], De.stats[i, 7], sep = is.sub,
+                        label = "se"),
+          .summary_line("se.rel", summary[j], De.stats[i, 8], sep = is.sub,
+                        label = "rel. se", percent = TRUE),
+          .summary_line("skewness", summary[j], De.stats[i, 11], sep = is.sub),
+          .summary_line("kurtosis", summary[j], De.stats[i, 12], sep = is.sub),
+          .summary_line("in.2s", summary[j],
+                        sum(data[[i]][, 1] > (De.stats[i, 2] - 2 *
+                                              De.stats[i, 5]) &
+                            data[[i]][, 1] < (De.stats[i, 2] + 2 *
+                                              De.stats[i, 5])) /
+                        nrow(data[[i]]) * 100, sep = is.sub,
+                        label = "in 2 sigma", percent = TRUE, digits = 1))
     }
 
-    ## remove outer vertical lines from string
-    for(i in 2:length(label.text)) {
-      label.text[[i]] <- substr(x = label.text[[i]],
-                                start = 3,
-                                stop = nchar(label.text[[i]]) - 3)
-    }
+    label.text[[length(label.text) + 1]] <- paste0(
+        if (is.sub) "" else stops,
+        paste(summary.text, collapse = ""),
+        stops)
   }
 
   ## remove dummy list element
   label.text[[1]] <- NULL
 
+  ## remove outer vertical lines from string
+  if (is.sub) {
+    for (i in seq_along(label.text)) {
+      label.text[[i]] <- substr(x = label.text[[i]],
+                                start = 1,
+                                stop = nchar(label.text[[i]]) - 3)
+    }
+  }
+
   ## read out additional parameters -------------------------------------------
   if("main" %in% names(list(...))) {
     main <- list(...)$main
@@ -628,9 +462,7 @@ plot_KDE <- function(
 
   if("ylim" %in% names(list(...))) {
     ylim.plot <- list(...)$ylim
-    if (length(ylim.plot) < 4) {
-      .throw_error("'ylim' must be a vector of length 4")
-    }
+    .validate_length(ylim.plot, 4, name = "'ylim'")
   } else {
     if(!is.na(De.density.range[1])){
       ylim.plot <- c(De.density.range[3],
@@ -643,9 +475,7 @@ plot_KDE <- function(
                      max(De.stats[,1]),
                      0,
                      max(De.stats[,1]))
-
     }
-
   }
 
   if("log" %in% names(list(...))) {
@@ -681,127 +511,32 @@ plot_KDE <- function(
     col.background <- NA
   } else {
 
-    if(length(layout$kde$colour$main) == 1) {
-      col.main <- c(layout$kde$colour$main, 2:length(data))
-    } else {
-      col.main <- layout$kde$colour$main
+    .set_colour_value <- function(layout_value) {
+      if (length(layout_value) == 1)
+        c(layout_value, 2:length(data))
+      else
+        layout_value
     }
-
-    if(length(layout$kde$colour$xlab) == 1) {
-      col.xlab <- c(layout$kde$colour$xlab, 2:length(data))
-    } else {
-      col.xlab <- layout$kde$colour$xlab
-    }
-
-    if(length(layout$kde$colour$ylab1) == 1) {
-      col.ylab1 <- c(layout$kde$colour$ylab1, 2:length(data))
-    } else {
-      col.ylab1 <- layout$kde$colour$ylab1
-    }
-
-    if(length(layout$kde$colour$ylab2) == 1) {
-      col.ylab2 <- c(layout$kde$colour$ylab2, 2:length(data))
-    } else {
-      col.ylab2 <- layout$kde$colour$ylab2
-    }
-
-    if(length(layout$kde$colour$xtck) == 1) {
-      col.xtck <- c(layout$kde$colour$xtck, 2:length(data))
-    } else {
-      col.xtck <- layout$kde$colour$xtck
-    }
-
-    if(length(layout$kde$colour$ytck1) == 1) {
-      col.ytck1 <- c(layout$kde$colour$ytck1, 2:length(data))
-    } else {
-      col.ytck1 <- layout$kde$colour$ytck1
-    }
-
-    if(length(layout$kde$colour$ytck2) == 1) {
-      col.ytck2 <- c(layout$kde$colour$ytck2, 2:length(data))
-    } else {
-      col.ytck2 <- layout$kde$colour$ytck2
-    }
-
-    if(length(layout$kde$colour$box) == 1) {
-      col.box <- c(layout$kde$colour$box, 2:length(data))
-    } else {
-      col.box <- layout$kde$colour$box
-    }
-
-    if(length(layout$kde$colour$mtext) == 1) {
-      col.mtext <- c(layout$kde$colour$mtext, 2:length(data))
-    } else {
-      col.mtext <- layout$kde$colour$mtext
-    }
-
-    if(length(layout$kde$colour$stats) == 1) {
-      col.stats <- c(layout$kde$colour$stats, 2:length(data))
-    } else {
-      col.stats <- layout$kde$colour$stats
-    }
-
-    if(length(layout$kde$colour$kde.line) == 1) {
-      col.kde.line <- c(layout$kde$colour$kde.line, 2:length(data))
-    } else {
-      col.kde.line <- layout$kde$colour$kde.line
-    }
-
-    if(length(layout$kde$colour$kde.fill) == 1) {
-      col.kde.fill <- c(layout$kde$colour$kde.fill, 2:length(data))
-    } else {
-      col.kde.fill <- layout$kde$colour$kde.fill
-    }
-
-    if(length(layout$kde$colour$value.dot) == 1) {
-      col.value.dot <- c(layout$kde$colour$value.dot, 2:length(data))
-    } else {
-      col.value.dot <- layout$kde$colour$value.dot
-    }
-
-    if(length(layout$kde$colour$value.bar) == 1) {
-      col.value.bar <- c(layout$kde$colour$value.bar, 2:length(data))
-    } else {
-      col.value.bar <- layout$kde$colour$value.bar
-    }
-
-    if(length(layout$kde$colour$value.rug) == 1) {
-      col.value.rug <- c(layout$kde$colour$value.rug, 2:length(data))
-    } else {
-      col.value.rug <- layout$kde$colour$value.rug
-    }
-
-    if(length(layout$kde$colour$boxplot.line) == 1) {
-      col.boxplot.line <- c(layout$kde$colour$boxplot.line, 2:length(data))
-    } else {
-      col.boxplot.line <- layout$kde$colour$boxplot.line
-    }
-
-    if(length(layout$kde$colour$boxplot.fill) == 1) {
-      col.boxplot.fill <- c(layout$kde$colour$boxplot.fill, 2:length(data))
-    } else {
-      col.boxplot.fill <- layout$kde$colour$boxplot.fill
-    }
-
-    if(length(layout$kde$colour$mean.line) == 1) {
-      col.mean.line <- adjustcolor(col = 1:length(data),
-                                   alpha.f = 0.4)
-    } else {
-      col.mean.line <- layout$kde$colour$mean.point
-    }
-
-    if(length(layout$kde$colour$sd.bar) == 1) {
-      col.sd.bar <- c(layout$kde$colour$sd.bar, 2:length(data))
-    } else {
-      col.sd.bar <- layout$kde$colour$sd.line
-    }
-
-    if(length(layout$kde$colour$background) == 1) {
-      col.background <- c(layout$kde$colour$background, 2:length(data))
-    } else {
-      col.background <- layout$kde$colour$background
-    }
-
+    col.main <- .set_colour_value(layout$kde$colour$main)
+    col.xlab <- .set_colour_value(layout$kde$colour$xlab)
+    col.ylab1 <- .set_colour_value(layout$kde$colour$ylab1)
+    col.ylab2 <- .set_colour_value(layout$kde$colour$ylab2)
+    col.xtck <- .set_colour_value(layout$kde$colour$xtck)
+    col.ytck1 <- .set_colour_value(layout$kde$colour$ytck1)
+    col.ytck2 <- .set_colour_value(layout$kde$colour$ytck2)
+    col.box <- .set_colour_value(layout$kde$colour$box)
+    col.mtext <- .set_colour_value(layout$kde$colour$mtext)
+    col.stats <- .set_colour_value(layout$kde$colour$stats)
+    col.kde.line <- .set_colour_value(layout$kde$colour$kde.line)
+    col.kde.fill <- .set_colour_value(layout$kde$colour$kde.fill)
+    col.value.dot <- .set_colour_value(layout$kde$colour$value.dot)
+    col.value.bar <- .set_colour_value(layout$kde$colour$value.bar)
+    col.value.rug <- .set_colour_value(layout$kde$colour$value.rug)
+    col.boxplot.line <- .set_colour_value(layout$kde$colour$boxplot.line)
+    col.boxplot.fill <- .set_colour_value(layout$kde$colour$boxplot.fill)
+    col.mean.line <- .set_colour_value(layout$kde$colour$mean.point)
+    col.sd.bar <- .set_colour_value(layout$kde$colour$sd.line)
+    col.background <- .set_colour_value(layout$kde$colour$background)
   }
 
   if("lty" %in% names(list(...))) {
@@ -829,50 +564,16 @@ plot_KDE <- function(
   }
 
   ## convert keywords into summary placement coordinates
-  if(missing(summary.pos) == TRUE) {
-    summary.pos <- c(xlim.plot[1], ylim.plot[2])
-    summary.adj <- c(0, 1)
-  } else if(length(summary.pos) == 2) {
-    summary.pos <- summary.pos
-    summary.adj <- c(0, 1)
-  } else if(summary.pos[1] == "topleft") {
-    summary.pos <- c(xlim.plot[1], ylim.plot[2])
-    summary.adj <- c(0, 1)
-  } else if(summary.pos[1] == "top") {
-    summary.pos <- c(mean(xlim.plot), ylim.plot[2])
-    summary.adj <- c(0.5, 1)
-  } else if(summary.pos[1] == "topright") {
-    summary.pos <- c(xlim.plot[2], ylim.plot[2])
-    summary.adj <- c(1, 1)
-  }  else if(summary.pos[1] == "left") {
-    summary.pos <- c(xlim.plot[1], mean(ylim.plot[1:2]))
-    summary.adj <- c(0, 0.5)
-  } else if(summary.pos[1] == "center") {
-    summary.pos <- c(mean(xlim.plot), mean(ylim.plot[1:2]))
-    summary.adj <- c(0.5, 0.5)
-  } else if(summary.pos[1] == "right") {
-    summary.pos <- c(xlim.plot[2], mean(ylim.plot[1:2]))
-    summary.adj <- c(1, 0.5)
-  }else if(summary.pos[1] == "bottomleft") {
-    summary.pos <- c(xlim.plot[1], ylim.plot[1])
-    summary.adj <- c(0, 0)
-  } else if(summary.pos[1] == "bottom") {
-    summary.pos <- c(mean(xlim.plot), ylim.plot[1])
-    summary.adj <- c(0.5, 0)
-  } else if(summary.pos[1] == "bottomright") {
-    summary.pos <- c(xlim.plot[2], ylim.plot[1])
-    summary.adj <- c(1, 0)
-  }
+  coords <- .get_keyword_coordinates(summary.pos, xlim.plot, ylim.plot[1:2])
+  summary.pos <- coords$pos
+  summary.adj <- coords$adj
 
   ## plot data sets -----------------------------------------------------------
 
   ## setup plot area
-  if(length(summary) >= 1 & summary.pos[1] == "sub") {
-
+  toplines <- 1
+  if (length(summary) >= 1 && is.sub) {
     toplines <- length(data)
-  } else {
-
-    toplines <- 1
   }
 
   ## extract original plot parameters
@@ -883,15 +584,11 @@ plot_KDE <- function(
       xpd = FALSE,
       cex = cex)
 
-  if(layout$kde$dimension$figure.width != "auto" |
-     layout$kde$dimension$figure.height != "auto") {
-    par(mai = layout$kde$dimension$margin / 25.4,
-        pin = c(layout$kde$dimension$figure.width / 25.4 -
-                  layout$kde$dimension$margin[2] / 25.4 -
-                  layout$kde$dimension$margin[4] / 25.4,
-                layout$kde$dimension$figure.height / 25.4 -
-                  layout$kde$dimension$margin[1] / 25.4 -
-                  layout$kde$dimension$margin[3]/25.4))
+  dim <- layout$kde$dimension
+  if (dim$figure.width != "auto" || dim$figure.height != "auto") {
+    par(mai = dim$margin / 25.4,
+        pin = c(dim$figure.width - dim$margin[2] - dim$margin[4],
+                dim$figure.height - dim$margin[1] - dim$margin[3]) / 25.4)
   }
 
   ## create empty plot to get plot dimensions
@@ -923,8 +620,8 @@ plot_KDE <- function(
        ann = FALSE)
 
   ## add box
-  box(which = "plot",
-      col = layout$kde$colour$box)
+  graphics::box(which = "plot",
+                col = layout$kde$colour$box)
 
   ## add x-axis
   axis(side = 1,
@@ -990,9 +687,7 @@ plot_KDE <- function(
               col = col.kde.fill[i],
               lty = lty[i],
               lwd = lwd[i])
-
     }
-
   }
 
   ## add plot title
@@ -1022,8 +717,7 @@ plot_KDE <- function(
   ## add summary content
   for(i in 1:length(data)) {
 
-    if(summary.pos[1] != "sub") {
-
+    if (!is.sub) {
       text(x = summary.pos[1],
            y = summary.pos[2],
            adj = summary.adj,
@@ -1033,7 +727,6 @@ plot_KDE <- function(
     } else {
 
       if(mtext == "") {
-
         mtext(side = 3,
               line = (toplines + 0.3 - i) * layout$kde$dimension$stats.line / 100,
               text = label.text[[i]],
@@ -1087,8 +780,8 @@ plot_KDE <- function(
       boxplot.data <- list(NA)
 
       for(i in 1:length(data)) {
-        boxplot.i <- boxplot(x = data[[i]][,1],
-                             plot = FALSE)
+        boxplot.i <- graphics::boxplot(x = data[[i]][,1],
+                                       plot = FALSE)
         boxplot.i$group <- mean(x = data[[i]][,1],
                                                    na.rm = TRUE)
         boxplot.i$names <- sd(x = data[[i]][,1],
@@ -1120,8 +813,7 @@ plot_KDE <- function(
         ## draw median line
         lines(x = c(boxplot.data[[i]]$stats[3,1],
                     boxplot.data[[i]]$stats[3,1]),
-              y = c(-11/8 * l_height,
-                    -7/8 * l_height),
+              y = c(-11/8, -7/8) * l_height,
               lwd = 2,
               col = col.boxplot.line[i])
 
@@ -1130,36 +822,29 @@ plot_KDE <- function(
                       boxplot.data[[i]]$stats[2,1],
                       boxplot.data[[i]]$stats[4,1],
                       boxplot.data[[i]]$stats[4,1]),
-                y = c(-11/8 * l_height,
-                      -7/8 * l_height,
-                      -7/8 * l_height,
-                      -11/8 * l_height),
+                y = c(-11/8, -7/8, -7/8, -11/8) * l_height,
                 col = col.boxplot.fill[i],
                 border = col.boxplot.line[i])
 
         ## draw whiskers
         lines(x = c(boxplot.data[[i]]$stats[2,1],
                     boxplot.data[[i]]$stats[1,1]),
-              y = c(-9/8 * l_height,
-                    -9/8 * l_height),
+              y = c(-9/8, -9/8) * l_height,
               col = col.boxplot.line[i])
 
         lines(x = c(boxplot.data[[i]]$stats[1,1],
                     boxplot.data[[i]]$stats[1,1]),
-              y = c(-10/8 * l_height,
-                    -8/8 * l_height),
+              y = c(-10/8, -8/8) * l_height,
               col = col.boxplot.line[i])
 
         lines(x = c(boxplot.data[[i]]$stats[4,1],
                     boxplot.data[[i]]$stats[5,1]),
-              y = c(-9/8 * l_height,
-                    -9/8 * l_height),
+              y = c(-9/8, -9/8) * l_height,
               col = col.boxplot.line[i])
 
         lines(x = c(boxplot.data[[i]]$stats[5,1],
                     boxplot.data[[i]]$stats[5,1]),
-              y = c(-10/8 * l_height,
-                    -8/8 * l_height),
+              y = c(-10/8, -8/8) * l_height,
               col = col.boxplot.line[i])
 
         ## draw outliers
@@ -1169,7 +854,6 @@ plot_KDE <- function(
                col = col.boxplot.line[i],
                cex = cex * 0.8)
       }
-
     }
 
     ## optionally add rug
@@ -1226,7 +910,7 @@ plot_KDE <- function(
     ## add De error bars
     for(i in 1:length(data)) {
       if (attr(data[[i]], "De.errors.available")) {
-        arrows(data[[i]][, 1] - data[[i]][, 2],
+        graphics::arrows(data[[i]][, 1] - data[[i]][, 2],
                1:length(data[[i]][,1]),
                data[[i]][, 1] + data[[i]][, 2],
                1:length(data[[i]][, 1]),
@@ -1258,10 +942,7 @@ plot_KDE <- function(
   ## FUN by R Luminescence Team
   if (fun == TRUE) sTeve() # nocov
 
-  if(output == TRUE) {
-    return(invisible(list(De.stats = De.stats,
-                          summary.pos = summary.pos,
-                          De.density = De.density)))
-  }
-
+  invisible(list(De.stats = De.stats,
+                 summary.pos = summary.pos,
+                 De.density = De.density))
 }
