@@ -1,7 +1,7 @@
-#' @title Plot function for an RLum.Analysis S4 class object
+#' @title Plot function for an `RLum.Analysis` S4 class object
 #'
 #' @description The function provides a standardised plot output for curve data of an
-#' RLum.Analysis S4 class object
+#' [RLum.Analysis-class] object
 #'
 #' The function produces a multiple plot output. A file output is recommended
 #' (e.g., [pdf]).
@@ -427,11 +427,13 @@ plot_RLum.Analysis <- function(
     temp.recordType <- as.character(unique(temp.object.structure$recordType))
 
     ##change graphic settings
-    par.default <- par()[c("cex", "mfrow")]
     if (!plot_singlePanels) {
+      par.default <- par()[c("cex", "mfrow")]
       if(!missing(ncols) & !missing(nrows)){
         par(mfrow = c(nrows, ncols))
       }
+    } else {
+      par.default <- par()["cex"]
     }
     ## this 2nd par request is needed as setting mfrow resets the par
     ## settings ... this might not be wanted
@@ -460,7 +462,6 @@ plot_RLum.Analysis <- function(
       names(abline) <- abline.names
     }
 
-
     ##(2) PLOT values
     for(k in 1:length(temp.recordType)) {
 
@@ -672,17 +673,15 @@ plot_RLum.Analysis <- function(
       ##if legend is outside of the plotting area we need to allow overplotting
       ##AFTER all lines have been drawn
       if (legend.pos == "outside") {
-        par(xpd = TRUE)
-
         # determine legend position on log(y) scale
         ypos <- par()$usr[4]
         if (grepl("y", plot.settings$log[[k]], ignore.case = TRUE))
-          ypos <- 10^par()$usr[4]
+          ypos <- 10^ypos
 
         # determine position on log(x) scale
         xpos <- par()$usr[2]
         if (grepl("x", plot.settings$log[[k]], ignore.case = TRUE))
-          xpos <- 10^par()$usr[2]
+          xpos <- 10^xpos
       }
 
       ##legend
@@ -699,12 +698,9 @@ plot_RLum.Analysis <- function(
             legend.col
           },
           bty = "n",
+          xpd = legend.pos == "outside",
           cex = 0.8 * plot.settings$cex[[k]]
         )
-
-        # revert the over plotting
-        if (legend.pos == "outside")
-          par(xpd = FALSE)
       }
     }
 
