@@ -65,9 +65,9 @@ plot_RLum.Results<- function(
   ## CONSISTENCY CHECK OF INPUT DATA
   ##============================================================================##
 
-  ##check if object is of class RLum.Data.Curve
-  if(!is(object,"RLum.Results")){
-    .throw_error("Input object is not of type 'RLum.Results'")
+  .validate_class(object, "RLum.Results")
+  if (is.null(object@originator) || is.na(object@originator)) {
+    .throw_error("Object originator not supported")
   }
 
   ##============================================================================##
@@ -117,12 +117,13 @@ plot_RLum.Results<- function(
   switch(object@originator,
       "analyse_SAR.CWOSL" = plot_AbanicoPlot(object),
       "analyse_pIRIRSequence" = plot_AbanicoPlot(object),
-      "analyse_IRSARRF" = plot_AbanicoPlot(object),
+      "analyse_IRSAR.RF" = plot_AbanicoPlot(object),
+      "calc_FiniteMixture" = do.call(calc_FiniteMixture, c(object, extraArgs)),
     NULL
     )
 
   ## CASE 1: Minimum Age Model / Maximum Age Model -------
-  if(object@originator=="calc_MinDose" || object@originator=="calc_MaxDose") {
+  if (object@originator %in% c("calc_MinDose", "calc_MaxDose")) {
 
     ## single MAM estimate
     # plot profile log likelihood
@@ -197,10 +198,10 @@ plot_RLum.Results<- function(
       ## --------- PLOT "RECYCLE" BOOTSTRAP RESULTS ------------ ##
 
       if(single==TRUE) {
-        layout(cbind(c(1,1,2, 5,5,6), c(3,3,4, 7,7,8)))
+        graphics::layout(cbind(c(1, 1, 2, 5, 5, 6), c(3, 3, 4, 7, 7, 8)))
         par(cex = 0.6)
       } else {
-        layout(matrix(c(1,1,2)),2,1)
+        graphics::layout(matrix(c(1, 1, 2)), 2, 1)
         par(cex = 0.8)
       }
 
@@ -214,7 +215,7 @@ plot_RLum.Results<- function(
         pairs<- pairs[order(pairs[,1]),]
 
         # remove invalid NA values
-        pairs<- na.omit(pairs)
+        pairs <- stats::na.omit(pairs)
 
         plot(x=pairs[,1],
              y=pairs[,2],
@@ -300,6 +301,7 @@ plot_RLum.Results<- function(
       ### possibly integrate this in the prior polynomial plot loop
 
       ### LOESS PLOT
+      if (!anyNA(object@data$bootstrap$loess.fit)) {
       pairs<- object@data$bootstrap$pairs$gamma
       pred<- predict(object@data$bootstrap$loess.fit)
       loess<- cbind(pairs[,1], pred)
@@ -314,6 +316,7 @@ plot_RLum.Results<- function(
 
       # add LOESS line
       lines(loess, type = "l", col = "black")
+      }
 
       ### ------ PLOT BOOTSTRAP LIKELIHOOD FIT
 
@@ -355,11 +358,9 @@ plot_RLum.Results<- function(
       y<- c(0, y, 0)
 
       # cutoff negative y values
-      neg<- which(y<0)
-      if (length(neg) != 0) {
-        y<- y[-neg]
-        x<- x[-neg]
-      }
+      y.idx <- !is.na(y) & y >= 0
+      x <- x[y.idx]
+      y <- y[y.idx]
 
       # add bootstrap likelihood polygon to plot
       polygon(x, y, col = "grey80", border = NA)
@@ -375,7 +376,7 @@ plot_RLum.Results<- function(
       sd<- object@data$summary$de_err
 
 
-      if (any(is.na(c(mean, sd)))) {
+      if (anyNA(c(mean, sd))) {
         .throw_warning("Unable to plot the MAM single estimate (NA value)")
 
       } else {
@@ -403,7 +404,7 @@ plot_RLum.Results<- function(
         y<- y/max(y)
 
         # fit a smoothing spline
-        l<- spline(x = x, y = y, method = "n", n = 1000)
+        l<- stats::spline(x = x, y = y, method = "n", n = 1000)
 
         # make the endpoints zero
         l$y[1]<- l$y[length(l$y)]<- 0
@@ -413,7 +414,6 @@ plot_RLum.Results<- function(
 
         # add vertical lines of the mean values
         #points(x = 80, y = 100,type = "l")
-
       }
 
       #### ------ PLOT DE
@@ -442,7 +442,7 @@ plot_RLum.Results<- function(
       err<- object@data$data[order(object@data$data[,1]),2]
 
       # fancy error bars
-      arrows(x0 = x-err, y0 = y,
+      graphics::arrows(x0 = x-err, y0 = y,
              x1 =  x+err, y1 = y,
              code = 3, angle = 90, length = 0.05)
 
@@ -566,7 +566,7 @@ plot_RLum.Results<- function(
           col="red",lty="dashed", lwd = lwd)
 
     #plot some further informations into the grafik
-    arrows(
+    graphics::arrows(
       usedDeValues[length(usedDeValues[,1])-n.usedDeValues+1,1]+usedDeValues[length(usedDeValues[,1])-n.usedDeValues+1,1]*0.02, #x1
       max(o)+0.5, #y1
       max(usedDeValues[,1]-usedDeValues[,1]*0.02), #x2
@@ -587,378 +587,13 @@ plot_RLum.Results<- function(
     mtext(side=3,mtext,cex=cex)
   }
 
-
-  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
-  ## CASE 4: Finite Mixture Model --------
-  if(object@originator == "calc_FiniteMixture") {
-    if(length(object@data$args$n.components) > 1) {
-      ##deal with addition arguments
-      extraArgs <- list(...)
-
-      main <- if("main" %in% names(extraArgs)) {extraArgs$main} else {"Finite Mixture Model"}
-      plot.proportions<- if("plot.proportions" %in% names(extraArgs)) {extraArgs$plot.proportions} else {TRUE}
-      pdf.colors<- if("pdf.colors" %in% names(extraArgs)) {extraArgs$pdf.colors} else {"gray"}
-      pdf.weight<- if("pdf.weight" %in% names(extraArgs)) {extraArgs$pdf.weight} else {TRUE}
-      pdf.sigma<- if("pdf.sigma" %in% names(extraArgs)) {extraArgs$pdf.sigma} else {"sigmab"}
-
-      # extract relevant data from object
-      n.components <- object@data$args$n.components
-      comp.n <- object@data$components
-      sigmab <- object@data$args$sigmab
-      BIC.n <- object@data$BIC$BIC
-      LLIK.n <- object@data$llik$llik
-
-      # save previous plot parameter and set new ones
-      .pardefault<- par(no.readonly = TRUE)
-
-      ## DEVICE AND PLOT LAYOUT
-      n.plots<- length(n.components) #number of PDF plots in plot area #1
-      seq.vertical.plots<- seq(from = 1, to = n.plots, by = 1) #indices
-      ID.plot.two<- n.plots+if(plot.proportions==TRUE){1}else{0} #ID of second plot area
-      ID.plot.three<- n.plots+if(plot.proportions==TRUE){2}else{1} #ID of third plot area
-
-      #empty vector for plot indices
-      seq.matrix<- vector(mode="integer", length=4*n.plots)
-
-      #fill vector with plot indices in correct order
-      cnt<- 1
-      seq<- seq(1,length(seq.matrix),4)
-      for(i in seq) {
-        seq.matrix[i]<- cnt
-        seq.matrix[i+1]<- cnt
-        seq.matrix[i+2]<- if(plot.proportions==TRUE){ID.plot.two}else{cnt}
-        seq.matrix[i+3]<- ID.plot.three
-
-        cnt<- cnt+1
-      }
-
-      # create device layout
-      layout(matrix(c(seq.matrix),4,n.plots))
-
-      # outer margins (bottom, left, top, right)
-      par(oma=c(2.5,5,3,5))
-
-      # general plot parameters (global scaling, allow overplotting)
-      par(cex = 0.8, xpd = NA)
-
-      # define colour palette for prettier output
-      if(pdf.colors == "colors") {
-        col.n<- c("red3", "slateblue3", "seagreen", "tan3", "yellow3",
-                  "burlywood4", "magenta4", "mediumpurple3", "brown4","grey",
-                  "aquamarine")
-        poly.border<- FALSE
-      }
-      if(pdf.colors == "gray" || pdf.colors == "grey") {
-        col.n<- gray.colors(length(n.components)*2)
-        poly.border<- FALSE
-      }
-      if(pdf.colors == "none") {
-        col.n<- NULL
-        poly.border<- TRUE
-      }
-
-      ##--------------------------------------------------------------------------
-      ## PLOT 1: EQUIVALENT DOSES OF COMPONENTS
-
-      ## create empty plot without x-axis
-      for(i in 1:n.plots) {
-        pos.n<- seq(from = 1, to = n.components[i]*3, by = 3)
-
-        # set margins (bottom, left, top, right)
-        par(mar=c(1,0,2,0))
-
-        # empty plot area
-        plot(NA, NA,
-             xlim=c(min(n.components)-0.2, max(n.components)+0.2),
-             ylim=c(min(comp.n[pos.n,]-comp.n[pos.n+1,], na.rm = TRUE),
-                    max((comp.n[pos.n,]+comp.n[pos.n+1,])*1.1, na.rm = TRUE)),
-             ylab="",
-             xaxt="n",
-             yaxt="n",
-             xlab="")
-
-        # add text in upper part of the plot ("k = 1,2..n")
-        mtext(bquote(italic(k) == .(n.components[i])),
-              side = 3, line = -2, cex=0.8)
-
-        # add y-axis label (only for the first plot)
-        if(i==1) {
-          mtext(expression(paste("D"[e]," [Gy]")), side=2,line=2.7, cex=1)
-        }
-
-        # empty list to store normal distribution densities
-        sapply.storage<- list()
-
-        ## NORMAL DISTR. OF EACH COMPONENT
-
-        # LOOP - iterate over number of components
-        for(j in 1:max(n.components)) {
-          # draw random values of the ND to check for NA values
-          suppressWarnings(
-          comp.nd.n<- sort(rnorm(n = length(object@data$data[,1]),
-                                 mean = comp.n[pos.n[j],i],
-                                 sd = comp.n[pos.n[j]+1,i]))
-          )
-          # proceed if no NA values occurred
-          if(length(comp.nd.n)!=0) {
-
-            # weight - proportion of the component
-            wi<- comp.n[pos.n[j]+2,i]
-
-            # calculate density values with(out) weights
-            fooX<- function(x) {
-              dnorm(x, mean = comp.n[pos.n[j],i],
-                    sd = if(pdf.sigma=="se"){comp.n[pos.n[j]+1,i]}
-                    else{if(pdf.sigma=="sigmab"){comp.n[pos.n[j],i]*sigmab}}
-              )*
-                if(pdf.weight==TRUE){wi}else{1}
-            }
-
-            # x-axis scaling - determine highest dose in first cycle
-            if(i==1 && j==1){
-              max.dose<- max(object@data$data[,1])+sd(object@data$data[,1])/2
-              min.dose<- min(object@data$data[,1])-sd(object@data$data[,1])/2
-
-              # density function to determine y-scaling if no weights are used
-              fooY<- function(x) {
-                dnorm(x, mean = na.exclude(comp.n[pos.n,]),
-                      sd = na.exclude(comp.n[pos.n+1,]))
-              }
-              # set y-axis scaling
-              dens.max<-max(sapply(0:max.dose, fooY))
-            }##EndOfIf::first cycle settings
-
-            # override y-axis scaling if weights are used
-            if(pdf.weight==TRUE){
-              sapply.temp<- list()
-              for(b in 1:max(n.components)){
-
-                # draw random values of the ND to check for NA values
-                suppressWarnings(
-                comp.nd.n<- sort(rnorm(n = length(object@data$data[,1]),
-                                       mean = comp.n[pos.n[b],i],
-                                       sd = comp.n[pos.n[b]+1,i]))
-                )
-                # proceed if no NA values occurred
-                if(length(comp.nd.n)!=0) {
-
-                  # weight - proportion of the component
-                  wi.temp<- comp.n[pos.n[b]+2,i]
-
-                  fooT<- function(x) {
-                    dnorm(x, mean = comp.n[pos.n[b],i],
-                          sd = if(pdf.sigma=="se"){comp.n[pos.n[b]+1,i]}
-                          else{if(pdf.sigma=="sigmab"){comp.n[pos.n[b],i]*sigmab}}
-                    )*wi.temp
-                  }
-                  sapply.temp[[b]]<- sapply(0:max.dose, fooT)
-                }
-              }
-              dens.max<- max(Reduce('+', sapply.temp))
-            }
-
-            # calculate density values for 0 to maximum dose
-            sapply<- sapply(0:max.dose, fooX)
-
-            # save density values in list for sum curve of gaussians
-            sapply.storage[[j]]<- sapply
-
-            ## determine axis scaling
-            # x-axis (dose)
-            if("dose.scale" %in% names(extraArgs)) {
-              y.scale<- extraArgs$dose.scale
-            } else {
-              y.scale<- c(min.dose,max.dose)
-            }
-            # y-axis (density)
-            if("pdf.scale" %in% names(extraArgs)) {
-              x.scale<- extraArgs$pdf.scale
-            } else {
-              x.scale<- dens.max*1.1
-            }
-
-            ## PLOT Normal Distributions
-            par(new=TRUE)
-            plot(sapply, 1:length(sapply)-1,
-                 type="l", yaxt="n", xaxt="n", col=col.n[j], lwd=1,
-                 ylim=y.scale,
-                 xlim=c(0,x.scale),
-                 xaxs="i", yaxs="i",
-                 ann=FALSE, xpd = FALSE)
-
-            # draw coloured polygons under curve
-            polygon(x=c(min(sapply), sapply,  min(sapply)),
-                    y=c(0, 0:max.dose,  0),
-                    col = adjustcolor(col.n[j], alpha.f = 0.66),
-                    yaxt="n", border=poly.border, xpd = FALSE, lty = 2, lwd = 1.5)
-
-          }
-        }##EndOf::Component loop
-
-        # Add sum of Gaussian curve
-        par(new = TRUE)
-
-        plot(Reduce('+', sapply.storage),1:length(sapply)-1,
-             type="l", yaxt="n", xaxt="n", col="black",
-             lwd=1.5, lty = 1,
-             ylim=y.scale,
-             xlim=c(0,x.scale),
-             xaxs="i", yaxs="i", ann=FALSE, xpd = FALSE)
-
-        # draw additional info during first k-cycle
-        if(i == 1) {
-
-          # plot title
-          mtext("Normal distributions",
-                side = 3, font = 2, line = 0, adj = 0, cex = 0.8)
-
-          # main title
-          mtext(main,
-                side = 3, font = 2, line = 3.5, adj = 0.5,
-                at = grconvertX(0.5, from = "ndc", to = "user"))
-
-          # subtitle
-          mtext(as.expression(bquote(italic(sigma[b]) == .(sigmab) ~
-                                       "|" ~ n == .(length(object@data$data[,1])))),
-                side = 3, font = 1, line = 2.2, adj = 0.5,
-                at = grconvertX(0.5, from = "ndc", to = "user"), cex = 0.9)
-
-          # x-axis label
-          mtext("Density [a.u.]",
-                side = 1, line = 0.5, adj = 0.5,
-                at = grconvertX(0.5, from = "ndc", to = "user"))
-
-          # draw y-axis with proper labels
-          axis(side=2, labels = TRUE)
-        }
-
-        if(pdf.colors == "colors") {
-          # create legend labels
-          dose.lab.legend<- paste("c", 1:n.components[length(n.components)], sep="")
-
-          if(max(n.components)>8) {
-            ncol.temp<- 8
-            yadj<- 1.025
-          } else {
-            ncol.temp<- max(n.components)
-            yadj<- 0.93
-          }
-
-          # add legend
-          if(i==n.plots) {
-            legend(grconvertX(0.55, from = "ndc", to = "user"),
-                   grconvertY(yadj, from = "ndc", to = "user"),
-                   legend = dose.lab.legend,
-                   col = col.n[1:max(n.components)],
-                   pch = 15, adj = c(0,0.2), pt.cex=1.4,
-                   bty = "n", ncol=ncol.temp, x.intersp=0.4)
-
-            mtext("Components: ", cex = 0.8,
-                  at = grconvertX(0.5, from = "ndc", to = "user"))
-          }
-        }
-
-      }##EndOf::k-loop and Plot 1
-
-      ##--------------------------------------------------------------------------
-      ## PLOT 2: PROPORTION OF COMPONENTS
-      if(plot.proportions==TRUE) {
-        # margins for second plot
-        par(mar=c(2,0,2,0))
-
-        # create matrix with proportions from a subset of the summary matrix
-        prop.matrix<- comp.n[pos.n+2,]*100
-
-        # stacked barplot of proportions without x-axis
-        barplot(prop.matrix,
-                width=1,
-                xlim=c(0.2, length(n.components)-0.2),
-                ylim=c(0,100),
-                axes=TRUE,
-                space=0,
-                col=col.n,
-                xpd=FALSE,
-                xaxt="n")
-
-        # y-axis label
-        mtext("Proportion [%]",
-              side=2,line=3, cex=1)
-
-
-        # add x-axis with corrected tick positions
-        axis(side = 1, labels = n.components, at = n.components+0.5-n.components[1])
-
-        # draw a box (not possible with barplot())
-        box(lty=1, col="black")
-
-        # add subtitle
-        mtext("Proportion of components",
-              side = 3, font = 2, line = 0, adj = 0, cex = 0.8)
-      }
-      ##--------------------------------------------------------------------------
-      ## PLOT 3: BIC & LLIK
-
-      # margins for third plot
-      par(mar=c(2,0,2,0))
-
-      # prepare scaling for both y-axes
-      BIC.scale<- c(min(BIC.n)*if(min(BIC.n)<0){1.2}else{0.8},
-                    max(BIC.n)*if(max(BIC.n)<0){0.8}else{1.2})
-      LLIK.scale<- c(min(LLIK.n)*if(min(LLIK.n)<0){1.2}else{0.8},
-                     max(LLIK.n)*if(max(LLIK.n)<0){0.8}else{1.2})
-
-      # plot BIC scores
-      plot(n.components, BIC.n,
-           main= "",
-           type="b",
-           pch=22,
-           cex=1.5,
-           xlim=c(min(n.components)-0.2, max(n.components)+0.2),
-           ylim=BIC.scale,
-           xaxp=c(min(n.components), max(n.components), length(n.components)-1),
-           xlab=expression(paste(italic(k), " Components")),
-           ylab=expression(paste("BIC")),
-           cex.lab=1.25)
-
-      # following plot should be added to previous
-      par(new = TRUE)
-
-      # plot LLIK estimates
-      plot(n.components, LLIK.n,
-           xlim=c(min(n.components)-0.2, max(n.components)+0.2),
-           xaxp=c(min(n.components), max(n.components), length(n.components)-1),
-           ylim=LLIK.scale,
-           yaxt="n", type="b", pch=16, xlab="", ylab="", lty=2, cex = 1.5)
-
-      # subtitle
-      mtext("Statistical criteria",
-            side = 3, font = 2, line = 0, adj = 0, cex = 0.8)
-
-      # second y-axis with proper scaling
-      axis(side = 4, ylim=c(0,100))
-
-      # LLIK axis label
-      mtext(bquote(italic(L)[max]),
-            side=4,line=3, cex=1.3)
-
-      # legend
-      legend(grconvertX(0.75, from = "nfc", to = "user"),
-             grconvertY(0.96, from = "nfc", to = "user"),
-             legend = c("BIC", as.expression(bquote(italic(L)[max]))),
-             pch = c(22,16), pt.bg=c("white","black"),
-             adj = 0, pt.cex=1.3, lty=c(1,2),
-             bty = "n", horiz = TRUE, x.intersp=0.5)
-
-
-      ## restore previous plot parameters
-      par(.pardefault)
-    }
-
-  }##EndOf::Case 4 - Finite Mixture Model
-
   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
   ## CASE 5: Aliquot Size ---------
   if(object@originator=="calc_AliquotSize") {
+    ## FIXME(mcol): The following code has been ported to calc_AliquotSize(),
+    ## where it has been further updated. This version is preserved to support
+    ## the plotting of an RLum.Results object directly, which calc_AliquotSize()
+    ## doesn't yet support.
     if(!is.null(object@data$MC$estimates)) {
       extraArgs <- list(...)
 
@@ -973,7 +608,7 @@ plot_RLum.Results<- function(
       MC.iter<- object@data$args$MC.iter
 
       # set layout of plotting device
-      layout(matrix(c(1,1,2)),2,1)
+      graphics::layout(matrix(c(1, 1, 2)), 2, 1)
       par(cex = 0.8)
 
       ## plot MC estimate distribution
@@ -988,7 +623,7 @@ plot_RLum.Results<- function(
            ylim = c(0, max(MC.n.kde$y)*1.1))
 
       # add rugs to histogram
-      rug(MC.n)
+      graphics::rug(MC.n)
 
       # add KDE curve
       lines(MC.n.kde, col = "black", lwd = 1)
@@ -1022,11 +657,9 @@ plot_RLum.Results<- function(
            xlab=xlab,  ylim=c(0.5,1.5),
            xaxt="n", yaxt="n", ylab="")
       par(bty="n")
-      boxplot(MC.n, horizontal = TRUE, add = TRUE, bty="n")
-    } else {
-      on.exit(NULL, add = TRUE) # FIXME(mcol): seems unnecessary
+      graphics::boxplot(MC.n, horizontal = TRUE, add = TRUE, bty = "n")
     }
-  }#EndOf::Case 5 - calc_AliqoutSize()
+  }#EndOf::Case 5 - calc_AliquotSize()
 
   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
   ## CASE 6: calc_SourceDoseRate() ----------
@@ -1036,10 +669,9 @@ plot_RLum.Results<- function(
     ##get data
     df <- get_RLum(object = object, data.object = "dose.rate")
 
-    ##reduce the size for plotting, more than 100 points makes no sense
-    if(nrow(df)>100) {
+    ##reduce the size for plotting, more than 100 points make no sense
+    if(nrow(df)>100)
       df <- df[seq(1,nrow(df), length = 100),]
-    }
 
     ##plot settings
     plot.settings <- list(
@@ -1096,7 +728,7 @@ plot_RLum.Results<- function(
     mtext(side = 3, plot.settings$mtext)
 
     if(!is.null(plot.settings$grid)){
-      grid(eval(plot.settings$grid))
+      graphics::grid(eval(plot.settings$grid))
     }
 
   }#EndOf::Case 6 - calc_SourceDoseRate()
