@@ -66,7 +66,7 @@
 #' @param mtext [character]:
 #' additional text below the plot title.
 #'
-#' @param summary [character] (*optional*):
+#' @param summary [character] (*with default*):
 #' add statistic measures of centrality and dispersion to the plot.
 #' Can be one or more of several keywords. See details for available keywords.
 #'
@@ -166,7 +166,7 @@
 #'
 #' ## load example data
 #' data(ExampleData.DeValues, envir = environment())
-#' ExampleData.DeValues <- Second2Gray(
+#' ExampleData.DeValues <- convert_Second2Gray(
 #'   ExampleData.DeValues$BT998, c(0.0438,0.0019))
 #'
 #' ## plot the example data straightforward
@@ -208,7 +208,7 @@
 #' ## now with legend, colour, different points and smaller scale
 #' plot_RadialPlot(
 #'   data = ExampleData.DeValues,
-#'   legend.text = "Sample 1",
+#'   legend = "Sample 1",
 #'   col = "tomato4",
 #'   bar.col = "peachpuff",
 #'   pch = "R",
@@ -275,8 +275,8 @@ plot_RadialPlot <- function(
   central.value,
   centrality = "mean.weighted",
   mtext,
-  summary,
-  summary.pos,
+  summary = c("n", "in.2s"),
+  summary.pos = "sub",
   legend,
   legend.pos,
   stats,
@@ -294,8 +294,13 @@ plot_RadialPlot <- function(
   .set_function_name("plot_RadialPlot")
   on.exit(.unset_function_name(), add = TRUE)
 
-  if (is(data, "list") && length(data) == 0) {
-    .throw_error("'data' is an empty list")
+  ## Integrity checks -------------------------------------------------------
+
+  .validate_not_empty(data)
+  .validate_class(centrality, c("character", "numeric"))
+  if (is.character(centrality)) {
+    centrality <- .validate_args(centrality, c("mean", "mean.weighted",
+                                               "median", "median.weighted"))
   }
 
   ## Homogenise input data format
@@ -303,18 +308,16 @@ plot_RadialPlot <- function(
 
   ## Check input data
   for(i in 1:length(data)) {
-    if(is(data[[i]], "RLum.Results") == FALSE &
-         is(data[[i]], "data.frame") == FALSE) {
-      .throw_error("Error: Input data must be 'data.frame' or 'RLum.Results'")
-    } else {
-      if(is(data[[i]], "RLum.Results") == TRUE) {
-        data[[i]] <- get_RLum(data[[i]], "data")
-      }
+    .validate_class(data[[i]], c("data.frame", "RLum.Results"),
+                    name = "All elements of 'data'")
 
-      ## ensure that the dataset it not degenerate
-      if (nrow(data[[i]]) == 0) {
-       .throw_error("Input data ", i, " has 0 rows")
-      }
+    if (inherits(data[[i]], "RLum.Results")) {
+        data[[i]] <- get_RLum(data[[i]], "data")
+    }
+
+    ## ensure that the dataset it not degenerate
+    .validate_not_empty(data[[i]],
+                        name = paste0("Input 'data[[", i, "]]'"))
 
       ## if `data[[i]]` is a single-column data frame, append a second
       ## column with a small non-zero value (10^-9 for consistency with
@@ -325,18 +328,21 @@ plot_RadialPlot <- function(
         ## keep only the first two columns
         data[[i]] <- data[[i]][, 1:2]
       }
-    }
   }
 
-  ## check data and parameter consistency--------------------------------------
+  .validate_class(summary, "character")
+  .validate_class(summary.pos, c("numeric", "character"))
+  if (is.numeric(summary.pos)) {
+    .validate_length(summary.pos, 2)
+  }
+  else {
+    .validate_args(summary.pos, c("sub", "left", "center", "right",
+                                  "topleft", "top", "topright",
+                                  "bottomleft", "bottom", "bottomright"))
+  }
+
   if(missing(stats) == TRUE) {stats <- numeric(0)}
-  if(missing(summary) == TRUE) {
-    summary <- c("n", "in.2s")
-  }
 
-  if(missing(summary.pos) == TRUE) {
-    summary.pos <- "sub"
-  }
   if(missing(bar.col) == TRUE) {
     bar.col <- rep("grey80", length(data))
   }
@@ -383,37 +389,32 @@ plot_RadialPlot <- function(
                   (1.1 + z.span) * max(De.global))
   }
 
-  ## calculate correction dose to shift negative values
-  if(min(De.global) < 0 && log.z) {
+  ## calculate correction dose to shift non-positive values
+  De.add <- 0
+  if (log.z && min(De.global) <= 0) {
     if("zlim" %in% names(extraArgs)) {
       De.add <- abs(extraArgs$zlim[1])
     } else {
+      ## exclude zeros, as they cause infinities when logged
+      De.global.not0 <- De.global[De.global != 0]
+
       ## estimate delta De to add to all data
-      De.add <-  min(10^ceiling(log10(abs(De.global))) * 10)
+      De.add <- min(10^ceiling(log10(abs(De.global.not0))) * 10)
 
       ## optionally readjust delta De for extreme values
-      if(De.add <= abs(min(De.global))) {
+      if (De.add <= abs(min(De.global.not0))) {
         De.add <- De.add * 10
       }
     }
-  } else {
-    De.add <- 0
-  }
 
-  ticks <- round(pretty(limits.z, n = 5), 3)
-  De.delta <- ticks[2] - ticks[1]
-
-  ## optionally add correction dose to data set and adjust error
-  if(log.z) {
+    ## add correction dose to data set and adjust error
     for(i in 1:length(data))
       data[[i]][,1] <- data[[i]][,1] + De.add
 
     De.global <- De.global + De.add
-
   }
 
   ## calculate major preliminary tick values and tick difference
-  extraArgs <- list(...)
   if("zlim" %in% names(extraArgs)) {
     limits.z <- extraArgs$zlim
   } else {
@@ -438,11 +439,7 @@ plot_RadialPlot <- function(
   ## calculate se-values based on log-option
   se <- lapply(1:length(data), function(x, De.add){
     if(log.z == TRUE) {
-      if(De.add != 0) {
-        data[[x]][,2] <- data[[x]][,2] / (data[[x]][,1] + De.add)
-      } else {
-        data[[x]][,2] / data[[x]][,1]
-      }
+      data[[x]][,2] <- data[[x]][,2] / (data[[x]][,1] + De.add)
     } else {
       data[[x]][,2]
     }}, De.add = De.add)
@@ -451,6 +448,24 @@ plot_RadialPlot <- function(
     cbind(data[[x]], se[[x]])})
   rm(se)
 
+  ## define function after isotone::weighted.mean
+  median.w <- function (y, w) {
+    ox <- order(y)
+    y <- y[ox]
+    w <- w[ox]
+    k <- 1
+    low <- cumsum(c(0, w))
+    up <- sum(w) - low
+    df <- low - up
+    repeat {
+      if (df[k] < 0)
+        k <- k + 1
+      else if (df[k] == 0)
+        return((w[k] * y[k] + w[k - 1] * y[k - 1]) / (w[k] + w[k - 1]))
+      else return(y[k - 1])
+    }
+  }
+
   ## calculate central values
   if(centrality[1] == "mean") {
     z.central <- lapply(1:length(data), function(x){
@@ -463,24 +478,6 @@ plot_RadialPlot <- function(
       sum(data[[x]][,3] / data[[x]][,4]^2) /
         sum(1 / data[[x]][,4]^2)})
   } else if(centrality[1] == "median.weighted") {
-    ## define function after isotone::weighted.median
-    median.w <- function (y, w)
-    {
-      ox <- order(y)
-      y <- y[ox]
-      w <- w[ox]
-      k <- 1
-      low <- cumsum(c(0, w))
-      up <- sum(w) - low
-      df <- low - up
-      repeat {
-        if (df[k] < 0)
-          k <- k + 1
-        else if (df[k] == 0)
-          return((w[k] * y[k] + w[k - 1] * y[k - 1]) / (w[k] + w[k - 1]))
-        else return(y[k - 1])
-      }
-    }
     z.central <- lapply(1:length(data), function(x){
       rep(median.w(y = data[[x]][,3],
                    w = data[[x]][,4]), length(data[[x]][,3]))})
@@ -496,8 +493,6 @@ plot_RadialPlot <- function(
               length(centrality) > length(data)) {
     z.central <- lapply(1:length(data), function(x){
       rep(median(data[[x]][,3], na.rm = TRUE), length(data[[x]][,3]))})
-  } else {
-    .throw_error("Measure of centrality not supported")
   }
 
   data <- lapply(1:length(data), function(x) {
@@ -553,25 +548,6 @@ if(centrality[1] == "mean") {
   z.central.global <- sum(data.global[,3] / data.global[,4]^2) /
     sum(1 / data.global[,4]^2)
 } else if(centrality[1] == "median.weighted") {
-  ## define function after isotone::weighted.mean
-  median.w <- function (y, w)
-  {
-    ox <- order(y)
-    y <- y[ox]
-    w <- w[ox]
-    k <- 1
-    low <- cumsum(c(0, w))
-    up <- sum(w) - low
-    df <- low - up
-    repeat {
-      if (df[k] < 0)
-        k <- k + 1
-      else if (df[k] == 0)
-        return((w[k] * y[k] + w[k - 1] * y[k - 1])/(w[k] + w[k - 1]))
-
-      else return(y[k - 1])
-    }
-  }
   z.central.global <- median.w(y = data.global[,3], w = data.global[,4])
 } else if(is.numeric(centrality) == TRUE &
             length(centrality == length(data))) {
@@ -621,7 +597,6 @@ if(centrality[1] == "mean") {
   }
 
   ## read out additional arguments---------------------------------------------
-  extraArgs <- list(...)
 
   main <- if("main" %in% names(extraArgs)) {extraArgs$main} else
     {expression(paste(D[e], " distribution"))}
@@ -629,15 +604,14 @@ if(centrality[1] == "mean") {
   sub <- if("sub" %in% names(extraArgs)) {extraArgs$sub} else {""}
 
   if("xlab" %in% names(extraArgs)) {
-    if(length(extraArgs$xlab) != 2) {
-      .throw_error("'xlab' must have length 2")
-    } else {xlab <- extraArgs$xlab}
+    xlab <- extraArgs$xlab
+    .validate_length(xlab,  2)
   } else {
     xlab <- c(if(log.z == TRUE) {
-      "Relative standard error (%)"
+        "Relative standard error [%]"
       } else {
         "Standard error"
-        },
+      },
       "Precision")
   }
 
@@ -660,7 +634,6 @@ if(centrality[1] == "mean") {
     z.span <- ifelse(z.span > 1, 0.9, z.span)
     limits.z <- c((0.9 - z.span) * min(data.global[[1]]),
                   (1.1 + z.span) * max(data.global[[1]]))
-
   }
 
   if("xlim" %in% names(extraArgs)) {
@@ -684,36 +657,42 @@ if(centrality[1] == "mean") {
   }
 
   cex <- if("cex" %in% names(extraArgs)) {
+    .validate_length(extraArgs$cex, 1, name = "'cex'")
     extraArgs$cex
   } else {
     1
   }
 
   lty <- if("lty" %in% names(extraArgs)) {
+    .validate_length(extraArgs$lty, length(data), name = "'lty'")
     extraArgs$lty
     } else {
       rep(2, length(data))
     }
 
   lwd <- if("lwd" %in% names(extraArgs)) {
+    .validate_length(extraArgs$lwd, length(data), name = "'lwd'")
     extraArgs$lwd
     } else {
       rep(1, length(data))
     }
 
   pch <- if("pch" %in% names(extraArgs)) {
+    .validate_length(extraArgs$pch, length(data), name = "'pch'")
     extraArgs$pch
     } else {
       rep(1, length(data))
     }
 
   col <- if("col" %in% names(extraArgs)) {
+    .validate_length(extraArgs$col, length(data), name = "'col'")
     extraArgs$col
     } else {
       1:length(data)
     }
 
   tck <- if("tck" %in% names(extraArgs)) {
+    .validate_length(extraArgs$tck, length(data), name = "'tck'")
     extraArgs$tck
   } else {
     NA
@@ -786,7 +765,6 @@ if(centrality[1] == "mean") {
   ticks.major <- cbind(0,
     tick.x1.major, tick.x2.major, tick.y1.major, tick.y2.major)
 
-
   ## calculate minor z-tick coordinates
   tick.x1.minor <- r / sqrt(1 + f^2 * (
     tick.values.minor - z.central.global)^2)
@@ -810,7 +788,6 @@ if(centrality[1] == "mean") {
 
   } else {
     label.z.text <- signif(tick.values.major, 3)
-
   }
 
   ## subtract De.add from label values
@@ -940,345 +917,90 @@ if(centrality[1] == "mean") {
                               to = limits.z[2]),
                       silent = TRUE)
 
+    De.stats[i,6] <- NA
     if(!inherits(De.density, "try-error")) {
-      De.stats[i,6] <- NA
-
-    } else {
       De.stats[i,6] <- De.density$x[which.max(De.density$y)]
     }
   }
 
-  label.text = list(NA)
+  ## helper to generate an element of the statistical summary
+  .summary_line <- function(keyword, summary, val, label = keyword,
+                            percent = FALSE, sep = FALSE, digits = 2) {
+    ifelse(keyword %in% summary,
+           paste0(label, " = ", round(val, digits),
+                  if (percent) " %" else NULL, if (sep) " | " else "\n"),
+           "")
+  }
 
-  if(summary.pos[1] != "sub") {
-    n.rows <- length(summary)
+  ## initialize list with a dummy element, it will be removed afterwards
+  label.text <- list(NA)
 
-    for(i in 1:length(data)) {
-      stops <- paste(rep("\n", (i - 1) * n.rows), collapse = "")
+  is.sub <- summary.pos[1] == "sub"
+  stops <- NULL
+  for (i in 1:length(data)) {
+    if (!is.sub)
+      stops <- paste(rep("\n", (i - 1) * length(summary)), collapse = "")
 
-      summary.text <- character(0)
-
-      for(j in 1:length(summary)) {
-        summary.text <- c(summary.text,
-                          paste(
-                            "",
-                            ifelse("n" %in% summary[j] == TRUE,
-                                   paste("n = ",
-                                         De.stats[i,1],
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("mean" %in% summary[j] == TRUE,
-                                   paste("mean = ",
-                                         round(De.stats[i,2], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("mean.weighted" %in% summary[j] == TRUE,
-                                   paste("weighted mean = ",
-                                         round(De.stats[i,3], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("median" %in% summary[j] == TRUE,
-                                   paste("median = ",
-                                         round(De.stats[i,4], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("median.weighted" %in% summary[j] == TRUE,
-                                   paste("weighted median = ",
-                                         round(De.stats[i,5], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("kdemax" %in% summary[j] == TRUE,
-                                   paste("kdemax = ",
-                                         round(De.stats[i,6], 2),
-                                         " \n ",
-                                         sep = ""),
-                                   ""),
-                            ifelse("sdabs" %in% summary[j] == TRUE,
-                                   paste("sd = ",
-                                         round(De.stats[i,7], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("sdrel" %in% summary[j] == TRUE,
-                                   paste("rel. sd = ",
-                                         round(De.stats[i,8], 2), " %",
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("seabs" %in% summary[j] == TRUE,
-                                   paste("se = ",
-                                         round(De.stats[i,9], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("serel" %in% summary[j] == TRUE,
-                                   paste("rel. se = ",
-                                         round(De.stats[i,10], 2), " %",
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("skewness" %in% summary[j] == TRUE,
-                                   paste("skewness = ",
-                                         round(De.stats[i,13], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("kurtosis" %in% summary[j] == TRUE,
-                                   paste("kurtosis = ",
-                                         round(De.stats[i,14], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("in.2s" %in% summary[j] == TRUE,
-                                   paste("in 2 sigma = ",
-                                         round(sum(data[[i]][,7] > -2 &
-                                                     data[[i]][,7] < 2) /
-                                                 nrow(data[[i]]) * 100 , 1),
-                                         " %",
-                                         sep = ""),
-                                   ""),
-                            ifelse("sdabs.weighted" %in% summary[j] == TRUE,
-                                   paste("abs. weighted sd = ",
-                                         round(De.stats[i,15], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("sdrel.weighted" %in% summary[j] == TRUE,
-                                   paste("rel. weighted sd = ",
-                                         round(De.stats[i,16], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("seabs.weighted" %in% summary[j] == TRUE,
-                                   paste("abs. weighted se = ",
-                                         round(De.stats[i,17], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("serel.weighted" %in% summary[j] == TRUE,
-                                   paste("rel. weighted se = ",
-                                         round(De.stats[i,18], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            sep = ""))
-      }
-
-      summary.text <- paste(summary.text, collapse = "")
-
-      label.text[[length(label.text) + 1]] <- paste(stops,
-                                                    summary.text,
-                                                    stops,
-                                                    sep = "")
+    summary.text <- character(0)
+    for (j in 1:length(summary)) {
+      summary.text <-
+        c(summary.text,
+          .summary_line("n", summary[j], De.stats[i, 1], sep = is.sub),
+          .summary_line("mean", summary[j], De.stats[i, 2], sep = is.sub),
+          .summary_line("mean.weighted", summary[j], De.stats[i, 3], sep = is.sub,
+                        label = "weighted mean"),
+          .summary_line("median", summary[j], De.stats[i, 4], sep = is.sub),
+          .summary_line("median.weighted", summary[j], De.stats[i, 5], sep = is.sub,
+                        label = "weighted median"),
+          .summary_line("kdemax", summary[j], De.stats[i, 6], sep = is.sub),
+          .summary_line("sdabs", summary[j], De.stats[i, 7], sep = is.sub,
+                        label = "sd"),
+          .summary_line("sdrel", summary[j], De.stats[i, 8], sep = is.sub,
+                        label = "rel. sd", percent = TRUE),
+          .summary_line("seabs", summary[j], De.stats[i, 9], sep = is.sub,
+                        label = "se"),
+          .summary_line("serel", summary[j], De.stats[i, 10], sep = is.sub,
+                        label = "rel. se", percent = TRUE),
+          .summary_line("skewness", summary[j], De.stats[i, 13], sep = is.sub),
+          .summary_line("kurtosis", summary[j], De.stats[i, 14], sep = is.sub),
+          .summary_line("in.2s", summary[j],
+                        sum(data[[i]][,7] > -2 & data[[i]][,7] < 2) /
+                        nrow(data[[i]]) * 100, sep = is.sub,
+                        label = "in 2 sigma", percent = TRUE, digits = 1),
+          .summary_line("sdabs.weighted", summary[j], De.stats[i, 15], sep = is.sub,
+                        label = "abs. weighted sd"),
+          .summary_line("sdrel.weighted", summary[j], De.stats[i, 16], sep = is.sub,
+                        label = "rel. weighted sd"),
+          .summary_line("seabs.weighted", summary[j], De.stats[i, 17], sep = is.sub,
+                        label = "abs. weighted se"),
+          .summary_line("serel.weighted", summary[j], De.stats[i, 18], sep = is.sub,
+                        label = "rel. weighted se"))
     }
-  } else {
-    for(i in 1:length(data)) {
+    label.text[[length(label.text) + 1]] <- paste0(
+        if (is.sub ) "" else stops,
+        paste(summary.text, collapse = ""),
+        stops)
+  }
 
-      summary.text <- character(0)
+  ## remove dummy list element
+  label.text[[1]] <- NULL
 
-      for(j in 1:length(summary)) {
-        summary.text <- c(summary.text,
-                          ifelse("n" %in% summary[j] == TRUE,
-                                 paste("n = ",
-                                       De.stats[i,1],
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("mean" %in% summary[j] == TRUE,
-                                 paste("mean = ",
-                                       round(De.stats[i,2], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("mean.weighted" %in% summary[j] == TRUE,
-                                 paste("weighted mean = ",
-                                       round(De.stats[i,3], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("median" %in% summary[j] == TRUE,
-                                 paste("median = ",
-                                       round(De.stats[i,4], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("median.weighted" %in% summary[j] == TRUE,
-                                 paste("weighted median = ",
-                                       round(De.stats[i,5], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("kdemax" %in% summary[j] == TRUE,
-                                 paste("kdemax = ",
-                                       round(De.stats[i,6], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("sdrel" %in% summary[j] == TRUE,
-                                 paste("rel. sd = ",
-                                       round(De.stats[i,8], 2), " %",
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("sdabs" %in% summary[j] == TRUE,
-                                 paste("abs. sd = ",
-                                       round(De.stats[i,7], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("serel" %in% summary[j] == TRUE,
-                                 paste("rel. se = ",
-                                       round(De.stats[i,10], 2), " %",
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("seabs" %in% summary[j] == TRUE,
-                                 paste("abs. se = ",
-                                       round(De.stats[i,9], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("skewness" %in% summary[j] == TRUE,
-                                 paste("skewness = ",
-                                       round(De.stats[i,13], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("kurtosis" %in% summary[j] == TRUE,
-                                 paste("kurtosis = ",
-                                       round(De.stats[i,14], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("in.2s" %in% summary[j] == TRUE,
-                                 paste("in 2 sigma = ",
-                                       round(sum(data[[i]][,7] > -2 &
-                                                   data[[i]][,7] < 2) /
-                                               nrow(data[[i]]) * 100 , 1),
-                                       " %   ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("sdabs.weighted" %in% summary[j] == TRUE,
-                                 paste("abs. weighted sd = ",
-                                       round(De.stats[i,15], 2), " %",
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("sdrel.weighted" %in% summary[j] == TRUE,
-                                 paste("rel. weighted sd = ",
-                                       round(De.stats[i,16], 2), " %",
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("seabs.weighted" %in% summary[j] == TRUE,
-                                 paste("abs. weighted se = ",
-                                       round(De.stats[i,17], 2), " %",
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("serel.weighted" %in% summary[j] == TRUE,
-                                 paste("rel. weighted se = ",
-                                       round(De.stats[i,18], 2), " %",
-                                       " | ",
-                                       sep = ""),
-                                 "")
-        )
-      }
-
-      summary.text <- paste(summary.text, collapse = "")
-
-      label.text[[length(label.text) + 1]]  <- paste(
-        "  ",
-        summary.text,
-        sep = "")
-    }
-
-    ## remove outer vertical lines from string
-    for(i in 2:length(label.text)) {
+  ## remove outer vertical lines from string
+  if (is.sub) {
+    for (i in seq_along(label.text)) {
       label.text[[i]] <- substr(x = label.text[[i]],
-                                start = 3,
+                                start = 1,
                                 stop = nchar(label.text[[i]]) - 3)
     }
   }
 
-## remove dummy list element
-label.text[[1]] <- NULL
-  ## convert keywords into summary placement coordinates
-  if(missing(summary.pos) == TRUE) {
-    summary.pos <- c(limits.x[1], limits.y[2])
-    summary.adj <- c(0, 1)
-  } else if(length(summary.pos) == 2) {
-    summary.pos <- summary.pos
-    summary.adj <- c(0, 1)
-  } else if(summary.pos[1] == "topleft") {
-    summary.pos <- c(limits.x[1], limits.y[2])
-    summary.adj <- c(0, 1)
-  } else if(summary.pos[1] == "top") {
-    summary.pos <- c(mean(limits.x), limits.y[2])
-    summary.adj <- c(0.5, 1)
-  } else if(summary.pos[1] == "topright") {
-    summary.pos <- c(limits.x[2], limits.y[2])
-    summary.adj <- c(1, 1)
-  }  else if(summary.pos[1] == "left") {
-    summary.pos <- c(limits.x[1], mean(limits.y))
-    summary.adj <- c(0, 0.5)
-  } else if(summary.pos[1] == "center") {
-    summary.pos <- c(mean(limits.x), mean(limits.y))
-    summary.adj <- c(0.5, 0.5)
-  } else if(summary.pos[1] == "right") {
-    summary.pos <- c(limits.x[2], mean(limits.y))
-    summary.adj <- c(1, 0.5)
-  }else if(summary.pos[1] == "bottomleft") {
-    summary.pos <- c(limits.x[1], limits.y[1])
-    summary.adj <- c(0, 0)
-  } else if(summary.pos[1] == "bottom") {
-    summary.pos <- c(mean(limits.x), limits.y[1])
-    summary.adj <- c(0.5, 0)
-  } else if(summary.pos[1] == "bottomright") {
-    summary.pos <- c(limits.x[2], limits.y[1])
-    summary.adj <- c(1, 0)
-  }
-
-  ## convert keywords into legend placement coordinates
-  if(missing(legend.pos) == TRUE) {
-    legend.pos <- c(limits.x[1], limits.y[2])
-    legend.adj <- c(0, 1)
-  } else if(length(legend.pos) == 2) {
-    legend.pos <- legend.pos
-    legend.adj <- c(0, 1)
-  } else if(legend.pos[1] == "topleft") {
-    legend.pos <- c(limits.x[1], limits.y[2])
-    legend.adj <- c(0, 1)
-  } else if(legend.pos[1] == "top") {
-    legend.pos <- c(mean(limits.x), limits.y[2])
-    legend.adj <- c(0.5, 1)
-  } else if(legend.pos[1] == "topright") {
-    legend.pos <- c(limits.x[2], limits.y[2])
-    legend.adj <- c(1, 1)
-  } else if(legend.pos[1] == "left") {
-    legend.pos <- c(limits.x[1], mean(limits.y))
-    legend.adj <- c(0, 0.5)
-  } else if(legend.pos[1] == "center") {
-    legend.pos <- c(mean(limits.x), mean(limits.y))
-    legend.adj <- c(0.5, 0.5)
-  } else if(legend.pos[1] == "right") {
-    legend.pos <- c(limits.x[2], mean(limits.y))
-    legend.adj <- c(1, 0.5)
-  } else if(legend.pos[1] == "bottomleft") {
-    legend.pos <- c(limits.x[1], limits.y[1])
-    legend.adj <- c(0, 0)
-  } else if(legend.pos[1] == "bottom") {
-    legend.pos <- c(mean(limits.x), limits.y[1])
-    legend.adj <- c(0.5, 0)
-  } else if(legend.pos[1] == "bottomright") {
-    legend.pos <- c(limits.x[2], limits.y[1])
-    legend.adj <- c(1, 0)
-  }
+  ## convert keywords into summary and legend placement coordinates
+  coords <- .get_keyword_coordinates(summary.pos, limits.x, limits.y)
+  summary.pos <- coords$pos
+  summary.adj <- coords$adj
+  coords <- .get_keyword_coordinates(legend.pos, limits.x, limits.y)
+  legend.pos <- coords$pos
+  legend.adj <- coords$adj
 
   ## calculate line coordinates and further parameters
   if(!missing(line)) {
@@ -1452,18 +1174,11 @@ label.text[[1]] <- NULL
     axis(side = 1, tcl = -0.5, lwd = 0, lwd.ticks = 1, at = limits.x[2],
          labels = FALSE)
 
-    ## add upper axis label
-    mtext(text = xlab[1],
+    ## add upper and lower axis label
+    mtext(text = xlab,
           at = (limits.x[1] + limits.x[2]) / 2,
           side = 1,
-          line = -3.5,
-          cex = cex)
-
-    ## add lower axis label
-    mtext(text = xlab[2],
-          at = (limits.x[1] + limits.x[2]) / 2,
-          side = 1,
-          line = 2.5,
+          line = c(-3.5, 2.5),
           cex = cex)
 
     ## plot upper x-axis
@@ -1549,14 +1264,14 @@ label.text[[1]] <- NULL
              pch = pch,
              col = col,
              text.col = col,
-             cex = 0.8 * cex,
+             cex = 0.8,
              bty = "n")
     }
 
     ## plot y-axis
     if(y.ticks == TRUE) {
       char.height <- par()$cxy[2]
-      tick.space <- axisTicks(usr = limits.y, log = FALSE)
+      tick.space <- grDevices::axisTicks(usr = limits.y, log = FALSE)
       tick.space <- (max(tick.space) - min(tick.space)) / length(tick.space)
       if(tick.space < char.height * 1.5) {
         axis(side = 2, at = c(-2, 2), labels = c("", ""), las = 1)
@@ -1612,5 +1327,4 @@ label.text[[1]] <- NULL
                 polygons = polygons,
                 ellipse.lims = ellipse.lims))
   }
-
 }
