@@ -1,8 +1,9 @@
-#' Plot single luminescence curves from a BIN file object
+#' @title Plot Single Luminescence Curves from a Risoe.BINfileData-class object
 #'
-#' Plots single luminescence curves from an object returned by the
-#' [read_BIN2R] function.
+#' @description Plots single luminescence curves from an object returned by the
+#' [read_BIN2R] function [Risoe.BINfileData-class]
 #'
+#' @details
 #' **Nomenclature**
 #'
 #' See [Risoe.BINfileData-class]
@@ -22,7 +23,7 @@
 #' done by using the 'Run Info' option within the Sequence Editor or by editing
 #' in R.
 #'
-#' @param BINfileData [Risoe.BINfileData-class] (**required**):
+#' @param data [Risoe.BINfileData-class] (**required**):
 #' requires an S4 object returned by the [read_BIN2R] function.
 #'
 #' @param position [vector] (*optional*):
@@ -49,8 +50,9 @@
 #'
 #' @param curve.transformation [character] (*optional*):
 #' allows transforming CW-OSL and CW-IRSL curves to pseudo-LM curves via
-#' transformation functions. Allowed values are: `CW2pLM`, `CW2pLMi`, `CW2pHMi` and
-#' `CW2pPMi`. See details.
+#' transformation functions. Allowed values are: `CW2pLM`, `CW2pLMi`,
+#' `CW2pHMi` and `CW2pPMi`, see details. If set to `None` (default), no
+#' transformation is applied.
 #'
 #' @param dose_rate [numeric] (*optional*):
 #' dose rate of the irradiation source at the measurement date.
@@ -70,14 +72,14 @@
 #' The function has been successfully tested for the Sequence Editor file
 #' output version 3 and 4.
 #'
-#' @section Function version: 0.4.1
+#' @section Function version: 0.4.2
 #'
 #' @author
 #' Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)\cr
 #' Michael Dietze, GFZ Potsdam (Germany)
 #'
-#' @seealso [Risoe.BINfileData-class],[read_BIN2R], [CW2pLM], [CW2pLMi],
-#' [CW2pPMi], [CW2pHMi]
+#' @seealso [Risoe.BINfileData-class], [read_BIN2R], [convert_CW2pLM],
+#' [convert_CW2pLMi], [convert_CW2pPMi], [convert_CW2pHMi]
 #'
 #' @references
 #' Duller, G., 2007. Analyst. pp. 1-45.
@@ -104,13 +106,13 @@
 #' @md
 #' @export
 plot_Risoe.BINfileData<- function(
-  BINfileData,
+  data,
   position,
   run,
   set,
   sorter = "POSITION",
   ltype = c("IRSL","OSL","TL","RIR","RBR","RL"),
-  curve.transformation,
+  curve.transformation = "None",
   dose_rate,
   temp.lab,
   cex.global = 1,
@@ -119,13 +121,18 @@ plot_Risoe.BINfileData<- function(
   .set_function_name("plot_Risoe.BINfileData")
   on.exit(.unset_function_name(), add = TRUE)
 
-  ##check if the object is of type Risoe.BINfileData
-  if(!inherits(BINfileData, "Risoe.BINfileData"))
-    .throw_error("'object' is expected to be of type 'Risoe.BINfileData'")
+  ## Integrity checks -------------------------------------------------------
 
-  temp<-BINfileData
+  .validate_class(data, "Risoe.BINfileData")
+  curve.transformation <- .validate_args(curve.transformation,
+                                         c("CW2pLM", "CW2pLMi",
+                                           "CW2pHMi", "CW2pPMi", "None"))
+  ## complete the function name
+  if (curve.transformation != "None") {
+    curve.transformation <- paste0("convert_", curve.transformation)
+  }
 
-  # Missing check ----------------------------------------------------------------
+  temp <- data
 
   ##set plot position if missing
   if(missing(position)==TRUE){position<-c(min(temp@METADATA[,"POSITION"]):max(temp@METADATA[,"POSITION"]))}
@@ -151,8 +158,6 @@ plot_Risoe.BINfileData<- function(
     temp@METADATA<-temp@METADATA[order(temp@METADATA[,"POSITION"]),]
   }
 
-
-
   # Select values for plotting ------------------------------------------------------------------
 
   ##(2) set SEL for selected position
@@ -171,7 +176,6 @@ plot_Risoe.BINfileData<- function(
   ##------------------------------------------------------------------------##
   ##(3) plot curves
   for(i in 1:length(temp@METADATA[,"ID"])){
-
     ##print only if SEL == TRUE
     if(temp@METADATA[i,"SEL"]==TRUE)
     {
@@ -186,29 +190,11 @@ plot_Risoe.BINfileData<- function(
       values.xy <- data.frame(values.x, values.y)
 
       ##set curve transformation if wanted
-      if((temp@METADATA[i,"LTYPE"] == "OSL" | temp@METADATA[i,"LTYPE"] == "IRSL") &
-           missing(curve.transformation) == FALSE){
+      if (grepl("IRSL|OSL", temp@METADATA[i, "LTYPE"]) &&
+          curve.transformation != "None") {
 
-        if(curve.transformation=="CW2pLM"){
-
-          values.xy <- CW2pLM(values.xy)
-
-        }else if(curve.transformation=="CW2pLMi"){
-
-          values.xy <- CW2pLMi(values.xy)[,1:2]
-
-        }else if(curve.transformation=="CW2pHMi"){
-
-          values.xy <- CW2pHMi(values.xy)[,1:2]
-
-        }else if(curve.transformation=="CW2pPMi"){
-
-          values.xy <- CW2pPMi(values.xy)[,1:2]
-
-        }else{
-          .throw_warning("Unknown 'curve.transformation', ",
-                         "no transformation performed")
-        }
+        ## get the actual function from the parameter value and apply it
+        values.xy <- get(curve.transformation)(values.xy)[, 1:2]
       }
 
       ##plot graph
@@ -232,8 +218,11 @@ plot_Risoe.BINfileData<- function(
 
       ##grep temperature (different for different verions)
 
-      temperature<-if(temp@METADATA[i,"VERSION"]=="03"){temp@METADATA[i,"AN_TEMP"]}
-      else{temp@METADATA[i,"TEMPERATURE"]}
+      temperature <- if(temp@METADATA[i,"VERSION"]=="03") {
+        temp@METADATA[i,"AN_TEMP"]
+        } else {
+          temp@METADATA[i,"TEMPERATURE"]
+        }
 
       ##mtext
       mtext(side=3,
