@@ -216,6 +216,7 @@ read_BIN2R <- function(
 
   ## Config -----------------------------------------------------------------
 
+  .validate_length(file, 1)
   .validate_logical_scalar(show.raw.values)
   .validate_logical_scalar(zero_data.rm)
   .validate_logical_scalar(duplicated.rm)
@@ -403,7 +404,7 @@ read_BIN2R <- function(
         strlen <- force.size
       return(suppressWarnings(readChar(raw[-1], strlen, useBytes = TRUE)))
     }
-    return("")
+    return("") # nocov
   }
 
   ##PRESET VALUES
@@ -1114,7 +1115,7 @@ read_BIN2R <- function(
 
   ##check for position that have no data at all (error during the measurement)
   if(zero_data.rm){
-    zero_data.check <- which(vapply(results.DATA, length, numeric(1)) == 0)
+    zero_data.check <- which(lengths(results.DATA) == 0)
 
     ##remove records if there is something to remove
     if (length(zero_data.check) > 0) {
