@@ -87,8 +87,9 @@
 #' count TL curve. Select `FALSE` to import the raw data delivered by the
 #' lexsyg. Works for TL curves and spectra.
 #'
-#' @param n_records [numeric] (*with default*): set the number of records to be imported; by default
-#' the function attempts to import all records
+#' @param n_records [numeric] (*with default*):
+#' number of records to be imported; by default the function attempts to import
+#' all records.
 #'
 #' @param fastForward [logical] (*with default*):
 #' if `TRUE` for a more efficient data processing only a list of [RLum.Analysis-class]
@@ -130,7 +131,7 @@
 #' Corresponding values in the XSXG file are skipped.
 #'
 #'
-#' @section Function version: 0.6.14
+#' @section Function version: 0.6.15
 #'
 #'
 #' @author
@@ -199,15 +200,16 @@ read_XSYG2R <- function(
   ##  - xlum should be general, xsyg should take care about subsequent details
 
   .validate_class(file, c("character", "list"))
-  .validate_class(n_records, c("numeric", "NULL"))
+  if (!is.null(n_records))
+    .validate_class(n_records, c("numeric", "integer"))
 
   # Self Call -----------------------------------------------------------------------------------
   # Option (a): Input is a list, every element in the list will be treated as file connection
   # with that many file can be read in at the same time
   # Option (b): The input is just a path, the function tries to grep ALL xsyg/XSYG files in the
   # directory and import them, if this is detected, we proceed as list
-  if(is(file, "character")) {
-    .validate_not_empty(file)
+  if (is.character(file)) {
+    .validate_length(file, 1)
 
     ##If this is not really a path we skip this here
     if (dir.exists(file) & length(dir(file)) > 0) {
@@ -280,14 +282,13 @@ read_XSYG2R <- function(
     ##1st grep wavelength table
     wavelength <- XML::xmlAttrs(curve.node)["wavelengthTable"]
 
-    ##string split
-    wavelength <- as.numeric(unlist(strsplit(wavelength, split = ";", fixed = TRUE)))
+    ##string split (scan is considerably faster by a factor of 2)
+    wavelength <- na.exclude(scan(text = wavelength, what = numeric(), sep = ";", quiet = TRUE))
 
     ##2nd grep time values
-    curve.node <- unlist(strsplit(XML::xmlValue(curve.node), split = ";", fixed = TRUE))
-    curve.node <- strsplit(curve.node, split = ",", fixed = TRUE)
-    curve.node.time <- as.numeric(vapply(curve.node, function(x) x[1], character(1)))
-
+    raw_curve_node <- XML::xmlValue(curve.node)
+    curve.node <- unlist(strsplit(raw_curve_node, split = "];", fixed = TRUE))
+    curve.node <- strsplit(curve.node, split = ",[", fixed = TRUE)
     curve.node.time <- as.numeric(vapply(curve.node, function(x) x[1], character(1)))
 
     ##3rd grep count values
@@ -298,16 +299,14 @@ read_XSYG2R <- function(
         x[3]
     }, character(1))
 
-    ##remove from pattern...
-    curve.node.count <- do.call(
-      what = "gsub", list(pattern="[[]|[]]", replacement="",
-                                             x=curve.node.count))
-
+    ## remove last bracket
+    curve.node.count[length(curve.node.count)] <- gsub(
+      pattern = "]", replacement = "", x = curve.node.count[length(curve.node.count)], fixed = TRUE)
 
     ##4th combine to spectrum matrix
     spectrum.matrix <- matrix(NA, nrow = length(wavelength), ncol = length(curve.node.time))
     for(i in 1:length(curve.node.time)) {
-      tmp <- as.numeric(unlist(strsplit(curve.node.count[i], "|", fixed = TRUE)))
+      tmp <- scan(text = curve.node.count[i], what = numeric(), sep = "|", quiet = TRUE)
       if(length(tmp) == length(wavelength))
         spectrum.matrix[,i] <- tmp
     }
@@ -374,10 +373,11 @@ read_XSYG2R <- function(
       }
 
     return(output)
+  }
+
+  ## ========================================================================
+  ## IMPORT XSYG FILE
 
-  } else {
-    ##==========================================================================##
-    ##IMPORT XSYG FILE
     if (verbose) {
       cat("\n[read_XSYG2R()] Importing ...")
       cat("\n path: ", dirname(file))
@@ -385,17 +385,16 @@ read_XSYG2R <- function(
       cat("\n")
     }
 
-    ##PROGRESS BAR
-    if(verbose && txtProgressBar){
-      pb <- txtProgressBar(min=0,max=XML::xmlSize(temp), char = "=", style=3)
-    }
+  ## set n_records
+  n_records <- min(XML::xmlSize(temp), n_records[1])
 
-    ## set n_records
-    if(is.null(n_records))
-      n_records <- XML::xmlSize(temp)
+  ## initialize the progress bar
+  if (verbose && txtProgressBar) {
+    pb <- txtProgressBar(min = 0, max = n_records, char = "=", style = 3)
+  }
 
-    ##loop over the entire sequence by sequence
-    output <- lapply(1:min(XML::xmlSize(temp),n_records[1]), function(x){
+  ## loop over the entire sequence up to the number of records to be read
+  output <- lapply(1:n_records, function(x) {
       ##read sequence header
       temp.sequence.header <- as.data.frame(XML::xmlAttrs(temp[[x]]), stringsAsFactors = FALSE)
 
@@ -414,7 +413,8 @@ read_XSYG2R <- function(
 
         ##the XSYG file might be broken due to a machine error during the measurement, this
         ##control flow helps; if a try-error is observed NULL is returned
-        if(!inherits(temp.sequence.object.recordType, "try-error")){
+        if (inherits(temp.sequence.object.recordType, "try-error"))
+          return(NULL)
 
          ##create a fallback, the function should not fail
          if(is.null(temp.sequence.object.recordType) || is.na(temp.sequence.object.recordType)){
@@ -422,18 +422,16 @@ read_XSYG2R <- function(
          }
 
         ##correct record type in depending on the stimulator
+        xml.size <- XML::xmlSize(temp[[x]][[i]])
         if(temp.sequence.object.recordType == "OSL"){
-          if(XML::xmlAttrs(temp[[x]][[i]][[
-            XML::xmlSize(temp[[x]][[i]])]])["stimulator"] == "ir_LED_850" |
-            XML::xmlAttrs(temp[[x]][[i]][[
-              XML::xmlSize(temp[[x]][[i]])]])["stimulator"] == "ir_LD_850"){
-
+          if (XML::xmlAttrs(temp[[x]][[i]][[xml.size]])["stimulator"] %in%
+              c("ir_LED_850", "ir_LD_850")) {
             temp.sequence.object.recordType  <- "IRSL"
           }
         }
 
         ##loop 3rd level
-        lapply(1:XML::xmlSize(temp[[x]][[i]]), function(j){
+        lapply(1:xml.size, function(j) {
           ##get values
           temp.sequence.object.curveValue <- temp[[x]][[i]][[j]]
           attrs <- XML::xmlAttrs(temp.sequence.object.curveValue)
@@ -451,7 +449,6 @@ read_XSYG2R <- function(
           temp.sequence.object.info <- c(as.list(attrs),
                                          position = as.integer(as.character(temp.sequence.header["position",])),
                                          name = as.character(temp.sequence.header["name",]))
-
           ## TL curve recalculation ============================================
           if(recalculate.TL.curves){
             ##TL curve heating values is stored in the 3rd curve of every set
@@ -461,48 +458,32 @@ read_XSYG2R <- function(
                 temp.sequence.object.curveValue.PMT <- src_get_XSYG_curve_values(XML::xmlValue(
                   temp[[x]][[i]][[j]]))
 
-                ##round values (1 digit is the technical resolution of the heating element)
-                temp.sequence.object.curveValue.PMT[,1] <- round(
-                  temp.sequence.object.curveValue.PMT[,1], digits = 1)
-
-                #grep values from heating element
-                temp.sequence.object.curveValue.heating.element <- src_get_XSYG_curve_values(XML::xmlValue(
-                  temp[[x]][[i]][[3]]))
-
+                time.values <- temp.sequence.object.curveValue.PMT[, 1]
               }else{
                 temp.sequence.object.curveValue.spectrum <- get_XSYG.spectrum.values(
                   temp.sequence.object.curveValue)
 
                 ##get time values which are stored in the row labels
-                temp.sequence.object.curveValue.spectrum.time <- as.numeric(
+                time.values <- as.numeric(
                   colnames(temp.sequence.object.curveValue.spectrum))
-
-                ##round values (1 digit is technical resolution of the heating element)
-                temp.sequence.object.curveValue.spectrum.time <- round(
-                  temp.sequence.object.curveValue.spectrum.time, digits = 1)
-
               }
 
+              ## round values (1 digit is technical resolution of the heating element)
+              time.values <- round(time.values, digits = 1)
+
               #grep values from heating element
               temp.sequence.object.curveValue.heating.element <- src_get_XSYG_curve_values(XML::xmlValue(
                 temp[[x]][[i]][[3]]))
 
               temp.element <- temp.sequence.object.curveValue.heating.element[, 1]
-              if(!"Spectrometer" %in% temp.sequence.object.detector){
-                #reduce matrix values to values of the detection
-                temp.sequence.object.curveValue.heating.element <-
+
+              ## reduce matrix values to values of the detection
+              temp.sequence.object.curveValue.heating.element <-
                   temp.sequence.object.curveValue.heating.element[
-                      temp.element >= min(temp.sequence.object.curveValue.PMT[, 1]) &
-                      temp.element <= max(temp.sequence.object.curveValue.PMT[, 1]), ,
+                      temp.element >= min(time.values) &
+                      temp.element <= max(time.values), ,
                       drop = FALSE]
-              }else{
-                #reduce matrix values to values of the detection
-                temp.sequence.object.curveValue.heating.element <-
-                  temp.sequence.object.curveValue.heating.element[
-                      temp.element >= min(temp.sequence.object.curveValue.spectrum.time) &
-                      temp.element <= max(temp.sequence.object.curveValue.spectrum.time), ,
-                      drop = FALSE]
-              }
+
               ## calculate corresponding heating rate, this makes only sense
               ## for linear heating, therefore is has to be the maximum value
 
@@ -533,7 +514,7 @@ read_XSYG2R <- function(
                   temp.sequence.object.curveValue.heating.element.i <- approx(
                     x = temp.sequence.object.curveValue.heating.element[,1],
                     y = temp.sequence.object.curveValue.heating.element[,2],
-                    xout = temp.sequence.object.curveValue.PMT[,1],
+                    xout = time.values,
                     rule = 2)
 
                   temperature.values <-
@@ -546,7 +527,7 @@ read_XSYG2R <- function(
                 }else if((nrow(temp.sequence.object.curveValue.PMT) <
                           nrow(temp.sequence.object.curveValue.heating.element))){
                   temp.sequence.object.curveValue.PMT.i <- approx(
-                    x = temp.sequence.object.curveValue.PMT[,1],
+                    x = time.values,
                     y = temp.sequence.object.curveValue.PMT[,2],
                     xout = temp.sequence.object.curveValue.heating.element[,1],
                     rule = 2)
@@ -578,12 +559,12 @@ read_XSYG2R <- function(
                 ##         Otherwise we would introduce some pseudo signals, as we have to
                 ##         take care of noise later one
 
-                if(length(temp.sequence.object.curveValue.spectrum.time) !=
-                   nrow(temp.sequence.object.curveValue.heating.element)){
+                if (length(time.values) !=
+                    nrow(temp.sequence.object.curveValue.heating.element)) {
                   temp.sequence.object.curveValue.heating.element.i <- approx(
                     x = temp.sequence.object.curveValue.heating.element[,1],
                     y = temp.sequence.object.curveValue.heating.element[,2],
-                    xout = temp.sequence.object.curveValue.spectrum.time,
+                    xout = time.values,
                     rule = 2,
                     ties = mean,
                     na.rm = FALSE)
@@ -617,56 +598,36 @@ read_XSYG2R <- function(
           }##endif recalculate.TL.curves == TRUE
 
 
-    # Cleanup info objects ------------------------------------------------------------------------
-    if("curveType" %in% names(temp.sequence.object.info))
-      temp.sequence.object.info[["curveType"]] <- NULL
-
-    # Set RLum.Data-objects -----------------------------------------------------------------------
-          if("Spectrometer" %in% temp.sequence.object.detector == FALSE){
-
-            if(is(temp.sequence.object.curveValue, "matrix") == FALSE){
+          ## Cleanup info objects -------------------------------------------
+          if ("curveType" %in% names(temp.sequence.object.info))
+            temp.sequence.object.info[["curveType"]] <- NULL
 
+          ## Set RLum.Data-objects ------------------------------------------
+          if (!"Spectrometer" %in% temp.sequence.object.detector) {
+            out.class <- "RLum.Data.Curve"
+            if (!inherits(temp.sequence.object.curveValue, "matrix")) {
               temp.sequence.object.curveValue <-
                 src_get_XSYG_curve_values(XML::xmlValue(temp.sequence.object.curveValue))
             }
 
-            set_RLum(
-              class = "RLum.Data.Curve",
-              originator = "read_XSYG2R",
-              recordType = paste(temp.sequence.object.recordType,
-                                 " (", temp.sequence.object.detector,")",
-                                 sep = ""),
-              curveType = temp.sequence.object.curveType,
-              data = temp.sequence.object.curveValue,
-              info = temp.sequence.object.info)
-
-          }else if("Spectrometer" %in% temp.sequence.object.detector == TRUE) {
-
-
-            if(is(temp.sequence.object.curveValue, "matrix") == FALSE){
-
+          } else {
+            out.class <- "RLum.Data.Spectrum"
+            if (!inherits(temp.sequence.object.curveValue, "matrix")) {
               temp.sequence.object.curveValue <-
                 get_XSYG.spectrum.values(temp.sequence.object.curveValue)
             }
+          }
 
-            set_RLum(
-              class = "RLum.Data.Spectrum",
+          set_RLum(
+              class = out.class,
               originator = "read_XSYG2R",
-              recordType = paste(temp.sequence.object.recordType,
-                                 " (",temp.sequence.object.detector,")",
-                                 sep = ""),
+              recordType = paste0(temp.sequence.object.recordType,
+                                  " (", temp.sequence.object.detector,")"),
               curveType = temp.sequence.object.curveType,
               data = temp.sequence.object.curveValue,
               info = temp.sequence.object.info)
-          }
         })
 
-        }else{
-
-         return(NULL)
-
-        }##if-try condition
-
       }),
        use.names = FALSE)
 
@@ -701,30 +662,21 @@ read_XSYG2R <- function(
         return(temp.sequence.object)
       }
 
-    })##end loop for sequence list
+  }) ##end loop for sequence list
 
-    ##close ProgressBar
-    if(verbose && txtProgressBar ){close(pb)}
+  ## close ProgressBar
+  if (verbose && txtProgressBar)
+    close(pb)
 
-    ##show output information
-    if(length(output[sapply(output, is.null)]) == 0){
-      if(verbose)
-        paste("\t >>",XML::xmlSize(temp), " sequence(s) loaded successfully.\n")
+  ## show output information
+  num.removed <- length(output[sapply(output, is.null)])
+  if (num.removed > 0)
+    .throw_warning(num.removed, " incomplete sequence(s) removed")
 
-    }else{
-
-      if(verbose){
-        paste("\t >>",XML::xmlSize(temp), " sequence(s) in file.", XML::xmlSize(temp)-length(output[sapply(output, is.null)]), "sequence(s) loaded successfully. \n")
-      }
-
-      .throw_warning(length(output[sapply(output, is.null)]),
-                     " incomplete sequence(s) removed.")
-    }
-
-    ##output
-    invisible(output)
-
-  }#end if
+  if (verbose) {
+    cat(sprintf("\t >> %d of %d sequence(s) loaded successfully\n",
+                n_records - num.removed, n_records))
+  }
 
   ##get rid of the NULL elements (as stated before ... invalid files)
   return(.rm_NULL_elements(output))
