@@ -1,9 +1,12 @@
-#' Verify single grain data sets and check for invalid grains, i.e.
+#' @title Verify single grain data sets and check for invalid grains, i.e.
 #' zero-light level grains
 #'
+#' @description
 #' This function tries to identify automatically zero-light level curves (grains)
 #' from single grain data measurements.
 #'
+#' @details
+#'
 #' **How does the method work?**
 #'
 #' The function compares the expected values (\eqn{E(X)}) and the variance
@@ -31,8 +34,11 @@
 #' numeric threshold value for the allowed difference between the `mean` and
 #' the `var` of the count values (see details)
 #'
+#' @param use_fft [logical] (*with default*): applies an additional approach based on [stats::fft].
+#' The threshold is fixed and cannot be changed.
+#'
 #' @param cleanup [logical] (*with default*):
-#' if set to `TRUE`, curves identified as zero light level curves are
+#' if set to `TRUE`, curves/aliquots identified as zero light level curves/aliquots are
 #' automatically removed. Output is an object as same type as the input, i.e.
 #' either [Risoe.BINfileData-class] or [RLum.Analysis-class]
 #'
@@ -46,10 +52,10 @@
 #' if all curves of this aliquot are marked as invalid.
 #'
 #' @param verbose [logical] (*with default*):
-#' enables or disables the terminal feedback
+#' enable/disables output to the terminal.
 #'
 #' @param plot [logical] (*with default*):
-#' enables or disables the graphical feedback
+#' enable/disable the plot output.
 #'
 #' @param ... further parameters to control the plot output; if selected.
 #' Supported arguments `main`, `ylim`
@@ -81,7 +87,7 @@
 #' For `cleanup = TRUE` the same object as the input is returned, but cleaned up
 #' (invalid curves were removed). This means: Either a [Risoe.BINfileData-class]
 #' or an [RLum.Analysis-class] object is returned in such cases.
-#' A [Risoe.BINfileData-class] object can be exported to a BIN-file by
+#' A [Risoe.BINfileData-class] object can be exported to a BINX-file by
 #' using the function [write_R2BIN].
 #'
 #' @note
@@ -93,9 +99,10 @@
 #'
 #' The function checking for invalid curves works rather robust and it is likely
 #' that Reg0 curves within a SAR cycle are removed as well. Therefore it is
-#' strongly recommended to use the argument `cleanup = TRUE` carefully.
+#' strongly recommended to use the argument `cleanup = TRUE` carefully if
+#' the cleanup works only on curves.
 #'
-#' @section Function version: 0.2.3
+#' @section Function version: 0.2.5
 #'
 #'
 #' @author
@@ -141,13 +148,15 @@
 verify_SingleGrainData <- function(
     object,
     threshold = 10,
+    use_fft = FALSE,
     cleanup = FALSE,
     cleanup_level = 'aliquot',
     verbose = TRUE,
     plot = FALSE,
     ...
-){
-
+) {
+  .set_function_name("verify_SingleGrainData")
+  on.exit(.unset_function_name(), add = TRUE)
 
   ##three types of input are allowed:
   ##(1) RisoeBINfileData
@@ -156,10 +165,14 @@ verify_SingleGrainData <- function(
 
   # Self Call -----------------------------------------------------------------------------------
   if(is(object, "list")){
+    if (length(object) == 0)
+      return(set_RLum(class = if (cleanup) "RLum.Analysis" else "RLum.Results"))
+
     results <- .warningCatcher(lapply(1:length(object), function(x) {
       verify_SingleGrainData(
         object = object[[x]],
         threshold = threshold,
+        use_fft = use_fft,
         cleanup = cleanup,
         cleanup_level = cleanup_level,
         verbose = verbose,
@@ -169,34 +182,57 @@ verify_SingleGrainData <- function(
     }))
 
     ##account for cleanup
-    if(cleanup){
+    if (cleanup) {
+      results <- .rm_NULL_elements(.rm_nonRLum(results))
+      if(length(results) == 0)
+        return(NULL)
+      else
       return(results)
 
     }else{
       return(merge_RLum(results))
-
     }
+  }
 
+  ## ------------------------------------------------------------------------
+  ## input validation
+
+  .validate_class(object, c("Risoe.BINfileData", "RLum.Analysis"),
+                  extra = "a 'list' of such objects")
+  .validate_class(threshold, c("numeric", "integer"))
+  .validate_logical_scalar(use_fft)
+  .validate_logical_scalar(cleanup)
+  cleanup_level <- .validate_args(cleanup_level, c("aliquot", "curve"))
+
+  ## implement Fourier Transform for Frequency Analysis
+  ## inspired by ChatGPT (OpenAI, 2024)
+  .calc_FFT_selection <- function(l, tmp_threshold = threshold/2) {
+    vapply(l, function(x){
+      x <- x[x>0]
+      tmp_power_spectrum <- Mod(stats::fft(x)^2)
+      tmp_mean_power <- mean(tmp_power_spectrum[-1])
+      tmp_dominant_power <- max(tmp_power_spectrum[2:(length(tmp_power_spectrum)/2)])
+      tmp_dominant_power > tmp_threshold * tmp_mean_power
+    }, logical(1))
   }
 
   ##++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   ##RisoeBINfileData
-  if(is(object, "Risoe.BINfileData")){
-
+  if(inherits(object, "Risoe.BINfileData")){
     ##run test on DATA slot
     ##MEAN + SD
-    temp.results_matrix <- lapply(X = object@DATA, FUN = function(x){
-      c(mean(x), var(x))
+    temp.results_matrix <- t(vapply(X = object@DATA, FUN = function(x){
+      c(mean(x), stats::var(x))
+    }, numeric(2)))
 
-    })
-
-    temp.results_matrix <- do.call(rbind,  temp.results_matrix)
 
     ##DIFF
     temp.results_matrix_RATIO <- temp.results_matrix[,2]/temp.results_matrix[,1]
 
     ##SEL
-    temp.results_matrix_VALID <- temp.results_matrix_RATIO > threshold
+    temp.results_matrix_VALID <-
+      temp.results_matrix_RATIO > threshold &
+      if (use_fft) .calc_FFT_selection(object@DATA) else TRUE
 
     ##combine everything to in a data.frame
     selection <- data.frame(
@@ -219,33 +255,30 @@ verify_SingleGrainData <- function(
           .subset2(selection, 1) == .subset2(unique_pairs, 1)[x] &
             .subset2(selection, 2) == .subset2(unique_pairs, 2)[x]
         )
-
-
       })))
 
-
     }else{
-
       ##reduce data to TRUE selection
       selection_id <- which(selection[["VALID"]])
-
     }
 
-
     ##select output on the chosen input
     if(cleanup){
       ##selected wanted elements
       object@DATA <- object@DATA[selection_id]
-      object@METADATA <- object@METADATA[selection_id,]
-      object@METADATA$ID <- 1:length(object@DATA)
-
-
-      ##print message
-      selection_id <- paste(selection_id, collapse = ", ")
-      if(verbose){
-        cat(paste0("\n[verify_SingleGrainData()] Risoe.BINfileData object reduced to records: \n", selection_id))
-        cat("\n\n[verify_SingleGrainData()] Risoe.BINfileData object record index reset.\n")
+      if(length(object@DATA) > 0) {
+        object@METADATA <- object@METADATA[selection_id,]
+        object@METADATA$ID <- 1:length(object@DATA)
 
+        ##print message
+        selection_id <- .collapse(selection_id, quote = FALSE)
+        if(verbose){
+          cat("\n[verify_SingleGrainData()] Risoe.BINfileData object reduced to records:\n",
+              selection_id)
+          cat("\n\n[verify_SingleGrainData()] Risoe.BINfileData object record index reset.\n")
+        }
+      } else {
+        object <- NULL
       }
 
       ##return
@@ -260,7 +293,6 @@ verify_SingleGrainData <- function(
           selection_full = selection),
         info = list(call = sys.call())
       )
-
     }
 
 
@@ -269,16 +301,14 @@ verify_SingleGrainData <- function(
     ## ... and yes it make sense not to mix that up with the code above
   }else if(is(object,"RLum.Analysis")){
     ##first extract all count values from all curves
-    object_list <- lapply(get_RLum(object), function(x){
+    object_list <- lapply(object@records, function(x){
       ##yes, would work differently, but it is faster
       x@data[,2]
-
     })
 
     ##MEAN + SD
     temp.results_matrix <- lapply(X = object_list, FUN = function(x){
-      c(mean(x), var(x))
-
+      c(mean(x), stats::var(x))
     })
 
     temp.results_matrix <- do.call(rbind,  temp.results_matrix)
@@ -287,14 +317,14 @@ verify_SingleGrainData <- function(
     temp.results_matrix_RATIO <- temp.results_matrix[,2]/temp.results_matrix[,1]
 
     ##SEL
-    temp.results_matrix_VALID <- temp.results_matrix_RATIO > threshold
+    temp.results_matrix_VALID <- temp.results_matrix_RATIO > threshold &
+      if (use_fft) .calc_FFT_selection(object_list) else TRUE
 
     ##get structure for the RLum.Analysis object
     temp_structure <- structure_RLum(object, fullExtent = TRUE)
 
     ##now we have two cases, depending on where measurement is coming from
     if (object@originator == "Risoe.BINfileData2RLum.Analysis") {
-
       ##combine everything to in a data.frame
       selection <- data.frame(
         POSITION = temp_structure$info.POSITION,
@@ -305,10 +335,7 @@ verify_SingleGrainData <- function(
         THRESHOLD = rep_len(threshold, length(object_list)),
         VALID = temp.results_matrix_VALID
       )
-
-      ##get unique pairs for POSITION and GRAIN for VALID == TRUE
-      unique_pairs <- unique(
-        selection[selection[["VALID"]], c("POSITION", "GRAIN")])
+      sel.cols <- c("POSITION", "GRAIN")
 
     } else if (object@originator == "read_XSYG2R") {
       ##combine everything to in a data.frame
@@ -324,36 +351,29 @@ verify_SingleGrainData <- function(
         THRESHOLD = rep_len(threshold, length(object_list)),
         VALID = temp.results_matrix_VALID
       )
-
-      ##get unique pairs for POSITION for VALID == TRUE
-      unique_pairs <- unique(
-        selection[["POSITION"]][selection[["VALID"]]])
+      sel.cols <- "POSITION"
 
     } else{
-
-      stop("[verify_SingleGrainData()] I don't know what to do object 'originator' not supported!",
-           call. = FALSE)
+      .throw_error("Object originator '", object@originator, "' not supported")
     }
 
+    ## get unique POSITION and GRAIN pairs where VALID == TRUE
+    unique_pairs <- unique(selection[selection[["VALID"]], sel.cols, drop = FALSE])
 
     ##set up cleanup
     if(cleanup_level == "aliquot") {
       if (object@originator == "read_XSYG2R") {
 
-        if(!is.na(unique_pairs)){
+        if (!all(is.na(unique_pairs))) {
           selection_id <-
             sort(unlist(lapply(1:nrow(unique_pairs), function(x) {
               which(.subset2(selection, 1) == .subset2(unique_pairs, 1)[x])
-
-
             })))
 
         }else{
           selection_id <- NA
-
         }
 
-
       } else if (object@originator == "Risoe.BINfileData2RLum.Analysis") {
         selection_id <-
           sort(unlist(lapply(1:nrow(unique_pairs), function(x) {
@@ -361,33 +381,28 @@ verify_SingleGrainData <- function(
               .subset2(selection, 1) == .subset2(unique_pairs, 1)[x] &
                 .subset2(selection, 2) == .subset2(unique_pairs, 2)[x]
             )
-
-
           })))
-
       }
 
       ##make sure that we do not break subsequent code
       if(length(selection_id) == 0) selection_id <- NA
 
-
     } else{
       ##reduce data to TRUE selection
       selection_id <- which(selection[["VALID"]])
-
     }
 
-
     ##return value
     ##select output on the chosen input
-    if(cleanup && !any(is.na(selection_id))){
-
+    if (cleanup && !anyNA(selection_id)) {
       ##print message
-      if(verbose){
-        selection_id_text <- paste(selection_id, collapse = ", ")
-        cat(paste0("\n[verify_SingleGrainData()] RLum.Analysis object reduced to records: ",
-                   selection_id_text), "\n")
+      if(verbose && cleanup_level == "curve"){
+        selection_id_text <- .collapse(selection_id, quote = FALSE)
+        if(selection_id_text == "")
+          selection_id_text <- "<none>"
 
+        message("[verify_SingleGrainData()] RLum.Analysis object reduced to records: ",
+                selection_id_text)
       }
 
       ##selected wanted elements
@@ -404,7 +419,6 @@ verify_SingleGrainData <- function(
         )
 
       } else{
-
         object <- set_RLum(
           class = "RLum.Analysis",
           records = get_RLum(object, record.id = selection_id, drop = FALSE),
@@ -413,18 +427,14 @@ verify_SingleGrainData <- function(
             selection_id = selection_id,
             selection_full = selection)
         )
-
       }
 
       ##return
       return_object <- object
 
     }else{
-      if(any(is.na(selection_id))){
-        warning("[verify_SingleGrainData()] selection_id is NA, nothing removed, everything selected for removal!",
-                call. = FALSE)
-
-      }
+      if (anyNA(selection_id))
+        .throw_warning("'selection_id' is NA, everything tagged for removal")
 
       return_object <- set_RLum(
         class = "RLum.Results",
@@ -435,12 +445,10 @@ verify_SingleGrainData <- function(
         info = list(call = sys.call())
       )
 
+      ## cleanup means cleanup
+      if (cleanup)
+        return_object <- NULL
     }
-
-
-  }else{
-    stop("[verify_SingleGrainData()] Input type '", is(object)[1],
-         "' is not allowed for this function!", call. = FALSE)
   }
 
   # Plot ----------------------------------------------------------------------------------------
@@ -453,8 +461,6 @@ verify_SingleGrainData <- function(
       ),
       val = list(...))
 
-
-
     ##plot area
     plot(
       NA,
@@ -482,9 +488,11 @@ verify_SingleGrainData <- function(
         " | valid: ", length(which(selection[["VALID"]])),
         " | invalid: ", length(which(!selection[["VALID"]])), ")"),
       cex = 0.9 * par()$cex)
-
   }
 
   # Return --------------------------------------------------------------------------------------
+  if(is.null(return_object))
+    .throw_warning("Verification and cleanup removed all records, NULL returned")
+
   return(return_object)
 }
