@@ -1,3 +1,4 @@
+## load data
 data(ExampleData.BINfileData, envir = environment())
 
 test_that("input validation", {
@@ -8,13 +9,13 @@ test_that("input validation", {
   expect_error(Risoe.BINfileData2RLum.Analysis(CWOSL.SAR.Data, pos = "test"),
                "'pos' should be of class 'numeric' or 'integer'")
   expect_error(Risoe.BINfileData2RLum.Analysis(CWOSL.SAR.Data, run = 10:12),
-               "run = 10, 11, 12 contains invalid runs")
+               "'run' contains invalid runs, valid runs are: 1, 2, 3, 4, 5, 6")
   expect_error(Risoe.BINfileData2RLum.Analysis(CWOSL.SAR.Data, set = 10:12),
-               "set = 10, 11, 12 contains invalid sets")
+               "'set' contains invalid sets, valid sets are: 2, 3, 5, 6")
   expect_error(Risoe.BINfileData2RLum.Analysis(CWOSL.SAR.Data, ltype = 10:12),
-               "ltype = '10', '11', '12' contains invalid ltypes")
+               "'ltype' contains invalid ltypes, valid ltypes are: 'TL', 'OSL'")
   expect_error(Risoe.BINfileData2RLum.Analysis(CWOSL.SAR.Data, dtype = 10:12),
-               "dtype = '10', '11', '12' contains invalid dtypes")
+               "'dtype' contains invalid dtypes, valid dtypes are: 'Natural'")
 
   expect_warning(Risoe.BINfileData2RLum.Analysis(CWOSL.SAR.Data, pos = 1:30),
                  "Invalid position number skipped")
