@@ -2,18 +2,28 @@ test_that("input validation", {
   testthat::skip_on_cran()
 
   expect_error(calc_AliquotSize(),
-               "Please provide the mean grain size or a range of grain sizes")
+               "Please provide the mean grain size or the range of grain sizes")
   expect_error(calc_AliquotSize(grain.size = 1:3),
-               "Please provide the mean grain size or a range of grain sizes")
+               "Please provide the mean grain size or the range of grain sizes")
+  expect_error(calc_AliquotSize(grain.size = 100, packing.density = "inf"),
+               "'packing.density' should be a positive scalar")
   expect_error(calc_AliquotSize(grain.size = 100, packing.density = 2),
-               "'packing.density' expects values between 0 and 1")
+               "'packing.density' should be a value between 0 and 1")
+
   expect_error(calc_AliquotSize(grain.size = 100, packing.density = 1, sample.diameter = -1),
                "'sample.diameter' should be a positive scalar")
+  expect_error(calc_AliquotSize(grain.size = 100, packing.density = 1,
+                                sample_carrier.diameter = -1),
+               "'sample_carrier.diameter' should be a positive scalar")
   expect_error(calc_AliquotSize(grain.size = 100, sample.diameter = 9.8,
                                 MC = TRUE),
                "'grain.size' must be a vector containing the min and max")
-  expect_output(calc_AliquotSize(grain.size = 100, packing.density = 1, sample.diameter = 9.8, grains.counted = 30, MC = TRUE),
-                regexp = "Monte Carlo simulation is only available for estimating the amount of grains on the sample disc.")
+  SW({
+  expect_message(calc_AliquotSize(grain.size = 100, packing.density = 1,
+                                  sample.diameter = 9.8, grains.counted = 30,
+                                  MC = TRUE),
+                 "Monte Carlo simulation is only available for estimating the")
+  })
 })
 
 test_that("check functionality", {
@@ -22,14 +32,14 @@ test_that("check functionality", {
   SW({
   expect_s4_class(
     calc_AliquotSize(
-      grain.size = 100, packing.density = "inf", sample.diameter = 9.8, MC = FALSE), "RLum.Results")
+      grain.size = 100, packing.density = Inf, sample.diameter = 9.8, MC = FALSE), "RLum.Results")
   expect_s4_class(
     calc_AliquotSize(
       grain.size = c(100, 150), grains.counted = 1000, sample.diameter = 9.8, MC = FALSE), "RLum.Results")
   expect_warning(calc_AliquotSize(
       grain.size = c(100, 150), grains.counted = c(1000, 1100, 900),
       sample.diameter = 10, MC = FALSE),
-      "A sample diameter of 10 mm was specified, but common sample discs are")
+      "A sample diameter of 10 mm was specified for a sample carrier of 9.8 mm")
   })
 
   set.seed(1)
@@ -37,7 +47,6 @@ test_that("check functionality", {
       grain.size = c(100, 150),
       sample.diameter = 1,
       MC.iter = 100,
-      plot = FALSE,
       verbose = FALSE)
 
   expect_equal(is(temp), c("RLum.Results", "RLum"))
@@ -79,9 +88,8 @@ test_that("snapshot tests", {
       plot = FALSE,
       verbose = TRUE),
       tolerance = snapshot.tolerance)
-  })
 
-  expect_output(
+  expect_message(
   expect_snapshot_RLum(calc_AliquotSize(
       grain.size = c(100, 200),
       sample.diameter = 8,
@@ -91,4 +99,14 @@ test_that("snapshot tests", {
       verbose = FALSE),
       tolerance = snapshot.tolerance),
   "Monte Carlo simulation is only available for estimating the amount of")
+
+  expect_snapshot_RLum(calc_AliquotSize(
+      grain.size = c(80, 150),
+      sample.diameter = 7,
+      sample_carrier.diameter = 7.2,
+      MC.iter = 20,
+      plot = FALSE,
+      verbose = TRUE),
+      tolerance = snapshot.tolerance)
+  })
 })
