@@ -92,11 +92,14 @@ test_that("check functionality", {
 
   ## RLum.Analysis object
   data(ExampleData.XSYG, envir = environment())
-  calc_FastRatio(get_RLum(OSL.SARMeasurement$Sequence.Object)[[1]])
+  calc_FastRatio(OSL.SARMeasurement$Sequence.Object)
 
   expect_warning(calc_FastRatio(get_RLum(TL.Spectrum)),
                  "L3 contains more counts (566) than L2 (562)",
                  fixed = TRUE)
+
+  ## RLum.Results object
+  calc_FastRatio(temp)
   })
 
   ## RLum.Data.Curve object
@@ -129,6 +132,17 @@ test_that("check functionality", {
   expect_equal(round(results$Cts_L3, digits = 5), 37.66667)
 })
 
+test_that("graphical snapshot tests", {
+  testthat::skip_on_cran()
+  testthat::skip_if_not_installed("vdiffr")
+  testthat::skip_if_not(getRversion() >= "4.4.0")
+
+  SW({
+  vdiffr::expect_doppelganger("FastRatio defaults",
+                              calc_FastRatio(ExampleData.CW_OSL_Curve))
+  })
+})
+
 test_that("regression tests", {
 
   ## issue 471 --------------------------------------------------------------
