@@ -1,16 +1,18 @@
-test_that("check class and length of output", {
+## load data
+data(ExampleData.DeValues, envir = environment())
+
+test_that("input validation", {
   testthat::skip_on_cran()
 
-  ##load example data
-  data(ExampleData.DeValues, envir = environment())
+  expect_error(calc_FuchsLang2001("error"),
+               "[calc_FuchsLang2001()] 'data' should be of class 'data.frame'",
+               fixed = TRUE)
+  expect_error(calc_FuchsLang2001(data.frame()),
+               "'data' cannot be an empty data.frame")
+})
 
-  ##break function
-  expect_error(calc_FuchsLang2001(
-    data = "ExampleData.DeValues$BT998",
-    cvThreshold = 5,
-    plot = FALSE,
-    verbose = FALSE),
-    "\\[calc_FuchsLang2001\\(\\)\\] 'data' has to be of type 'data.frame' or 'RLum.Results'\\!")
+test_that("check class and length of output", {
+  testthat::skip_on_cran()
 
   ##the simple and silent run
   temp <- expect_s4_class(
@@ -47,4 +49,9 @@ test_that("check class and length of output", {
 
     ), "RLum.Results")
   })
+
+  ## issue 818
+  expect_silent(calc_FuchsLang2001(data.frame(ED = c(rep(0, 4), 10),
+                                              ED_Error = rnorm(5) + 1),
+                                   verbose = FALSE))
 })
