@@ -1,51 +1,38 @@
-##preloads
+## load data
 data(ExampleData.LxTxOSLData, envir = environment())
-temp <- calc_OSLLxTxRatio(
-  Lx.data = Lx.data,
-  Tx.data = Tx.data,
-  signal.integral = c(1:2),
-  background.integral = c(85:100))
-
-test_that("check class and length of output", {
-  testthat::skip_on_cran()
-
-  expect_equal(is(temp), c("RLum.Results", "RLum"))
-  expect_equal(length(temp), 2)
-
-})
 
 test_that("test arguments", {
   testthat::skip_on_cran()
+  snapshot.tolerance <- 1.5e-6
 
   ##digits
-  expect_silent(calc_OSLLxTxRatio(
+  expect_snapshot_RLum(expect_silent(calc_OSLLxTxRatio(
     Lx.data,
     Tx.data,
     signal.integral = c(1:2),
     background.integral = c(85:100),
-    digits = 1))
+    digits = 1)
+    ), tolerance = snapshot.tolerance)
 
   ##sigmab
-  expect_silent(calc_OSLLxTxRatio(
+  expect_snapshot_RLum(expect_silent(calc_OSLLxTxRatio(
     Lx.data,
     Tx.data,
     signal.integral = c(1:2),
     background.integral = c(85:100),
-    sigmab = c(1000,100)
-    ))
+    sigmab = c(1000,100))
+    ), tolerance = snapshot.tolerance)
 
   ##poisson
-  expect_silent(calc_OSLLxTxRatio(
+  expect_snapshot_RLum(expect_silent(calc_OSLLxTxRatio(
     Lx.data,
     Tx.data,
     signal.integral = c(1:2),
     background.integral = c(85:100),
-    background.count.distribution = "poisson"
-  ))
-
+    background.count.distribution = "poisson")
+    ), tolerance = snapshot.tolerance)
 })
 
-
 test_that("test input", {
   testthat::skip_on_cran()
 
@@ -70,7 +57,6 @@ test_that("test input", {
     signal.integral = c(1:2),
     background.integral = c(70:100)))
 
-
   ##RLum.Curve
   expect_silent(calc_OSLLxTxRatio(
     set_RLum(class = "RLum.Data.Curve", data = as.matrix(Lx.data)),
@@ -91,47 +77,32 @@ test_that("test input", {
     Tx.data = NULL,
     signal.integral = c(1:2),
     background.integral = c(70:100)))
-
 })
 
-test_that("force function break", {
+test_that("input validation", {
   testthat::skip_on_cran()
 
-  expect_error(calc_OSLLxTxRatio(
-    Lx.data[1:10,],
-    Tx.data,
-    signal.integral = c(1:2),
-    background.integral = c(85:100)
-  ), "Channel numbers of Lx and Tx data differ!")
-
-  expect_error(calc_OSLLxTxRatio(
-    "Lx.data",
-    Tx.data,
-    signal.integral = c(1:2),
-    background.integral = c(85:100)
-  ), "Data type of Lx and Tx data differs!")
-
-  expect_error(calc_OSLLxTxRatio(
-    "Lx.data",
-    "Tx.data",
-    signal.integral = c(1:2),
-    background.integral = c(85:100)
-  ), "Data type error! Required types are data.frame or numeric vector.")
-
+  expect_error(calc_OSLLxTxRatio(Lx.data, "error"),
+               "'Tx.data' should be of class 'RLum.Data.Curve', 'data.frame', 'numeric' or 'matrix'")
+  expect_error(calc_OSLLxTxRatio("error", "error"),
+               "'Lx.data' should be of class 'RLum.Data.Curve', 'data.frame'")
+  expect_error(calc_OSLLxTxRatio(Lx.data[1:10, ], Tx.data),
+               "Different number of channels for Lx (10) and Tx (100)",
+               fixed = TRUE)
 
   expect_error(calc_OSLLxTxRatio(
     Lx.data,
     Tx.data,
     signal.integral = c(1:2000),
     background.integral = c(85:100)
-  ), "signal.integral is not valid!")
+  ), "'signal.integral' is not valid, max: 100")
 
   expect_error(calc_OSLLxTxRatio(
     Lx.data,
     Tx.data,
     signal.integral = c(1:90),
     background.integral = c(85:100)
-  ), "Overlapping of 'signal.integral' and 'background.integral' is not permitted!")
+  ), "'signal.integral' and 'background.integral' overlap")
 
   expect_error(calc_OSLLxTxRatio(
     Lx.data,
@@ -140,14 +111,14 @@ test_that("force function break", {
     signal.integral.Tx = c(1:90),
     background.integral = c(85:100),
     background.integral.Tx = c(85:100)
-  ), "Overlapping of 'signal.integral.Tx' and 'background.integral.Tx' is not permitted!")
+  ), "'signal.integral.Tx' and 'background.integral.Tx' overlap")
 
   expect_error(calc_OSLLxTxRatio(
     Lx.data,
     Tx.data,
     signal.integral = c(1:20),
     background.integral = c(85:1000)
-  ), "background.integral is not valid! Max: 100")
+  ), "'background.integral' is not valid, max: 100")
 
   expect_error(calc_OSLLxTxRatio(
     Lx.data,
@@ -156,7 +127,7 @@ test_that("force function break", {
     signal.integral.Tx = c(1:10),
     background.integral = c(85:100),
     background.integral.Tx = c(85:10000)
-  ), "background.integral.Tx is not valid! Max: 100")
+  ), "'background.integral.Tx' is not valid, max: 100")
 
   expect_error(calc_OSLLxTxRatio(
     Lx.data,
@@ -165,7 +136,7 @@ test_that("force function break", {
     signal.integral.Tx = c(1:1000),
     background.integral = c(85:100),
     background.integral.Tx = c(85:100)
-  ), "signal.integral.Tx is not valid!")
+  ), "'signal.integral.Tx' is not valid, max: 100")
 
   expect_error(calc_OSLLxTxRatio(
     Lx.data,
@@ -174,7 +145,7 @@ test_that("force function break", {
     signal.integral.Tx = c(1:20),
     background.integral = 80:100,
     background.integral.Tx = NULL
-  ), "You have to provide both: signal.integral.Tx and background.integral.Tx!")
+  ), "You have to provide both 'signal.integral.Tx' and 'background.integral.Tx'")
 
   expect_error(calc_OSLLxTxRatio(
     Lx.data,
@@ -182,17 +153,15 @@ test_that("force function break", {
     signal.integral = c(1:20),
     background.integral = 80:100,
     sigmab = "test"
-  ), "'sigmab' has to be of type numeric.")
+  ), "'sigmab' should be of class 'numeric'")
 
   expect_error(calc_OSLLxTxRatio(
     Lx.data,
     Tx.data,
     signal.integral = c(1:20),
     background.integral = 80:100,
-    sigmab = 1:100
-  ), "Maximum allowed vector length for 'sigmab' is 2.")
-
-
+    sigmab = c(1, 2, 3)
+  ), "'sigmab' can have at most length 2")
 })
 
 test_that("create warnings", {
@@ -205,7 +174,7 @@ test_that("create warnings", {
     signal.integral.Tx = c(1:20),
     background.integral = 80:100,
     background.integral.Tx = 60:100
-  ), "Number of background channels for Lx < 25; error estimation might not be reliable!")
+  ), "Number of background channels for Lx < 25, error estimation might not be reliable")
 
   expect_warning(calc_OSLLxTxRatio(
     Lx.data,
@@ -214,7 +183,7 @@ test_that("create warnings", {
     signal.integral.Tx = c(1:20),
     background.integral = 60:100,
     background.integral.Tx = 80:100
-  ), "Number of background channels for Tx < 25; error estimation might not be reliable!",)
+  ), "Number of background channels for Tx < 25, error estimation might not be reliable")
 
   expect_warning(calc_OSLLxTxRatio(
     Lx.data,
@@ -222,7 +191,7 @@ test_that("create warnings", {
     signal.integral = c(1:20),
     background.integral = 60:100,
     background.count.distribution = "hallo"
-  ), "Unknown method for background.count.distribution. A non-poisson distribution is assumed!")
+  ), "Unknown method for 'background.count.distribution', a non-poisson")
 
   expect_warning(calc_OSLLxTxRatio(
     Lx.data,
@@ -232,82 +201,68 @@ test_that("create warnings", {
     background.integral = 60:100,
     background.integral.Tx = 40:100,
     use_previousBG = TRUE
-  ), "For option use_previousBG = TRUE independent Lx and Tx integral limits are not allowed. Integral limits of Lx used for Tx.")
-
+  ), "With 'use_previousBG = TRUE' independent Lx and Tx integral limits are")
 })
 
-
-test_that("check weird circumstances", {
+test_that("snapshot tests", {
   testthat::skip_on_cran()
+  snapshot.tolerance <- 1.5e-6
 
+  expect_snapshot_RLum(calc_OSLLxTxRatio(
+      Lx.data = Lx.data,
+      Tx.data = Tx.data,
+      signal.integral = c(1:2),
+      background.integral = c(85:100)
+  ), tolerance = snapshot.tolerance)
+
+  ## check weird circumstances
   ##(1) - Lx curve 0
-  expect_type(calc_OSLLxTxRatio(
+  expect_snapshot_RLum(calc_OSLLxTxRatio(
     data.frame(Lx.data[,1],0),
     Tx.data,
     signal.integral = c(1:2),
     background.integral = c(85:100)
-  )$LxTx.table, type = "list")
-
+  ))
 
   ##(2) - Tx curve 0
-  expect_type(calc_OSLLxTxRatio(
+  expect_snapshot_RLum(calc_OSLLxTxRatio(
     Lx.data,
     data.frame(Tx.data[,1],0),
     signal.integral = c(1:2),
     background.integral = c(85:100)
-  )$LxTx.table, type = "list")
+  ))
 
   ##(3) - Lx and Tx curve 0
-  expect_type(calc_OSLLxTxRatio(
+  expect_snapshot_RLum(calc_OSLLxTxRatio(
     data.frame(Lx.data[,1],0),
     data.frame(Tx.data[,1],0),
     signal.integral = c(1:2),
     background.integral = c(85:100)
-  )$LxTx.table, type = "list")
+  ))
 
   ##(4) - Lx < 0
-  expect_type(calc_OSLLxTxRatio(
+  expect_snapshot_RLum(calc_OSLLxTxRatio(
     data.frame(Lx.data[,1],-1000),
     data.frame(Tx.data[,1],0),
     signal.integral = c(1:2),
     background.integral = c(85:100)
-  )$LxTx.table, type = "list")
+  ))
 
   ##(5) - Tx < 0
-  expect_type(calc_OSLLxTxRatio(
+  expect_snapshot_RLum(calc_OSLLxTxRatio(
     Lx.data,
     data.frame(Lx.data[,1],-1000),
     signal.integral = c(1:2),
     background.integral = c(85:100)
-  )$LxTx.table, type = "list")
+  ))
 
   ##(6) - Lx & Tx < 0
-  expect_type(calc_OSLLxTxRatio(
+  expect_snapshot_RLum(calc_OSLLxTxRatio(
     data.frame(Lx.data[,1],-1000),
     data.frame(Tx.data[,1],-1000),
     signal.integral = c(1:2),
     background.integral = c(85:100)
-  )$LxTx.table, type = "list")
-
-
-})
-
-test_that("check values from output example", {
-  testthat::skip_on_cran()
-
-  results <- get_RLum(temp)
-
-  expect_equal(results$LnLx, 81709)
-  expect_equal(results$LnLx.BG, 530)
-  expect_equal(results$TnTx, 7403)
-  expect_equal(results$TnTx.BG, 513)
-  expect_equal(results$Net_LnLx, 81179)
-  expect_equal(round(results$Net_LnLx.Error, digits = 4), 286.5461)
-  expect_equal(results$Net_TnTx, 6890)
-  expect_equal(round(results$Net_TnTx.Error, digits = 5), 88.53581)
-  expect_equal(round(results$LxTx, digits = 5), 11.78215)
-  expect_equal(round(results$LxTx.Error, digits = 7), 0.1570077)
-
+  ))
 })
 
 test_that("test NA mode with no signal integrals", {
@@ -321,5 +276,4 @@ test_that("test NA mode with no signal integrals", {
     background.integral = NA), "RLum.Results")
 
   expect_equal(round(sum(temp$LxTx.table[1,]),0), 391926)
-
 })
