@@ -1,3 +1,4 @@
+## load data
 data("ExampleData.SurfaceExposure", envir = environment())
 d1 <- ExampleData.SurfaceExposure$sample_1
 d2 <- ExampleData.SurfaceExposure$sample_2
@@ -19,6 +20,8 @@ test_that("input validation", {
                "'age' must be of the same length")
   expect_error(fit_SurfaceExposure(d4, age = 1e4),
                "'age' must be of the same length")
+  expect_error(fit_SurfaceExposure(d4, age = rep(1e4, 5), mu = c(0.8, 0.9)),
+               "'mu' must either be of the same length or of length 1")
 
   SW({
   expect_message(fit_SurfaceExposure(rbind(d1, NA), mu = 0.9),
@@ -44,6 +47,10 @@ test_that("check values from output example", {
 
   expect_equal(round(fit$summary$age), 9893)
   expect_equal(round(fit$summary$age_error), 369)
+
+  expect_s4_class(fit_SurfaceExposure(d1, sigmaphi = 5e-10, age = 12000,
+                                      verbose = FALSE),
+                  "RLum.Results")
 })
 
 # Sub-test - weighted fitting
@@ -89,6 +96,18 @@ test_that("check values from output example", {
 
   expect_equal(round(unique(fit$summary$mu), 3), 0.901)
   expect_equal(round(unique(fit$summary$mu_error), 3), 0.002)
+
+  ## multiple mus given
+  expect_s4_class(fit_SurfaceExposure(data = d3, age = c(1e3, 1e4, 1e5, 1e6),
+                                      mu = 0.1 * 7:10, verbose = FALSE),
+                  "RLum.Results")
+
+  SW({
+  ## all paramers specified, nothing to fit
+  expect_message(fit_SurfaceExposure(data = d3, age = c(1e3, 1e4, 1e5, 1e6),
+                                     mu = 0.1 * 7:10, sigmaphi = 5e-10),
+                 "Error: Unable to fit the data")
+  })
 })
 
 
@@ -141,3 +160,19 @@ test_that("not enough parameters provided", {
                  fixed = TRUE)
   })
 })
+
+test_that("graphical snapshot tests", {
+  testthat::skip_on_cran()
+  testthat::skip_if_not_installed("vdiffr")
+  testthat::skip_if_not(getRversion() >= "4.4.0")
+
+  SW({
+  vdiffr::expect_doppelganger("single",
+                              fit_SurfaceExposure(d1, mu = 0.9,
+                                                  age = 12000, sigmaphi = 5e-10))
+
+  vdiffr::expect_doppelganger("multiple",
+                              fit_SurfaceExposure(d3, mu = 0.1 * 7:10,
+                                                  age = c(1e3, 1e4, 1e5, 1e6)))
+  })
+})
