@@ -7,6 +7,8 @@ test_that("input validation", {
                "'file' should be of class 'character' or 'list'")
   expect_error(read_BIN2R(character(0)),
                "'file' cannot be an empty character")
+  expect_error(read_BIN2R(letters),
+               "'file' should have length 1")
   expect_error(read_BIN2R(file = "error"),
                "File '.*error' does not exist") # windows CI needs the regexp
   expect_message(expect_null(read_BIN2R(test_path("test_read_BIN2R.R"))),
@@ -78,7 +80,7 @@ test_that("test the import of various BIN-file versions", {
 
   ## this test needs an internet connection ... test for it
   github.url <- file.path("https://github.com/R-Lum/Luminescence",
-                          "raw/dev_0.9.x/tests/testthat/_data")
+                          "raw/archive/dev_0.9.x/tests/testthat/_data")
   if (!httr::http_error(github.url)) {
     ## V3
     expect_snapshot_plain(read_BIN2R(file.path(github.url, "BINfile_V3.bin"),
